@@ -20,19 +20,12 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: classbuilder.c 3446 2009-01-04 18:30:42Z eht16 $
+ * $Id: classbuilder.c 3966 2009-07-14 15:06:20Z ntrel $
  */
 
 /* Class Builder - creates source files containing a new class interface and definition. */
 
-#include "geany.h"
-#include "plugindata.h"
-#include "support.h"
-#include "filetypes.h"
-#include "document.h"
-#include "editor.h"
-#include "ui_utils.h"
-#include "geanyfunctions.h"
+#include "geanyplugin.h"
 
 
 GeanyData		*geany_data;
@@ -168,52 +161,24 @@ struct _{class_name}Private\n\
 	/* add your private declarations here */\n\
 };\n\
 \n\
-static void {class_name_low}_class_init			({class_name}Class *klass);\n\
-static void {class_name_low}_init      			({class_name} *self);\n\
 {destructor_decl}\
 \n\
-/* Local data */\n\
-static {base_name}Class *parent_class = NULL;\n\
-\n\
-GType {class_name_low}_get_type(void)\n\
-{\n\
-	static GType self_type = 0;\n\
-	if (! self_type)\n\
-	{\n\
-		static const GTypeInfo self_info = \n\
-		{\n\
-			sizeof({class_name}Class),\n\
-			NULL, /* base_init */\n\
-			NULL, /* base_finalize */\n\
-			(GClassInitFunc){class_name_low}_class_init,\n\
-			NULL, /* class_finalize */\n\
-			NULL, /* class_data */\n\
-			sizeof({class_name}),\n\
-			0,\n\
-			(GInstanceInitFunc){class_name_low}_init,\n\
-			NULL /* value_table */\n\
-		};\n\
-		\n\
-		self_type = g_type_register_static({base_gtype}, \"{class_name}\", &self_info, 0);\n\
-	}\n\
-	\n\
-	return self_type;\n\
-}\n\
+G_DEFINE_TYPE({class_name}, {class_name_low}, {base_gtype});\n\
 \n\n\
 static void {class_name_low}_class_init({class_name}Class *klass)\n\
 {\n\
 	{gtk_destructor_registration}\n\
-	parent_class = ({base_name}Class*)g_type_class_peek({base_gtype});\n\
 	g_type_class_add_private((gpointer)klass, sizeof({class_name}Private));\n\
 }\n\
-\n\n\
+\n\
+{destructor_impl}\n\
+\n\
 static void {class_name_low}_init({class_name} *self)\n\
 {\n\
 	\n\
 }\n\
 \n\
 {constructor_impl}\n\
-{destructor_impl}\n\
 ";
 
 
@@ -225,7 +190,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg);
 
 /* The list must be ended with NULL as an extra check that arg_count is correct. */
 static void
-utils_free_pointers(gsize arg_count, ...)
+free_pointers(gsize arg_count, ...)
 {
 	va_list a;
 	gsize i;
@@ -626,7 +591,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 	}
 	class_info->header = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->header_entry)));
 	class_info->header_guard = g_ascii_strup(class_info->header, -1);
-	g_strdelimit(class_info->header_guard, ".", '_');
+	g_strdelimit(class_info->header_guard, ".-", '_');
 	switch (class_info->type)
 	{
 		case GEANY_CLASS_TYPE_CPP:
@@ -679,13 +644,12 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 						gtk_entry_get_text(GTK_ENTRY(cc_dlg->gtk_constructor_type_entry)),
 						class_info->class_name_low);
 				class_info->constructor_impl = g_strdup_printf("\n"
-						"%s* %s_new(void)\n"
+						"%s *%s_new(void)\n"
 						"{\n"
-						"\treturn (%s*)g_object_new(%s_TYPE, NULL);\n"
+						"\treturn g_object_new(%s_TYPE, NULL);\n"
 						"}\n",
 						gtk_entry_get_text(GTK_ENTRY(cc_dlg->gtk_constructor_type_entry)),
 						class_info->class_name_low,
-						gtk_entry_get_text(GTK_ENTRY(cc_dlg->gtk_constructor_type_entry)),
 						class_info->class_name_up);
 			}
 			else
@@ -710,13 +674,13 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 						"\tg_return_if_fail(object != NULL);\n"
 						"\tg_return_if_fail(IS_%s(object));\n\n"
 						"\tself = %s(object);\n\n"
-						"\tif (G_OBJECT_CLASS(parent_class)->finalize)\n"
-						"\t\t(* G_OBJECT_CLASS(parent_class)->finalize)(object);\n"
+						"\tG_OBJECT_CLASS(%s_parent_class)->finalize(object);\n"
 						"}\n",
 						class_info->class_name_low,
 						class_info->class_name,
 						class_info->class_name_up,
-						class_info->class_name_up);
+						class_info->class_name_up,
+						class_info->class_name_low);
 			}
 			else
 			{
@@ -745,7 +709,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 		g_free(text);
 	}
 
-	utils_free_pointers(17, tmp, class_info->class_name, class_info->class_name_up,
+	free_pointers(17, tmp, class_info->class_name, class_info->class_name_up,
 		class_info->base_name, class_info->class_name_low, class_info->base_include,
 		class_info->header, class_info->header_guard, class_info->source, class_info->base_decl,
 		class_info->constructor_decl, class_info->constructor_impl,
