@@ -19,28 +19,18 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: filebrowser.c 3733 2009-04-24 19:31:23Z eht16 $
+ * $Id: filebrowser.c 4041 2009-07-29 17:40:20Z ntrel $
  */
 
 /* Sidebar file browser plugin. */
 
-#include "geany.h"
+#include "geanyplugin.h"
 #include <string.h>
 
 #include <gdk/gdkkeysyms.h>
 
-#include "support.h"
-#include "prefs.h"
-#include "document.h"
-#include "utils.h"
-#include "keybindings.h"
-#include "project.h"
-#include "ui_utils.h"
-
-#include "plugindata.h"
-#include "geanyfunctions.h"
-
 
+GeanyPlugin		*geany_plugin;
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
@@ -98,12 +88,10 @@ static struct
 } popup_items;
 
 
-static void document_activate_cb(GObject *obj, GeanyDocument *doc, gpointer data);
 static void project_change_cb(GObject *obj, GKeyFile *config, gpointer data);
 
 PluginCallback plugin_callbacks[] =
 {
-	{ "document-activate", (GCallback) &document_activate_cb, TRUE, NULL },
 	{ "project-open", (GCallback) &project_change_cb, TRUE, NULL },
 	{ "project-save", (GCallback) &project_change_cb, TRUE, NULL },
 	{ NULL, NULL, FALSE, NULL }
@@ -168,13 +156,11 @@ static void add_item(const gchar *name)
 	if (! show_hidden_files && check_hidden(name))
 		return;
 
-	if (check_filtered(name))
-		return;
-
 	sep = (utils_str_equal(current_dir, "/")) ? "" : G_DIR_SEPARATOR_S;
 	fname = g_strconcat(current_dir, sep, name, NULL);
 	dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
 	utf8_fullname = utils_get_locale_from_utf8(fname);
+	utf8_name = utils_get_utf8_from_locale(name);
 	g_free(fname);
 
 	if (dir)
@@ -189,9 +175,16 @@ static void add_item(const gchar *name)
 		last_dir_iter = gtk_tree_iter_copy(&iter);
 	}
 	else
+	{
+		if (check_filtered(utf8_name))
+		{
+			g_free(utf8_name);
+			g_free(utf8_fullname);
+			return;
+		}
 		gtk_list_store_append(file_store, &iter);
+	}
 
-	utf8_name = utils_get_utf8_from_locale(name);
 
 	gtk_list_store_set(file_store, &iter,
 		FILEVIEW_COLUMN_ICON, (dir) ? GTK_STOCK_DIRECTORY : GTK_STOCK_FILE,
@@ -274,6 +267,7 @@ static void on_go_home(void)
 }
 
 
+/* TODO: use utils_get_default_dir_utf8() */
 static gchar *get_default_dir(void)
 {
 	const gchar *dir = NULL;
@@ -1025,6 +1019,9 @@ void plugin_init(GeanyData *data)
 		0, 0, "focus_file_list", _("Focus File List"), NULL);
 	keybindings_set_item(plugin_key_group, KB_FOCUS_PATH_ENTRY, kb_activate,
 		0, 0, "focus_path_entry", _("Focus Path Entry"), NULL);
+
+	plugin_signal_connect(geany_plugin, NULL, "document-activate", TRUE,
+		(GCallback) &document_activate_cb, NULL);
 }
 
 
@@ -1128,7 +1125,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	gtk_box_pack_start(GTK_BOX(vbox), checkbox_fp, FALSE, FALSE, 5);
 	pref_widgets.follow_path_checkbox = checkbox_fp;
 
-	checkbox_pb = gtk_check_button_new_with_label(_("Set the project's base directory"));
+	checkbox_pb = gtk_check_button_new_with_label(_("Use the project's base directory"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_pb), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_pb), fb_set_project_base_path);
 	ui_widget_set_tooltip_text(checkbox_pb,
