@@ -19,19 +19,11 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: saveactions.c 3447 2009-01-04 18:31:22Z eht16 $
+ * $Id: saveactions.c 3966 2009-07-14 15:06:20Z ntrel $
  */
 
 
-#include "geany.h"
-#include "support.h"
-#include "document.h"
-#include "utils.h"
-#include "ui_utils.h"
-#include "filetypes.h"
-
-#include "plugindata.h"
-#include "geanyfunctions.h"
+#include "geanyplugin.h"
 
 #include <unistd.h>
 #include <errno.h>
@@ -42,7 +34,7 @@ GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
 
-PLUGIN_VERSION_CHECK(98)
+PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 
 PLUGIN_SET_INFO(_("Save Actions"), _("This plugin provides different actions related to saving of files."),
 	VERSION, _("The Geany developer team"))
@@ -353,7 +345,7 @@ void plugin_init(GeanyData *data)
 		config, "saveactions", "enable_backupcopy", FALSE);
 
 	instantsave_default_ft = utils_get_setting_string(config, "instantsave", "default_ft",
-		filetypes_index(GEANY_FILETYPES_NONE)->name);
+		filetypes[GEANY_FILETYPES_NONE]->name);
 
 	autosave_src_id = G_MAXUINT; /* mark as invalid */
 	autosave_interval = utils_get_setting_integer(config, "autosave", "interval", 300);
@@ -603,6 +595,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	{
 		GtkWidget *combo;
 		guint i;
+		GSList *node;
 
 		notebook_vbox = gtk_vbox_new(FALSE, 2);
 		inner_vbox = gtk_vbox_new(FALSE, 5);
@@ -624,14 +617,16 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 		gtk_box_pack_start(GTK_BOX(inner_vbox), label, FALSE, FALSE, 0);
 
 		pref_widgets.instantsave_ft_combo = combo = gtk_combo_box_new_text();
-		for (i = 0; i < geany->filetypes_array->len; i++)
+		i = 0;
+		foreach_slist(node, geany->filetypes_by_title)
 		{
-			GeanyFiletype *ft = filetypes_index(i);
+			GeanyFiletype *ft = node->data;
 
 			gtk_combo_box_append_text(GTK_COMBO_BOX(combo), ft->name);
 
 			if (utils_str_equal(ft->name, instantsave_default_ft))
 				gtk_combo_box_set_active(GTK_COMBO_BOX(combo), i);
+			i++;
 		}
 		gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(combo), 3);
 		gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
