@@ -19,33 +19,40 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: splitwindow.c 3460 2009-01-11 18:29:39Z eht16 $
+ * $Id: splitwindow.c 4012 2009-07-23 11:00:32Z ntrel $
  */
 
 /* Split Window plugin. */
 
-#include "geany.h"
-#include "support.h"
+#include "geanyplugin.h"
+#include <string.h>
+
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 #include "SciLexer.h"
 
-#include "ui_utils.h"
-#include "document.h"
-#include "editor.h"
-#include "plugindata.h"
-#include "geanyfunctions.h"
-
 
 PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 PLUGIN_SET_INFO(_("Split Window"), _("Splits the editor view into two windows."),
-	"0.1", _("The Geany developer team"))
+	VERSION, _("The Geany developer team"))
 
 
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
 
+/* Keybinding(s) */
+enum
+{
+	KB_SPLIT_HORIZONTAL,
+	KB_SPLIT_VERTICAL,
+	KB_SPLIT_UNSPLIT,
+	KB_COUNT
+};
+
+PLUGIN_KEY_GROUP(split_window, KB_COUNT);
+
+
 enum State
 {
 	STATE_SPLIT_HORIZONTAL,
@@ -107,14 +114,6 @@ static void set_styles(ScintillaObject *oldsci, ScintillaObject *newsci)
 }
 
 
-static void sci_set_font(ScintillaObject *sci, gint style, const gchar *font,
-	gint size)
-{
-	scintilla_send_message(sci, SCI_STYLESETFONT, style, (sptr_t) font);
-	scintilla_send_message(sci, SCI_STYLESETSIZE, style, size);
-}
-
-
 static void update_font(ScintillaObject *current, ScintillaObject *sci)
 {
 	gint style_id;
@@ -164,7 +163,7 @@ static void sync_to_current(ScintillaObject *sci, ScintillaObject *current)
 
 	/* set the new sci widget to view the existing Scintilla document */
 	sdoc = (gpointer) scintilla_send_message(current, SCI_GETDOCPOINTER, 0, 0);
-	scintilla_send_message(sci, SCI_SETDOCPOINTER, 0, GPOINTER_TO_INT(sdoc));
+	scintilla_send_message(sci, SCI_SETDOCPOINTER, 0, (sptr_t) sdoc);
 
 	update_font(current, sci);
 	lexer = scintilla_send_message(current, SCI_GETLEXER, 0, 0);
@@ -212,14 +211,44 @@ static void set_state(enum State id)
 }
 
 
-static GtkWidget *create_tool_button(const gchar *label, const gchar *stock_id)
+static const gchar *ui_get_stock_label(const gchar *stock_id)
+{
+	GtkStockItem item;
+
+	if (gtk_stock_lookup(stock_id, &item))
+		return item.label;
+
+	g_warning("No stock id '%s'!", stock_id);
+	return "";
+}
+
+
+/* Create a GtkToolButton with stock icon, label and tooltip.
+ * @param label can be NULL to use stock label text. @a label can contain underscores,
+ * which will be removed.
+ * @param tooltip can be NULL to use label text (useful for GTK_TOOLBAR_ICONS). */
+static GtkWidget *ui_tool_button_new(const gchar *stock_id, const gchar *label, const gchar *tooltip)
 {
 	GtkToolItem *item;
+	gchar *dup = NULL;
+
+	if (stock_id && !label)
+	{
+		label = ui_get_stock_label(stock_id);
+	}
+	dup = utils_str_remove_chars(g_strdup(label), "_");
+	label = dup;
 
 	item = gtk_tool_button_new(NULL, label);
-	gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(item), stock_id);
-	ui_widget_set_tooltip_text(GTK_WIDGET(item), label);
+	if (stock_id)
+		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(item), stock_id);
 
+	if (!tooltip)
+		tooltip = label;
+	if (tooltip)
+		ui_widget_set_tooltip_text(GTK_WIDGET(item), tooltip);
+
+	g_free(dup);
 	return GTK_WIDGET(item);
 }
 
@@ -235,18 +264,6 @@ static void on_refresh(void)
 }
 
 
-/* avoid adding new strings which are the same but without a leading underscore */
-static const gchar *after_underscore(const gchar *str)
-{
-	const gchar *u = g_strstr_len(str, -1, "_");
-
-	if (u)
-		return ++u;
-	else
-		return str;
-}
-
-
 static GtkWidget *create_toolbar(void)
 {
 	GtkWidget *toolbar, *item;
@@ -256,7 +273,8 @@ static GtkWidget *create_toolbar(void)
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_MENU);
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 
-	item = (GtkWidget*)gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH);
+	item = ui_tool_button_new(GTK_STOCK_JUMP_TO, "",
+		_("Show the current document"));
 	gtk_container_add(GTK_CONTAINER(toolbar), item);
 	g_signal_connect(item, "clicked", G_CALLBACK(on_refresh), NULL);
 
@@ -269,7 +287,7 @@ static GtkWidget *create_toolbar(void)
 	gtk_container_add(GTK_CONTAINER(tool_item), item);
 	edit_window.name_label = item;
 
-	item = create_tool_button(after_underscore(_("_Unsplit")), GTK_STOCK_CLOSE);
+	item = ui_tool_button_new(GTK_STOCK_CLOSE, _("_Unsplit"), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), item);
 	g_signal_connect(item, "clicked", G_CALLBACK(on_unsplit), NULL);
 
@@ -354,12 +372,32 @@ static void on_unsplit(GtkMenuItem *menuitem, gpointer user_data)
 }
 
 
+static void kb_activate(guint key_id)
+{
+	switch (key_id)
+	{
+		case KB_SPLIT_HORIZONTAL:
+			if (plugin_state == STATE_UNSPLIT)
+				split_view(TRUE);
+			break;
+		case KB_SPLIT_VERTICAL:
+			if (plugin_state == STATE_UNSPLIT)
+				split_view(FALSE);
+			break;
+		case KB_SPLIT_UNSPLIT:
+			if (plugin_state != STATE_UNSPLIT)
+				on_unsplit(NULL, NULL);
+			break;
+	}
+}
+
+
 void plugin_init(GeanyData *data)
 {
 	GtkWidget *item, *menu;
 
 	menu_items.main = item = gtk_menu_item_new_with_mnemonic(_("_Split Window"));
-	gtk_menu_append(geany_data->main_widgets->tools_menu, item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(geany_data->main_widgets->tools_menu), item);
 	ui_add_document_sensitive(item);
 
 	menu = gtk_menu_new();
@@ -368,21 +406,29 @@ void plugin_init(GeanyData *data)
 	menu_items.horizontal = item =
 		gtk_menu_item_new_with_mnemonic(_("_Horizontally"));
 	g_signal_connect(item, "activate", G_CALLBACK(on_split_horizontally), NULL);
-	gtk_menu_append(menu, item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
 	menu_items.vertical = item =
 		gtk_menu_item_new_with_mnemonic(_("_Vertically"));
 	g_signal_connect(item, "activate", G_CALLBACK(on_split_vertically), NULL);
-	gtk_menu_append(menu, item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
 	menu_items.unsplit = item =
 		gtk_menu_item_new_with_mnemonic(_("_Unsplit"));
 	g_signal_connect(item, "activate", G_CALLBACK(on_unsplit), NULL);
-	gtk_menu_append(menu, item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
 	gtk_widget_show_all(menu_items.main);
 
 	set_state(STATE_UNSPLIT);
+
+	/* setup keybindings */
+	keybindings_set_item(plugin_key_group, KB_SPLIT_HORIZONTAL, kb_activate,
+		0, 0, "split_horizontal", _("Split Horizontally"), menu_items.horizontal);
+	keybindings_set_item(plugin_key_group, KB_SPLIT_VERTICAL, kb_activate,
+		0, 0, "split_vertical", _("Split Vertically"), menu_items.vertical);
+	keybindings_set_item(plugin_key_group, KB_SPLIT_UNSPLIT, kb_activate,
+		0, 0, "split_unsplit", _("Unsplit"), menu_items.unsplit);
 }
 
 
