@@ -141,32 +141,29 @@ int Scintilla_LinkLexers() {
 
 //++Autogenerated -- run src/LexGen.py to regenerate
 //**\(\tLINK_LEXER(\*);\n\)
+	LINK_LEXER(lmAda);
 	LINK_LEXER(lmAsm);
-	LINK_LEXER(lmASP);
 	LINK_LEXER(lmBash);
-	LINK_LEXER(lmFreeBasic);
-	LINK_LEXER(lmBatch);
 	LINK_LEXER(lmCaml);
+	LINK_LEXER(lmCmake);
 	LINK_LEXER(lmCPP);
-	LINK_LEXER(lmCPPNoCase);
 	LINK_LEXER(lmCss);
 	LINK_LEXER(lmD);
 	LINK_LEXER(lmDiff);
-	LINK_LEXER(lmErrorList);
 	LINK_LEXER(lmF77);
 	LINK_LEXER(lmFortran);
+	LINK_LEXER(lmFreeBasic);
 	LINK_LEXER(lmHaskell);
 	LINK_LEXER(lmHTML);
 	LINK_LEXER(lmLatex);
 	LINK_LEXER(lmLua);
 	LINK_LEXER(lmMake);
-	LINK_LEXER(lmNncrontab);
+	LINK_LEXER(lmMarkdown);
+	LINK_LEXER(lmMatlab);
+	LINK_LEXER(lmNsis);
 	LINK_LEXER(lmNull);
-	LINK_LEXER(lmOMS);
 	LINK_LEXER(lmPascal);
 	LINK_LEXER(lmPerl);
-	LINK_LEXER(lmPHP);
-	LINK_LEXER(lmPHPSCRIPT);
 	LINK_LEXER(lmPo);
 	LINK_LEXER(lmProps);
 	LINK_LEXER(lmPython);
@@ -176,6 +173,7 @@ int Scintilla_LinkLexers() {
 	LINK_LEXER(lmTCL);
 	LINK_LEXER(lmVHDL);
 	LINK_LEXER(lmXML);
+	LINK_LEXER(lmYAML);
 
 //--Autogenerated -- end of automatically generated section
 
