@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 3570 2009-02-10 21:10:50Z eht16 $
+ * $Id: build.c 4050 2009-08-02 13:33:39Z eht16 $
  */
 
 /*
@@ -60,7 +60,7 @@
 #include "geanymenubuttonaction.h"
 
 
-GeanyBuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_NONE, NULL};
+GeanyBuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_NONE, NULL, 0};
 
 static gchar *current_dir_entered = NULL;
 
@@ -94,10 +94,10 @@ static BuildMenuItems latex_menu_items =
 
 static struct
 {
-	GtkWidget	*run_button;
-	GtkWidget	*compile_button;
-	GtkWidget	*build_button;
+	GtkAction	*run_action;
+	GtkAction	*compile_action;
 	GtkAction	*build_action;
+	GtkWidget	*toolmenu;
 
 	GtkWidget	*toolitem_build;
 	GtkWidget	*toolitem_make_all;
@@ -108,11 +108,13 @@ static struct
 widgets;
 
 
+#ifndef G_OS_WIN32
+static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data);
+#endif
 static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, gboolean autoclose);
 static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *dir);
 static void set_stop_button(gboolean stop);
-static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void on_build_arguments_activate(GtkMenuItem *menuitem, gpointer user_data);
 
@@ -124,9 +126,11 @@ static void on_build_execute_activate(GtkMenuItem *menuitem, gpointer user_data)
 static void on_build_next_error(GtkMenuItem *menuitem, gpointer user_data);
 static void on_build_previous_error(GtkMenuItem *menuitem, gpointer user_data);
 static void kill_process(GPid *pid);
+static void show_build_result_message(gboolean failure);
+static void process_build_output_line(const gchar *line, gint color);
 
 
-void build_finalize()
+void build_finalize(void)
 {
 	g_free(build_info.dir);
 	g_free(build_info.custom_target);
@@ -135,6 +139,8 @@ void build_finalize()
 		gtk_widget_destroy(default_menu_items.menu);
 	if (latex_menu_items.menu != NULL && GTK_IS_WIDGET(latex_menu_items.menu))
 		gtk_widget_destroy(latex_menu_items.menu);
+
+	g_object_unref(widgets.toolmenu);
 }
 
 
@@ -173,7 +179,6 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 	gchar  *working_dir;
 	gint	term_argv_len, i;
 	GError *error = NULL;
-	struct stat st;
 
 	if (doc == NULL || doc->file_name == NULL)
 		return (GPid) 1;
@@ -183,24 +188,15 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 	executable = utils_remove_ext_from_filename(doc->file_name);
 	view_file = g_strconcat(executable, (mode == LATEX_CMD_VIEW_DVI) ? ".dvi" : ".pdf", NULL);
 
-	/* try convert in locale for stat() */
+	/* try convert in locale */
 	locale_filename = utils_get_locale_from_utf8(view_file);
 
-	/* check whether view_file exists */
-	if (g_stat(locale_filename, &st) != 0)
-	{
-		ui_set_statusbar(TRUE, _("Failed to view %s (make sure it is already compiled)"), view_file);
-		utils_free_pointers(3, executable, view_file, locale_filename, NULL);
-
-		return (GPid) 1;
-	}
-
 	/* replace %f and %e in the run_cmd string */
 	cmd_string = g_strdup((mode == LATEX_CMD_VIEW_DVI) ?
 										g_strdup(doc->file_type->programs->run_cmd) :
 										g_strdup(doc->file_type->programs->run_cmd2));
-	cmd_string = utils_str_replace(cmd_string, "%f", view_file);
-	cmd_string = utils_str_replace(cmd_string, "%e", executable);
+	utils_str_replace_all(&cmd_string, "%f", view_file);
+	utils_str_replace_all(&cmd_string, "%e", executable);
 
 	/* try convert in locale */
 	locale_cmd_string = utils_get_locale_from_utf8(cmd_string);
@@ -256,11 +252,11 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 		if (strstr(argv[0], "cmd.exe") != NULL)
 		{
 			argv[term_argv_len   ]  = g_strdup("/Q /C");
-			argv[term_argv_len + 1] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
+			argv[term_argv_len + 1] = g_strdup(RUN_SCRIPT_CMD);
 		}
 		else
 		{
-			argv[term_argv_len    ] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
+			argv[term_argv_len    ] = g_strdup(RUN_SCRIPT_CMD);
 			argv[term_argv_len + 1] = NULL;
 		}
 #else
@@ -413,24 +409,42 @@ static void clear_errors(GeanyDocument *doc)
 
 
 #ifdef G_OS_WIN32
-/* cmd is a command line separated with spaces, first element will be escaped with double quotes
- * and a newly allocated string will be returned */
-static gchar *quote_executable(const gchar *cmd)
+static void parse_build_output(const gchar **output, gint status)
 {
-	gchar **fields;
-	gchar *result;
+	guint x, i, len;
+	gchar *line, **lines;
 
-	if (! NZV(cmd))
-		return NULL;
+	for (x = 0; x < 2; x++)
+	{
+		if (NZV(output[x]))
+		{
+			lines = g_strsplit_set(output[x], "\r\n", -1);
+			len = g_strv_length(lines);
 
-	fields = g_strsplit(cmd, " ", 2);
-	if (fields == NULL || g_strv_length(fields) != 2)
-		return g_strdup(cmd);
+			for (i = 0; i < len; i++)
+			{
+				if (NZV(lines[i]))
+				{
+					line = lines[i];
+					while (*line != '\0')
+					{	/* replace any conrol characters in the output */
+						if (*line < 32)
+							*line = 32;
+						line++;
+					}
+					process_build_output_line(lines[i], COLOR_BLACK);
+				}
+			}
+			g_strfreev(lines);
+		}
+	}
 
-	result = g_strconcat("\"", fields[0], "\" ", fields[1], NULL);
+	show_build_result_message(status != 0);
+	utils_beep();
 
-	g_strfreev(fields);
-	return result;
+	build_info.pid = 0;
+	/* enable build items again */
+	build_menu_update(NULL);
 }
 #endif
 
@@ -448,8 +462,13 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	gchar	*locale_filename;
 	gchar	*executable;
 	gchar	*tmp;
+#ifdef G_OS_WIN32
+	gchar	*output[2];
+	gint	 status;
+#else
 	gint     stdout_fd;
 	gint     stderr_fd;
+#endif
 
 	g_return_val_if_fail(doc != NULL, (GPid) 1);
 
@@ -462,21 +481,15 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	cmd_string = g_strdup(cmd);
 	/* replace %f and %e in the command string */
 	tmp = g_path_get_basename(locale_filename);
-	cmd_string = utils_str_replace(cmd_string, "%f", tmp);
+	utils_str_replace_all(&cmd_string, "%f", tmp);
 	g_free(tmp);
 	tmp = g_path_get_basename(executable);
-	cmd_string = utils_str_replace(cmd_string, "%e", tmp);
+	utils_str_replace_all(&cmd_string, "%e", tmp);
 	g_free(tmp);
 	g_free(executable);
 
 #ifdef G_OS_WIN32
-	/* due to g_shell_parse_argv() we need to enclose the command(first element) of cmd_string with
-	 * "" if the command contains a full path(i.e. backslashes) otherwise the backslashes will be
-	 * eaten by g_shell_parse_argv(). */
-	setptr(cmd_string, quote_executable(cmd_string));
-	if (! g_shell_parse_argv(cmd_string, NULL, &argv, NULL))
-		/* if automatic parsing failed, fall back to simple, unsafe argv creation */
-		argv = g_strsplit(cmd_string, " ", 0);
+	argv = g_strsplit(cmd_string, " ", 0);
 #else
 	argv = g_new0(gchar *, 4);
 	argv[0] = g_strdup("/bin/sh");
@@ -500,9 +513,16 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	g_free(build_info.dir);
 	build_info.dir = g_strdup(working_dir);
 	build_info.file_type_id = FILETYPE_ID(doc->file_type);
+	build_info.message_count = 0;
 
-	if (! g_spawn_async_with_pipes(working_dir, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-						NULL, NULL, &(build_info.pid), NULL, &stdout_fd, &stderr_fd, &error))
+#ifdef G_OS_WIN32
+	if (! utils_spawn_sync(working_dir, argv, NULL, G_SPAWN_SEARCH_PATH,
+			NULL, NULL, &output[0], &output[1], &status, &error))
+#else
+	if (! g_spawn_async_with_pipes(working_dir, argv, NULL,
+			G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+			&(build_info.pid), NULL, &stdout_fd, &stderr_fd, &error))
+#endif
 	{
 		geany_debug("g_spawn_async_with_pipes() failed: %s", error->message);
 		ui_set_statusbar(TRUE, _("Process failed (%s)"), error->message);
@@ -514,6 +534,11 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 		return (GPid) 0;
 	}
 
+#ifdef G_OS_WIN32
+	parse_build_output((const gchar**) output, status);
+	g_free(output[0]);
+	g_free(output[1]);
+#else
 	if (build_info.pid > 0)
 	{
 		g_child_watch_add(build_info.pid, (GChildWatchFunc) build_exit_cb, NULL);
@@ -526,6 +551,7 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 		TRUE, build_iofunc, GINT_TO_POINTER(0));
 	utils_set_up_io_channel(stderr_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
 		TRUE, build_iofunc, GINT_TO_POINTER(1));
+#endif
 
 	g_strfreev(argv);
 	g_free(working_dir);
@@ -592,9 +618,9 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 	/* replace %f and %e in the run_cmd string */
 	cmd = g_strdup(cmd);
 	tmp = g_path_get_basename(locale_filename);
-	cmd = utils_str_replace(cmd, "%f", tmp);
+	utils_str_replace_all(&cmd, "%f", tmp);
 	g_free(tmp);
-	cmd = utils_str_replace(cmd, "%e", executable);
+	utils_str_replace_all(&cmd, "%e", executable);
 
 #ifdef HAVE_VTE
 	if (vte_info.load_vte && vc != NULL && vc->run_in_vte)
@@ -772,47 +798,62 @@ static GPid build_run_cmd(GeanyDocument *doc)
 }
 
 
+static void process_build_output_line(const gchar *str, gint color)
+{
+	gchar *msg, *tmp;
+
+	msg = g_strdup(str);
+
+	g_strchomp(msg);
+
+	if (! NZV(msg))
+		return;
+
+	if (editor_prefs.use_indicators && build_info.message_count < GEANY_BUILD_ERR_HIGHLIGHT_MAX)
+	{
+		gchar *filename;
+		gint line;
+
+		build_info.message_count++;
+
+		if (build_parse_make_dir(msg, &tmp))
+		{
+			setptr(current_dir_entered, tmp);
+		}
+		msgwin_parse_compiler_error_line(msg, current_dir_entered, &filename, &line);
+		if (line != -1 && filename != NULL)
+		{
+			GeanyDocument *doc = document_find_by_filename(filename);
+
+			if (doc)
+			{
+				if (line > 0) /* some compilers, like pdflatex report errors on line 0 */
+					line--;   /* so only adjust the line number if it is greater than 0 */
+				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line);
+			}
+			color = COLOR_RED;	/* error message parsed on the line */
+		}
+		g_free(filename);
+	}
+	msgwin_compiler_add_string(color, msg);
+
+	g_free(msg);
+}
+
+
+#ifndef G_OS_WIN32
 static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 {
 	if (cond & (G_IO_IN | G_IO_PRI))
 	{
-		/*GIOStatus s;*/
 		gchar *msg;
 
 		while (g_io_channel_read_line(ioc, &msg, NULL, NULL, NULL) && msg)
 		{
-			/*if (s != G_IO_STATUS_NORMAL && s != G_IO_STATUS_EOF) break;*/
-			gint color;
-			gchar *tmp;
+			gint color = (GPOINTER_TO_INT(data)) ? COLOR_DARK_RED : COLOR_BLACK;
 
-			color = (GPOINTER_TO_INT(data)) ? COLOR_DARK_RED : COLOR_BLACK;
-			g_strchomp(msg);
-
-			if (build_parse_make_dir(msg, &tmp))
-			{
-				setptr(current_dir_entered, tmp);
-			}
-
-			if (editor_prefs.use_indicators)
-			{
-				gchar *filename;
-				gint line;
-
-				msgwin_parse_compiler_error_line(msg, current_dir_entered,
-					&filename, &line);
-				if (line != -1 && filename != NULL)
-				{
-					GeanyDocument *doc = document_find_by_filename(filename);
-
-					if (doc)
-						editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
-					color = COLOR_RED;	/* error message parsed on the line */
-				}
-				g_free(filename);
-			}
-			msgwin_compiler_add_string(color, msg);
-
-			g_free(msg);
+			process_build_output_line(msg, color);
+ 			g_free(msg);
 		}
 	}
 	if (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
@@ -820,6 +861,7 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 
 	return TRUE;
 }
+#endif
 
 
 gboolean build_parse_make_dir(const gchar *string, gchar **prefix)
@@ -853,6 +895,12 @@ gboolean build_parse_make_dir(const gchar *string, gchar **prefix)
 		return TRUE;
 	}
 
+	if (strstr(string, "Leaving directory") != NULL)
+	{
+		*prefix = NULL;
+		return TRUE;
+	}
+
 	return FALSE;
 }
 
@@ -886,9 +934,9 @@ static void show_build_result_message(gboolean failure)
 }
 
 
+#ifndef G_OS_WIN32
 static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 {
-#ifdef G_OS_UNIX
 	gboolean failure = FALSE;
 
 	if (WIFEXITED(status))
@@ -906,9 +954,6 @@ static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 		failure = TRUE;
 	}
 	show_build_result_message(failure);
-#else
-	show_build_result_message(! win32_get_exit_status(child_pid));
-#endif
 
 	utils_beep();
 	g_spawn_close_pid(child_pid);
@@ -918,6 +963,7 @@ static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 	build_menu_update(NULL);
 	ui_progress_bar_stop();
 }
+#endif
 
 
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data)
@@ -936,15 +982,22 @@ static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, g
 {
 	FILE *fp;
 	gchar *str;
+#ifdef G_OS_WIN32
+	gchar *expanded_cmd;
+#endif
 
 	fp = g_fopen(fname, "w");
-	if (! fp) return FALSE;
+	if (! fp)
+		return FALSE;
 #ifdef G_OS_WIN32
-	str = g_strdup_printf("%s\n\n%s\ndel %s\n", cmd, (autoclose) ? "" : "pause", fname);
+	/* Expand environment variables like %blah%. */
+	expanded_cmd = win32_expand_environment_variables(cmd);
+	str = g_strdup_printf("%s\n\n%s\ndel \"%%0\"\n\npause\n", expanded_cmd, (autoclose) ? "" : "pause");
+	g_free(expanded_cmd);
 #else
 	str = g_strdup_printf(
-		"#!/bin/sh\n\n%s\n\necho \"\n\n------------------\n(program exited with code: $?)\" \
-		\n\n%s\nrm $0\n", cmd, (autoclose) ? "" :
+		"#!/bin/sh\n\nrm $0\n\n%s\n\necho \"\n\n------------------\n(program exited with code: $?)\" \
+		\n\n%s\n", cmd, (autoclose) ? "" :
 		"\necho \"Press return to continue\"\n#to be more compatible with shells like dash\ndummy_var=\"\"\nread dummy_var");
 #endif
 
@@ -1365,8 +1418,12 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
                                         gint response,
                                         gpointer user_data)
 {
-	GeanyFiletype *ft = user_data;
+	GeanyDocument *doc = user_data;
+	GeanyFiletype *ft;
 
+	g_return_if_fail(doc != NULL);
+
+	ft = doc->file_type;
 	g_return_if_fail(ft != NULL);
 
 	if (response == GTK_RESPONSE_ACCEPT)
@@ -1407,6 +1464,8 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
 				programs->modified = TRUE;
 			}
 		}
+		if (programs->modified)
+			build_menu_update(doc);
 	}
 }
 
@@ -1530,7 +1589,7 @@ static void show_includes_arguments_gen(void)
 	/* run modally to prevent user changing idx filetype */
 	response = gtk_dialog_run(GTK_DIALOG(dialog));
 	/* call the callback manually */
-	on_includes_arguments_dialog_response(GTK_DIALOG(dialog), response, ft);
+	on_includes_arguments_dialog_response(GTK_DIALOG(dialog), response, doc);
 
 	gtk_widget_destroy(dialog);
 }
@@ -1572,14 +1631,13 @@ void build_menu_update(GeanyDocument *doc)
 		menubar_build_menu = ui_lookup_widget(main_widgets.window, "menu_build1");
 	if (doc == NULL)
 		doc = document_get_current();
-	if (doc == NULL ||
-		(FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE &&	doc->file_name == NULL))
+	if (doc == NULL || (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE && doc->file_name == NULL))
 	{
 		gtk_widget_set_sensitive(menubar_build_menu, FALSE);
 		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menubar_build_menu));
-		ui_widget_set_sensitive(widgets.compile_button, FALSE);
-		ui_widget_set_sensitive(widgets.build_button, FALSE);
-		ui_widget_set_sensitive(widgets.run_button, FALSE);
+		gtk_action_set_sensitive(widgets.compile_action, FALSE);
+		gtk_action_set_sensitive(widgets.build_action, FALSE);
+		gtk_action_set_sensitive(widgets.run_action, FALSE);
 		return;
 	}
 	else
@@ -1594,7 +1652,7 @@ void build_menu_update(GeanyDocument *doc)
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menubar_build_menu),
 		menu_items->menu);
 
-	have_path = (doc->file_name != NULL);
+	have_path = doc->file_name != NULL;
 
 	can_make = have_path && build_info.pid <= (GPid) 1;
 
@@ -1605,9 +1663,11 @@ void build_menu_update(GeanyDocument *doc)
 		can_build = can_make;
 
 	if (menu_items->item_compile)
-		gtk_widget_set_sensitive(menu_items->item_compile, can_build && ft->actions->can_compile);
+		gtk_widget_set_sensitive(menu_items->item_compile,
+			can_build && ft->actions->can_compile && NZV(ft->programs->compiler));
 	if (menu_items->item_link)
-		gtk_widget_set_sensitive(menu_items->item_link, can_build && ft->actions->can_link);
+		gtk_widget_set_sensitive(menu_items->item_link,
+			can_build && ft->actions->can_link && NZV(ft->programs->linker));
 	if (menu_items->item_make_all)
 		gtk_widget_set_sensitive(menu_items->item_make_all, can_make);
 	if (menu_items->item_make_custom)
@@ -1618,7 +1678,7 @@ void build_menu_update(GeanyDocument *doc)
 	if (app->project && NZV(app->project->run_cmd))
 		can_run = have_path;	/* for now run is disabled for all untitled files */
 	else
-		can_run = have_path && ft->actions->can_exec;
+		can_run = have_path && ft->actions->can_exec && NZV(ft->programs->run_cmd);
 
 	can_stop = run_info.pid > (GPid) 1;
 	can_run &= ! can_stop;
@@ -1651,9 +1711,10 @@ void build_menu_update(GeanyDocument *doc)
 	if (widgets.toolitem_set_args != NULL)
 		gtk_widget_set_sensitive(widgets.toolitem_set_args, can_set_args);
 
-	ui_widget_set_sensitive(widgets.compile_button, can_build && ft->actions->can_compile);
-	ui_widget_set_sensitive(widgets.build_button, can_make);
-	ui_widget_set_sensitive(widgets.run_button, can_run || can_stop);
+	gtk_action_set_sensitive(widgets.compile_action,
+		can_build && ft->actions->can_compile && NZV(ft->programs->compiler));
+	gtk_action_set_sensitive(widgets.build_action, can_make);
+	gtk_action_set_sensitive(widgets.run_action, can_run || can_stop);
 
 	/* show the stop command if a program is running, otherwise show run command */
 	set_stop_button(can_stop);
@@ -1671,22 +1732,25 @@ void build_menu_update(GeanyDocument *doc)
 /* Call build_menu_update() instead of calling this directly. */
 static void set_stop_button(gboolean stop)
 {
+	const gchar *button_stock_id = NULL;
 	GtkStockItem sitem;
-	GtkWidget *menuitem =
-		build_get_menu_items(run_info.file_type_id)->item_exec;
+	GtkToolButton *run_button;
+	GtkWidget *menuitem = build_get_menu_items(run_info.file_type_id)->item_exec;
 
-	if (widgets.run_button == NULL)
+	run_button = GTK_TOOL_BUTTON(toolbar_get_widget_by_name("Run"));
+	if (run_button != NULL)
+		button_stock_id = gtk_tool_button_get_stock_id(run_button);
+
+	if (stop && utils_str_equal(button_stock_id, "gtk-stop"))
+		return;
+	if (! stop && utils_str_equal(button_stock_id, "gtk-execute"))
 		return;
-
-	if (stop && utils_str_equal(
-		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(widgets.run_button)), "gtk-stop")) return;
-	if (! stop && utils_str_equal(
-		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(widgets.run_button)), "gtk-execute")) return;
 
 	/* use the run button also as stop button */
 	if (stop)
 	{
-		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(widgets.run_button), "gtk-stop");
+		if (run_button != NULL)
+			gtk_tool_button_set_stock_id(run_button, "gtk-stop");
 
 		if (menuitem != NULL)
 		{
@@ -1699,7 +1763,8 @@ static void set_stop_button(gboolean stop)
 	}
 	else
 	{
-		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(widgets.run_button), "gtk-execute");
+		if (run_button != NULL)
+			gtk_tool_button_set_stock_id(run_button, "gtk-execute");
 
 		if (menuitem != NULL)
 		{
@@ -2042,66 +2107,69 @@ on_build_previous_error                (GtkMenuItem     *menuitem,
 }
 
 
-void build_init()
+void build_init(void)
 {
 	GtkWidget *item;
 	GtkWidget *toolmenu;
 
+	/* create the toolbar Build item sub menu */
+	toolmenu = gtk_menu_new();
+	g_object_ref(toolmenu);
+
+	/* build the code */
+	item = ui_image_menu_item_new(GEANY_STOCK_BUILD, _("_Build"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_build_activate), NULL);
+	widgets.toolitem_build = item;
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+
+	/* build the code with make all */
+	item = gtk_image_menu_item_new_with_mnemonic(_("_Make All"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+		GINT_TO_POINTER(GBO_MAKE_ALL));
+	widgets.toolitem_make_all = item;
+
+	/* build the code with make custom */
+	item = gtk_image_menu_item_new_with_mnemonic(_("Make Custom _Target"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+		GINT_TO_POINTER(GBO_MAKE_CUSTOM));
+	widgets.toolitem_make_custom = item;
+
+	/* build the code with make object */
+	item = gtk_image_menu_item_new_with_mnemonic(_("Make _Object"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+		GINT_TO_POINTER(GBO_MAKE_OBJECT));
+	widgets.toolitem_make_object = item;
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+
+	/* arguments */
+	item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Includes and Arguments"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(toolmenu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
+	widgets.toolitem_set_args = item;
+
+
+	/* get toolbar action pointers */
 	widgets.build_action = toolbar_get_action_by_name("Build");
-	toolmenu = geany_menu_button_action_get_menu(GEANY_MENU_BUTTON_ACTION(widgets.build_action));
-
-	if (toolmenu != NULL)
-	{
-		/* build the code */
-		item = ui_image_menu_item_new(GEANY_STOCK_BUILD, _("_Build"));
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_build_activate), NULL);
-		widgets.toolitem_build = item;
-
-		item = gtk_separator_menu_item_new();
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-
-		/* build the code with make all */
-		item = gtk_image_menu_item_new_with_mnemonic(_("_Make All"));
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
-			GINT_TO_POINTER(GBO_MAKE_ALL));
-		widgets.toolitem_make_all = item;
-
-		/* build the code with make custom */
-		item = gtk_image_menu_item_new_with_mnemonic(_("Make Custom _Target"));
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
-			GINT_TO_POINTER(GBO_MAKE_CUSTOM));
-		widgets.toolitem_make_custom = item;
-
-		/* build the code with make object */
-		item = gtk_image_menu_item_new_with_mnemonic(_("Make _Object"));
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
-			GINT_TO_POINTER(GBO_MAKE_OBJECT));
-		widgets.toolitem_make_object = item;
-
-		item = gtk_separator_menu_item_new();
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-
-		/* arguments */
-		item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Includes and Arguments"));
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(toolmenu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
-		widgets.toolitem_set_args = item;
-	}
-
-	widgets.compile_button = toolbar_get_widget_by_name("Compile");
-	widgets.run_button = toolbar_get_widget_by_name("Run");
-	widgets.build_button = toolbar_get_widget_by_name("Build");
+	widgets.compile_action = toolbar_get_action_by_name("Compile");
+	widgets.run_action = toolbar_get_action_by_name("Run");
+	widgets.toolmenu = toolmenu;
+	/* set the submenu to the toolbar item */
+	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(widgets.build_action), toolmenu);
 }
 
 
