@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- *  $Id: documentprivate.h 3495 2009-01-21 15:50:48Z eht16 $
+ *  $Id: documentprivate.h 4063 2009-08-09 20:31:03Z eht16 $
  */
 
 
@@ -40,9 +40,7 @@ enum
 typedef enum
 {
 	FILE_OK,
-	FILE_CHANGED,
-	FILE_MISSING,
-	FILE_CREATED_PENDING,
+	FILE_CHANGED, /* also valid for deleted files */
 	FILE_IGNORE
 }
 FileDiskStatus;
@@ -61,13 +59,11 @@ typedef struct GeanyDocumentPrivate
 {
 	/* GtkLabel shown in the notebook header. */
 	GtkWidget		*tab_label;
-	/* GtkLabel shown in the notebook right-click menu. */
-	GtkWidget		*tabmenu_label;
-	/* GtkTreeView object for this %document within the Symbols treeview of the sidebar. */
+	/* GtkTreeView object for this document within the Symbols treeview of the sidebar. */
 	GtkWidget		*tag_tree;
-	/* GtkTreeStore object for this %document within the Symbols treeview of the sidebar. */
+	/* GtkTreeStore object for this document within the Symbols treeview of the sidebar. */
 	GtkTreeStore	*tag_store;
-	/* Iter for this %document within the Open Files treeview of the sidebar. */
+	/* Iter for this document within the Open Files treeview of the sidebar. */
 	GtkTreeIter		 iter;
 	/* Used by the Undo/Redo management code. */
 	GTrashStack		*undo_actions;
@@ -78,10 +74,9 @@ typedef struct GeanyDocumentPrivate
 	gboolean		 colourise_needed;	/* use document.c:queue_colourise() instead */
 	gint			 line_count;		/* Number of lines in the document. */
 	gint			 symbol_list_sort_mode;
-	/* indicates whether a file is on a remote filesystem, works only with GIO/GVFS */
+	/* indicates whether a file is on a remote filesystem, works only with GIO/GVfs */
 	gboolean		 is_remote;
-	/* File status on disk of the document, can be 'FILE_CHANGED', 'FILE_MISSING' (deleted) or
-	 * 'FILE_OK' if there are no known changes */
+	/* File status on disk of the document */
 	FileDiskStatus	 file_disk_status;
 	/* Reference to a GFileMonitor object, only used when GIO file monitoring is used. */
 	gpointer		 monitor;
