@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: editor.h 3567 2009-02-08 19:52:21Z eht16 $
+ * $Id: editor.h 4025 2009-07-24 15:44:14Z ntrel $
  */
 
 
@@ -100,7 +100,7 @@ typedef struct GeanyEditorPrefs
 	gboolean	show_white_space;
 	gboolean	show_indent_guide;
 	gboolean	show_line_endings;
-	gint		long_line_type;
+	gint		long_line_type; /* 0 - line, 1 - background, 2 - none */
 	gint		long_line_column;
 	gchar		*long_line_color;
 	gboolean	show_markers_margin;		/* view menu */
@@ -129,7 +129,10 @@ typedef struct GeanyEditorPrefs
 	gchar		*comment_toggle_mark;
 	guint		autocompletion_max_entries;
 	guint		autoclose_chars;
-} GeanyEditorPrefs;
+	gboolean	autocomplete_doc_words;
+	gboolean	completion_drops_rest_of_word;
+}
+GeanyEditorPrefs;
 
 extern GeanyEditorPrefs editor_prefs;
 
@@ -175,8 +178,6 @@ void snippet_goto_next_cursor(ScintillaObject *sci, gint current_pos);
 
 gboolean editor_complete_snippet(GeanyEditor *editor, gint pos);
 
-void editor_auto_latex(GeanyEditor *editor, gint pos);
-
 void editor_show_macro_list(GeanyEditor *editor);
 
 gboolean editor_show_calltip(GeanyEditor *editor, gint pos);
@@ -187,16 +188,14 @@ void editor_do_comment(GeanyEditor *editor, gint line, gboolean allow_empty_line
 
 gint editor_do_uncomment(GeanyEditor *editor, gint line, gboolean toggle);
 
-void editor_highlight_braces(GeanyEditor *editor, gint cur_pos);
-
-gboolean editor_lexer_is_c_like(gint lexer);
-
 gint editor_lexer_get_type_keyword_idx(gint lexer);
 
 void editor_insert_multiline_comment(GeanyEditor *editor);
 
 void editor_insert_alternative_whitespace(GeanyEditor *editor);
 
+void editor_indent(GeanyEditor *editor, gboolean increase);
+
 void editor_smart_line_indentation(GeanyEditor *editor, gint pos);
 
 void editor_indentation_by_one_space(GeanyEditor *editor, gint pos, gboolean decrease);
