@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 3531 2009-01-30 18:53:23Z eht16 $
+ * $Id: filetypes.c 4048 2009-07-30 12:38:10Z ntrel $
  */
 
 /**
@@ -67,10 +67,39 @@ GeanyFiletypePrivate;
 
 GPtrArray *filetypes_array = NULL;	/* Dynamic array of filetype pointers */
 
-GHashTable *filetypes_hash = NULL;	/* Hash of filetype pointers based on name keys */
+static GHashTable *filetypes_hash = NULL;	/* Hash of filetype pointers based on name keys */
+
+/** List of filetype pointers sorted by name, but with @c filetypes_index(GEANY_FILETYPES_NONE)
+ * first, as this is usually treated specially.
+ * The list does not change (after filetypes have been initialized), so you can use
+ * @code g_slist_nth_data(filetypes_by_title, n) @endcode and expect the same result at different times. */
+GSList *filetypes_by_title = NULL;
 
 
-static void create_radio_menu_item(GtkWidget *menu, const gchar *label, GeanyFiletype *ftype);
+static void create_radio_menu_item(GtkWidget *menu, GeanyFiletype *ftype);
+
+
+enum TitleType
+{
+	TITLE_SOURCE_FILE,
+	TITLE_FILE
+};
+
+/* Save adding many translation strings if the filetype name doesn't need translating */
+static void filetype_make_title(GeanyFiletype *ft, enum TitleType type)
+{
+	const gchar *fmt = NULL;
+
+	switch (type)
+	{
+		default:
+		case TITLE_SOURCE_FILE:	fmt = _("%s source file"); break;
+		case TITLE_FILE:		fmt = _("%s file"); break;
+	}
+	g_assert(!ft->title);
+	g_assert(ft->name);
+	ft->title = g_strdup_printf(fmt, ft->name);
+}
 
 
 /* Note: remember to update HACKING if this function is renamed. */
@@ -78,11 +107,22 @@ static void init_builtin_filetypes(void)
 {
 	GeanyFiletype *ft;
 
-#define C	/* these macros are only to ease navigation */
+#define NONE	/* these macros are only to ease navigation */
+	ft = filetypes[GEANY_FILETYPES_NONE];
+	ft->lang = -2;
+	ft->name = g_strdup(_("None"));
+	ft->title = g_strdup(_("None"));
+	ft->extension = g_strdup("*");
+	ft->pattern = utils_strv_new("*", NULL);
+	ft->comment_open = NULL;
+	ft->comment_close = NULL;
+	ft->group = GEANY_FILETYPE_GROUP_NONE;
+
+#define C
 	ft = filetypes[GEANY_FILETYPES_C];
 	ft->lang = 0;
 	ft->name = g_strdup("C");
-	ft->title = g_strdup_printf(_("%s source file"), "C");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("c");
 	ft->pattern = utils_strv_new("*.c", "*.h", NULL);
 	ft->comment_open = g_strdup("/*");
@@ -93,7 +133,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_CPP];
 	ft->lang = 1;
 	ft->name = g_strdup("C++");
-	ft->title = g_strdup_printf(_("%s source file"), "C++");
+	filetype_make_title(ft, TITLE_FILE);
 	ft->extension = g_strdup("cpp");
 	ft->pattern = utils_strv_new("*.cpp", "*.cxx", "*.c++", "*.cc",
 		"*.h", "*.hpp", "*.hxx", "*.h++", "*.hh", "*.C", NULL);
@@ -105,7 +145,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_CS];
 	ft->lang = 25;
 	ft->name = g_strdup("C#");
-	ft->title = g_strdup_printf(_("%s source file"), "C#");
+	filetype_make_title(ft, TITLE_FILE);
 	ft->extension = g_strdup("cs");
 	ft->pattern = utils_strv_new("*.cs", NULL);
 	ft->comment_open = g_strdup("//");
@@ -116,7 +156,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_VALA];
 	ft->lang = 33;
 	ft->name = g_strdup("Vala");
-	ft->title = g_strdup_printf(_("%s source file"), "Vala");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("vala");
 	ft->pattern = utils_strv_new("*.vala", "*.vapi", NULL);
 	ft->comment_open = g_strdup("//");
@@ -127,7 +167,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_D];
 	ft->lang = 17;
 	ft->name = g_strdup("D");
-	ft->title = g_strdup_printf(_("%s source file"), "D");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("d");
 	ft->pattern = utils_strv_new("*.d", "*.di", NULL);
 	ft->comment_open = g_strdup("//");
@@ -138,7 +178,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_JAVA];
 	ft->lang = 2;
 	ft->name = g_strdup("Java");
-	ft->title = g_strdup_printf(_("%s source file"), "Java");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("java");
 	ft->pattern = utils_strv_new("*.java", "*.jsp", NULL);
 	ft->comment_open = g_strdup("/*");
@@ -149,7 +189,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_PASCAL];
 	ft->lang = 4;
 	ft->name = g_strdup("Pascal");
-	ft->title = g_strdup_printf(_("%s source file"), "Pascal");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("pas");
 	ft->pattern = utils_strv_new("*.pas", "*.pp", "*.inc", "*.dpr",
 		"*.dpk", NULL);
@@ -172,7 +212,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_BASIC];
 	ft->lang = 26;
 	ft->name = g_strdup("FreeBasic");
-	ft->title = g_strdup_printf(_("%s source file"), "FreeBasic");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("bas");
 	ft->pattern = utils_strv_new("*.bas", "*.bi", NULL);
 	ft->comment_open = g_strdup("'");
@@ -205,7 +245,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_GLSL];
 	ft->lang = 31;
 	ft->name = g_strdup("GLSL");
-	ft->title = g_strdup_printf(_("%s source file"), "GLSL");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("glsl");
 	ft->pattern = utils_strv_new("*.glsl", "*.frag", "*.vert", NULL);
 	ft->comment_open = g_strdup("/*");
@@ -227,7 +267,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_PERL];
 	ft->lang = 5;
 	ft->name = g_strdup("Perl");
-	ft->title = g_strdup_printf(_("%s source file"), "Perl");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("pl");
 	ft->pattern = utils_strv_new("*.pl", "*.perl", "*.pm", "*.agi",
 		"*.pod", NULL);
@@ -239,7 +279,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_PHP];
 	ft->lang = 6;
 	ft->name = g_strdup("PHP");
-	ft->title = g_strdup_printf(_("%s source file"), "PHP");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("php");
 	ft->pattern = utils_strv_new("*.php", "*.php3", "*.php4", "*.php5",
 		"*.phtml", NULL);
@@ -251,7 +291,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_JS];
 	ft->lang = 23;
 	ft->name = g_strdup("Javascript");
-	ft->title = g_strdup_printf(_("%s source file"), "Javascript");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("js");
 	ft->pattern = utils_strv_new("*.js", NULL);
 	ft->comment_open = g_strdup("//");
@@ -262,7 +302,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_PYTHON];
 	ft->lang = 7;
 	ft->name = g_strdup("Python");
-	ft->title = g_strdup_printf(_("%s source file"), "Python");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("py");
 	ft->pattern = utils_strv_new("*.py", "*.pyw", NULL);
 	ft->comment_open = g_strdup("#");
@@ -273,7 +313,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_RUBY];
 	ft->lang = 14;
 	ft->name = g_strdup("Ruby");
-	ft->title = g_strdup_printf(_("%s source file"), "Ruby");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("rb");
 	ft->pattern = utils_strv_new("*.rb", "*.rhtml", "*.ruby", NULL);
 	ft->comment_open = g_strdup("#");
@@ -284,7 +324,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_TCL];
 	ft->lang = 15;
 	ft->name = g_strdup("Tcl");
-	ft->title = g_strdup_printf(_("%s source file"), "Tcl");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("tcl");
 	ft->pattern = utils_strv_new("*.tcl", "*.tk", "*.wish", NULL);
 	ft->comment_open = g_strdup("#");
@@ -295,7 +335,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_LUA];
 	ft->lang = 22;
 	ft->name = g_strdup("Lua");
-	ft->title = g_strdup_printf(_("%s source file"), "Lua");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("lua");
 	ft->pattern = utils_strv_new("*.lua", NULL);
 	ft->comment_open = g_strdup("--");
@@ -306,7 +346,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_FERITE];
 	ft->lang = 19;
 	ft->name = g_strdup("Ferite");
-	ft->title = g_strdup_printf(_("%s source file"), "Ferite");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("fe");
 	ft->pattern = utils_strv_new("*.fe", NULL);
 	ft->comment_open = g_strdup("/*");
@@ -317,13 +357,24 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_HASKELL];
 	ft->lang = 24;
 	ft->name = g_strdup("Haskell");
-	ft->title = g_strdup_printf(_("%s source file"), "Haskell");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("hs");
 	ft->pattern = utils_strv_new("*.hs", "*.lhs", NULL);
 	ft->comment_open = g_strdup("--");
 	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
+#define MARKDOWN
+	ft = filetypes[GEANY_FILETYPES_MARKDOWN];
+	ft->lang = 36;
+	ft->name = g_strdup("Markdown");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->extension = g_strdup("md");
+	ft->pattern = utils_strv_new("*.mdml", "*.mdwn", "*.markdown", "*.md", NULL);
+	ft->comment_open = NULL;
+	ft->comment_close = NULL;
+	ft->group = GEANY_FILETYPE_GROUP_MISC;
+
 #define SH
 	ft = filetypes[GEANY_FILETYPES_SH];
 	ft->lang = 16;
@@ -364,7 +415,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_DOCBOOK];
 	ft->lang = 12;
 	ft->name = g_strdup("Docbook");
-	ft->title = g_strdup_printf(_("%s source file"), "Docbook");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("docbook");
 	ft->pattern = utils_strv_new("*.docbook", NULL);
 	ft->comment_open = g_strdup("<!--");
@@ -375,7 +426,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_HTML];
 	ft->lang = 29;
 	ft->name = g_strdup("HTML");
-	ft->title = g_strdup_printf(_("%s source file"), "HTML");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("html");
 	ft->pattern = utils_strv_new(
 		"*.htm", "*.html", "*.shtml", "*.hta", "*.htd", "*.htt", "*.cfm", NULL);
@@ -409,7 +460,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_LATEX];
 	ft->lang = 8;
 	ft->name = g_strdup("LaTeX");
-	ft->title = g_strdup_printf(_("%s source file"), "LaTeX");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("tex");
 	ft->pattern = utils_strv_new("*.tex", "*.sty", "*.idx", "*.ltx", NULL);
 	ft->comment_open = g_strdup("%");
@@ -420,7 +471,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_VHDL];
 	ft->lang = 21;
 	ft->name = g_strdup("VHDL");
-	ft->title = g_strdup_printf(_("%s source file"), "VHDL");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("vhd");
 	ft->pattern = utils_strv_new("*.vhd", "*.vhdl", NULL);
 	ft->comment_open = g_strdup("--");
@@ -431,7 +482,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_DIFF];
 	ft->lang = 20;
 	ft->name = g_strdup("Diff");
-	ft->title = g_strdup(_("Diff file"));
+	filetype_make_title(ft, TITLE_FILE);
 	ft->extension = g_strdup("diff");
 	ft->pattern = utils_strv_new("*.diff", "*.patch", "*.rej", NULL);
 	ft->comment_open = g_strdup("#");
@@ -465,13 +516,24 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_HAXE];
 	ft->lang = 27;
 	ft->name = g_strdup("Haxe");
-	ft->title = g_strdup_printf(_("%s source file"), "Haxe");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("hx");
 	ft->pattern = utils_strv_new("*.hx", NULL);
 	ft->comment_open = g_strdup("//");
 	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
+#define ACTIONSCRIPT
+	ft = filetypes[GEANY_FILETYPES_AS];
+	ft->lang = 34;
+	ft->name = g_strdup("ActionScript");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->extension = g_strdup("as");
+	ft->pattern = utils_strv_new("*.as", NULL);
+	ft->comment_open = g_strdup("//");
+	ft->comment_close = NULL;
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
+
 #define R
 	ft = filetypes[GEANY_FILETYPES_R];
 	ft->lang = 34;
@@ -487,7 +549,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_REST];
 	ft->lang = 28;
 	ft->name = g_strdup("reStructuredText");
-	ft->title = g_strdup(_("reStructuredText file"));
+	filetype_make_title(ft, TITLE_FILE);
 	ft->extension = g_strdup("rst");
 	ft->pattern = utils_strv_new(
 		"*.rest", "*.reST", "*.rst", NULL);
@@ -499,7 +561,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_MATLAB];
 	ft->lang = 32;
 	ft->name = g_strdup("Matlab");
-	ft->title = g_strdup_printf(_("%s source file"), "Matlab");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("m");
 	ft->pattern = utils_strv_new("*.m", NULL);
 	ft->comment_open = g_strdup("%");
@@ -510,7 +572,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_YAML];
 	ft->lang = -2;
 	ft->name = g_strdup("YAML");
-	ft->title = g_strdup_printf(_("%s source file"), "YAML");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("yaml");
 	ft->pattern = utils_strv_new("*.yaml", "*.yml", NULL);
 	ft->comment_open = g_strdup("#");
@@ -521,7 +583,7 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_CMAKE];
 	ft->lang = -2;
 	ft->name = g_strdup("CMake");
-	ft->title = g_strdup_printf(_("%s source file"), "CMake");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("cmake");
 	ft->pattern = utils_strv_new("CMakeLists.txt", "*.cmake", "*.ctest", NULL);
 	ft->comment_open = g_strdup("#");
@@ -530,9 +592,9 @@ static void init_builtin_filetypes(void)
 
 #define NSIS
 	ft = filetypes[GEANY_FILETYPES_NSIS];
-	ft->lang = -2;
+	ft->lang = 35;
 	ft->name = g_strdup("NSIS");
-	ft->title = g_strdup_printf(_("%s source file"), "NSIS");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("nsis");
 	ft->pattern = utils_strv_new("*.nsi", "*.nsh", NULL);
 	ft->comment_open = g_strdup(";");
@@ -543,23 +605,12 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_ADA];
 	ft->lang = -2;
 	ft->name = g_strdup("Ada");
-	ft->title = g_strdup_printf(_("%s source file"), "Ada");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->extension = g_strdup("adb");
 	ft->pattern = utils_strv_new("*.adb", "*.ads", NULL);
 	ft->comment_open = g_strdup("--");
 	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
-
-#define ALL
-	ft = filetypes[GEANY_FILETYPES_NONE];
-	ft->lang = -2;
-	ft->name = g_strdup("None");
-	ft->title = g_strdup(_("All files"));
-	ft->extension = g_strdup("*");
-	ft->pattern = utils_strv_new("*", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
-	ft->group = GEANY_FILETYPE_GROUP_NONE;
 }
 
 
@@ -577,7 +628,20 @@ static GeanyFiletype *filetype_new(void)
 }
 
 
-/* Add a filetype pointer to the list of available filetypes,
+static gint cmp_filetype(gconstpointer pft1, gconstpointer pft2)
+{
+	const GeanyFiletype *ft1 = pft1, *ft2 = pft2;
+
+	if (G_UNLIKELY(ft1->id == GEANY_FILETYPES_NONE))
+		return -1;
+	if (G_UNLIKELY(ft2->id == GEANY_FILETYPES_NONE))
+		return 1;
+
+	return utils_str_casecmp(ft1->title, ft2->title);
+}
+
+
+/* Add a filetype pointer to the lists of available filetypes,
  * and set the filetype::id field. */
 static void filetype_add(GeanyFiletype *ft)
 {
@@ -587,6 +651,8 @@ static void filetype_add(GeanyFiletype *ft)
 	ft->id = filetypes_array->len;	/* len will be the index for filetype_array */
 	g_ptr_array_add(filetypes_array, ft);
 	g_hash_table_insert(filetypes_hash, ft->name, ft);
+
+	filetypes_by_title = g_slist_insert_sorted(filetypes_by_title, ft, cmp_filetype);
 }
 
 
@@ -623,6 +689,18 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 	if (utils_str_equal(doc->real_path,
 		utils_build_path(app->configdir, "filetype_extensions.conf", NULL)))
 		filetypes_read_extensions();
+	else if (utils_str_equal(doc->real_path,
+		utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL)))
+	{
+		guint i;
+
+		/* Note: we don't reload other filetypes, even though the named styles may have changed.
+		 * The user can do this manually with 'Tools->Reload Configuration' */
+		filetypes_load_config(GEANY_FILETYPES_NONE, TRUE);
+
+		documents_foreach(i)
+			document_reload_config(documents[i]);
+	}
 }
 
 
@@ -630,66 +708,47 @@ static void setup_config_file_menus(void)
 {
 	ui_add_config_file_menu_item(
 		utils_build_path(app->configdir, "filetype_extensions.conf", NULL), NULL, NULL);
+	ui_add_config_file_menu_item(
+		utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL), NULL, NULL);
 
 	g_signal_connect(geany_object, "document-save", G_CALLBACK(on_document_save), NULL);
 }
 
 
-#define create_sub_menu(menu, item, title) \
-	(menu) = gtk_menu_new(); \
-	(item) = gtk_menu_item_new_with_mnemonic((title)); \
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM((item)), (menu)); \
-	gtk_container_add(GTK_CONTAINER(filetype_menu), (item)); \
-	gtk_widget_show((item));
+static GtkWidget *group_menus[GEANY_FILETYPE_GROUP_COUNT] = {NULL};
+
+static void create_sub_menu(GtkWidget *parent, gsize group_id, const gchar *title)
+{
+	GtkWidget *menu, *item;
+
+	menu = gtk_menu_new();
+	item = gtk_menu_item_new_with_mnemonic((title));
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+	gtk_container_add(GTK_CONTAINER(parent), item);
+	gtk_widget_show(item);
+	group_menus[group_id] = menu;
+}
 
 
 static void create_set_filetype_menu(void)
 {
-	filetype_id ft_id;
+	GSList *node;
 	GtkWidget *filetype_menu = ui_lookup_widget(main_widgets.window, "set_filetype1_menu");
-	GtkWidget *sub_menu = filetype_menu;
-	GtkWidget *sub_menu_programming, *sub_menu_scripts, *sub_menu_markup, *sub_menu_misc;
-	GtkWidget *sub_item_programming, *sub_item_scripts, *sub_item_markup, *sub_item_misc;
 
-	create_sub_menu(sub_menu_programming, sub_item_programming, _("_Programming Languages"));
-	create_sub_menu(sub_menu_scripts, sub_item_scripts, _("_Scripting Languages"));
-	create_sub_menu(sub_menu_markup, sub_item_markup, _("_Markup Languages"));
-	create_sub_menu(sub_menu_misc, sub_item_misc, _("M_iscellaneous Languages"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_COMPILED, _("_Programming Languages"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_SCRIPT, _("_Scripting Languages"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MARKUP, _("_Markup Languages"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MISC, _("M_iscellaneous Languages"));
 
 	/* Append all filetypes to the filetype menu */
-	for (ft_id = 0; ft_id < filetypes_array->len; ft_id++)
+	foreach_slist(node, filetypes_by_title)
 	{
-		GeanyFiletype *ft = filetypes[ft_id];
-		const gchar *title = ft->title;
+		GeanyFiletype *ft = node->data;
 
-		/* insert separators for different filetype groups */
-		switch (ft->group)
-		{
-			case GEANY_FILETYPE_GROUP_COMPILED:	/* programming */
-				sub_menu = sub_menu_programming;
-				break;
-
-			case GEANY_FILETYPE_GROUP_SCRIPT:	/* scripts */
-				sub_menu = sub_menu_scripts;
-				break;
-
-			case GEANY_FILETYPE_GROUP_MARKUP:	/* markup */
-				sub_menu = sub_menu_markup;
-				break;
-
-			case GEANY_FILETYPE_GROUP_MISC:	/* misc */
-				sub_menu = sub_menu_misc;
-				break;
-
-			case GEANY_FILETYPE_GROUP_NONE:	/* none */
-				sub_menu = filetype_menu;
-				title = _("None");
-				break;
-
-			default: break;
-		}
-		create_radio_menu_item(sub_menu, title, ft);
+		if (ft->id != GEANY_FILETYPES_NONE)
+			create_radio_menu_item(group_menus[ft->group], ft);
 	}
+	create_radio_menu_item(filetype_menu, filetypes[GEANY_FILETYPES_NONE]);
 }
 
 
@@ -701,11 +760,8 @@ void filetypes_init()
 }
 
 
-typedef gboolean FileTypesPredicate(GeanyFiletype *ft, gpointer user_data);
-
 /* Find a filetype that predicate returns TRUE for, otherwise return NULL. */
-static GeanyFiletype *filetypes_find(gboolean source_only,
-		FileTypesPredicate predicate, gpointer user_data)
+GeanyFiletype *filetypes_find(GCompareFunc predicate, gpointer user_data)
 {
 	guint i;
 
@@ -713,9 +769,6 @@ static GeanyFiletype *filetypes_find(gboolean source_only,
 	{
 		GeanyFiletype *ft = filetypes[i];
 
-		if (source_only && i == GEANY_FILETYPES_NONE)
-			continue;	/* None is not for source files */
-
 		if (predicate(ft, user_data))
 			return ft;
 	}
@@ -723,12 +776,16 @@ static GeanyFiletype *filetypes_find(gboolean source_only,
 }
 
 
-static gboolean match_basename(GeanyFiletype *ft, gpointer user_data)
+static gboolean match_basename(gconstpointer pft, gconstpointer user_data)
 {
+	const GeanyFiletype *ft = pft;
 	const gchar *base_filename = user_data;
 	gint j;
 	gboolean ret = FALSE;
 
+	if (G_UNLIKELY(ft->id == GEANY_FILETYPES_NONE))
+		return FALSE;
+
 	for (j = 0; ft->pattern[j] != NULL; j++)
 	{
 		GPatternSpec *pattern = g_pattern_spec_new(ft->pattern[j]);
@@ -759,7 +816,7 @@ GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
 	setptr(base_filename, g_utf8_strdown(base_filename, -1));
 #endif
 
-	ft = filetypes_find(TRUE, match_basename, base_filename);
+	ft = filetypes_find(match_basename, base_filename);
 	if (ft == NULL)
 		ft = filetypes[GEANY_FILETYPES_NONE];
 
@@ -784,9 +841,9 @@ static gboolean shebang_find_and_match_filetype(const gchar *utf8_filename, gint
 		return FALSE;
 
 	va_start(args, first);
+	test = first;
 	while (1)
 	{
-		test = va_arg(args, gint);
 		if (test == -1)
 			break;
 
@@ -795,6 +852,7 @@ static gboolean shebang_find_and_match_filetype(const gchar *utf8_filename, gint
 			result = TRUE;
 			break;
 		}
+		test = va_arg(args, gint);
 	}
 	va_end(args);
 
@@ -922,7 +980,7 @@ GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc)
  *
  *  @param utf8_filename The filename in UTF-8 encoding.
  *
- *  @return The detected filetype for @c utf8_filename or @c filetypes[GEANY_FILETYPES_NONE]
+ *  @return The detected filetype for @a utf8_filename or @c filetypes[GEANY_FILETYPES_NONE]
  *          if it could not be detected.
  **/
 GeanyFiletype *filetypes_detect_from_file(const gchar *utf8_filename)
@@ -960,23 +1018,23 @@ void filetypes_select_radio_item(const GeanyFiletype *ft)
 
 
 static void
-on_filetype_change                     (GtkMenuItem     *menuitem,
+on_filetype_change                     (GtkCheckMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	GeanyDocument *doc = document_get_current();
-	if (ignore_callback || doc == NULL)
+	if (ignore_callback || doc == NULL || ! gtk_check_menu_item_get_active(menuitem))
 		return;
 
 	document_set_filetype(doc, (GeanyFiletype*)user_data);
 }
 
 
-static void create_radio_menu_item(GtkWidget *menu, const gchar *label, GeanyFiletype *ftype)
+static void create_radio_menu_item(GtkWidget *menu, GeanyFiletype *ftype)
 {
 	static GSList *group = NULL;
 	GtkWidget *tmp;
 
-	tmp = gtk_radio_menu_item_new_with_label(group, label);
+	tmp = gtk_radio_menu_item_new_with_label(group, ftype->title);
 	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(tmp));
 	ftype->priv->menu_item = tmp;
 	gtk_widget_show(tmp);
@@ -1034,12 +1092,13 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	set_error_regex(ft, NULL);
 
 	g_strfreev(ft->pattern);
+	g_free(ft->priv);
 	g_free(ft);
 }
 
 
 /* frees the array and all related pointers */
-void filetypes_free_types()
+void filetypes_free_types(void)
 {
 	g_return_if_fail(filetypes_array != NULL);
 	g_return_if_fail(filetypes_hash != NULL);
@@ -1060,7 +1119,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	/* default extension */
 	result = g_key_file_get_string(configh, "settings", "extension", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "settings", "extension", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		setptr(filetypes[ft_id]->extension, result);
 	}
@@ -1068,7 +1127,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	/* read comment notes */
 	result = g_key_file_get_string(configh, "settings", "comment_open", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "settings", "comment_open", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		g_free(filetypes[ft_id]->comment_open);
 		filetypes[ft_id]->comment_open = result;
@@ -1076,7 +1135,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 
 	result = g_key_file_get_string(configh, "settings", "comment_close", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "settings", "comment_close", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		g_free(filetypes[ft_id]->comment_close);
 		filetypes[ft_id]->comment_close = result;
@@ -1096,7 +1155,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	/* read context action */
 	result = g_key_file_get_string(configh, "settings", "context_action_cmd", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "settings", "context_action_cmd", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		filetypes[ft_id]->context_action_cmd = result;
 	}
@@ -1104,7 +1163,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	/* read build settings */
 	result = g_key_file_get_string(configh, "build_settings", "compiler", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "compiler", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		filetypes[ft_id]->programs->compiler = result;
 		filetypes[ft_id]->actions->can_compile = TRUE;
@@ -1120,7 +1179,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 
 	result = g_key_file_get_string(configh, "build_settings", "run_cmd", NULL);
 	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "run_cmd", NULL);
-	if (result != NULL)
+	if (G_LIKELY(result != NULL))
 	{
 		filetypes[ft_id]->programs->run_cmd = result;
 		filetypes[ft_id]->actions->can_exec = TRUE;
@@ -1149,7 +1208,7 @@ static void load_system_keyfile(GKeyFile *key_file, const gchar *file, GKeyFileF
 {
 	GError *error = NULL;
 	gboolean done = g_key_file_load_from_file(key_file, file, flags, &error);
-	if (! done && error != NULL)
+	if (G_UNLIKELY(! done) && G_UNLIKELY(error != NULL))
 	{
 		geany_debug("Failed to open %s (%s)", file, error->message);
 		g_error_free(error);
@@ -1171,11 +1230,11 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 	pft = filetypes[ft_id]->priv;
 
 	/* when reloading, proceed only if the settings were already loaded */
-	if (reload && ! pft->keyfile_loaded)
+	if (reload && G_UNLIKELY(! pft->keyfile_loaded))
 		return;
 
 	/* when not reloading, load the settings only once */
-	if (! reload && pft->keyfile_loaded)
+	if (! reload && G_LIKELY(pft->keyfile_loaded))
 		return;
 	pft->keyfile_loaded = TRUE;
 
@@ -1198,9 +1257,7 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 	}
 
 	load_settings(ft_id, config, config_home);
-	if (! reload)
-		/* reloading highlighting settings not yet supported */
-		highlighting_init_styles(ft_id, config, config_home);
+	highlighting_init_styles(ft_id, config, config_home);
 
 	g_key_file_free(config);
 	g_key_file_free(config_home);
@@ -1223,19 +1280,20 @@ gchar *filetypes_get_conf_extension(gint filetype_idx)
 }
 
 
-void filetypes_save_commands()
+void filetypes_save_commands(void)
 {
 	gchar *conf_prefix = g_strconcat(app->configdir,
 		G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", NULL);
 	gint i;
 
-	for (i = 0; i < GEANY_FILETYPES_NONE; i++)
+	for (i = 1; i < GEANY_MAX_BUILT_IN_FILETYPES; i++)
 	{
 		struct build_programs *bp = filetypes[i]->programs;
 		GKeyFile *config_home;
 		gchar *fname, *ext, *data;
 
-		if (! bp->modified) continue;
+		if (! bp->modified)
+			continue;
 
 		ext = filetypes_get_conf_extension(i);
 		fname = g_strconcat(conf_prefix, ext, NULL);
@@ -1244,13 +1302,13 @@ void filetypes_save_commands()
 		config_home = g_key_file_new();
 		g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
-		if (bp->compiler && *bp->compiler)
+		if (NZV(bp->compiler))
 			g_key_file_set_string(config_home, "build_settings", "compiler", bp->compiler);
-		if (bp->linker && *bp->linker)
+		if (NZV(bp->linker))
 			g_key_file_set_string(config_home, "build_settings", "linker", bp->linker);
-		if (bp->run_cmd && *bp->run_cmd)
+		if (NZV(bp->run_cmd))
 			g_key_file_set_string(config_home, "build_settings", "run_cmd", bp->run_cmd);
-		if (bp->run_cmd2 && *bp->run_cmd2)
+		if (NZV(bp->run_cmd2))
 			g_key_file_set_string(config_home, "build_settings", "run_cmd2", bp->run_cmd2);
 
 		data = g_key_file_to_data(config_home, NULL, NULL);
@@ -1264,7 +1322,7 @@ void filetypes_save_commands()
 
 
 /* create one file filter which has each file pattern of each filetype */
-GtkFileFilter *filetypes_create_file_filter_all_source()
+GtkFileFilter *filetypes_create_file_filter_all_source(void)
 {
 	GtkFileFilter *new_filter;
 	guint i, j;
@@ -1274,7 +1332,7 @@ GtkFileFilter *filetypes_create_file_filter_all_source()
 
 	for (i = 0; i < filetypes_array->len; i++)
 	{
-		if (i == GEANY_FILETYPES_NONE)
+		if (G_UNLIKELY(i == GEANY_FILETYPES_NONE))
 			continue;
 
 		for (j = 0; filetypes[i]->pattern[j]; j++)
@@ -1290,11 +1348,13 @@ GtkFileFilter *filetypes_create_file_filter(const GeanyFiletype *ft)
 {
 	GtkFileFilter *new_filter;
 	gint i;
+	const gchar *title;
 
 	g_return_val_if_fail(ft != NULL, NULL);
 
 	new_filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(new_filter, ft->title);
+	title = ft->id == GEANY_FILETYPES_NONE ? _("All files") : ft->title;
+	gtk_file_filter_set_name(new_filter, title);
 
 	for (i = 0; ft->pattern[i]; i++)
 	{
@@ -1314,7 +1374,7 @@ gboolean filetype_has_tags(GeanyFiletype *ft)
 }
 
 
-/** Find a filetype pointer from its @c name field.
+/** Find a filetype pointer from its @a name field.
  * @param name Filetype name.
  * @return The filetype found, or @c NULL.
  *
@@ -1327,7 +1387,7 @@ GeanyFiletype *filetypes_lookup_by_name(const gchar *name)
 	g_return_val_if_fail(NZV(name), NULL);
 
 	ft = g_hash_table_lookup(filetypes_hash, name);
-	if (ft == NULL)
+	if (G_UNLIKELY(ft == NULL))
 		geany_debug("Could not find filetype '%s'.", name);
 	return ft;
 }
@@ -1347,7 +1407,7 @@ static void compile_regex(GeanyFiletype *ft, regex_t *regex)
 
 	ft->priv->error_regex_compiled = (retval == 0);	/* prevent recompilation */
 
-	if (retval != 0)
+	if (G_UNLIKELY(retval != 0))
 	{
 		gchar buf[256];
 		regerror(retval, regex, buf, sizeof buf);
@@ -1441,7 +1501,7 @@ void filetypes_read_extensions(void)
 			g_key_file_has_key(userconfig, "Extensions", filetypes[i]->name, NULL);
 		list = g_key_file_get_string_list(
 			(userset) ? userconfig : sysconfig, "Extensions", filetypes[i]->name, &len, NULL);
-		if (list && len > 0)
+		if (G_LIKELY(list) && G_LIKELY(len > 0))
 		{
 			g_strfreev(filetypes[i]->pattern);
 			filetypes[i]->pattern = list;
@@ -1467,5 +1527,3 @@ GeanyFiletype *filetypes_index(gint idx)
 {
 	return (idx >= 0 && idx < (gint) filetypes_array->len) ? filetypes[idx] : NULL;
 }
-
-
