@@ -1,8 +1,8 @@
 /*
  *      filetypes.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 3018 2008-09-28 16:32:49Z eht16 $
+ * $Id: filetypes.h 4066 2009-08-11 13:22:42Z ntrel $
  */
 
 
@@ -29,60 +29,58 @@
 #include "ScintillaWidget.h"
 
 
-/* Each group should be alpha-sorted, based on filetype::name (not enum name).
- * Warning: remember to break the plugin ABI when adding items. */
-/* TODO: for a stable ABI put GEANY_FILETYPES_NONE first, and use a sorted duplicate
- * filetypes array for GUI elements. */
 typedef enum
 {
-	/* normally compiled languages */
-	GEANY_FILETYPES_ASM = 0,
-	GEANY_FILETYPES_C,
-	GEANY_FILETYPES_CPP,
-	GEANY_FILETYPES_CS,
-	GEANY_FILETYPES_CAML,
-	GEANY_FILETYPES_D,
-	GEANY_FILETYPES_F77,
-	GEANY_FILETYPES_FORTRAN,
-	GEANY_FILETYPES_BASIC,	/* FreeBasic */
-	GEANY_FILETYPES_GLSL,
-	GEANY_FILETYPES_HASKELL,
-	GEANY_FILETYPES_HAXE,
-	GEANY_FILETYPES_JAVA,
-	GEANY_FILETYPES_PASCAL,
-	GEANY_FILETYPES_VHDL,
+	GEANY_FILETYPES_NONE = 0,	/* first filetype is always None */
 
-	/* script languages */
-	GEANY_FILETYPES_FERITE,
-	GEANY_FILETYPES_JS,
-	GEANY_FILETYPES_LUA,
-	GEANY_FILETYPES_MAKE,
-	GEANY_FILETYPES_OMS,
-	GEANY_FILETYPES_PERL,
 	GEANY_FILETYPES_PHP,
-	GEANY_FILETYPES_PYTHON,
-	GEANY_FILETYPES_R,
+	GEANY_FILETYPES_BASIC,	/* FreeBasic */
+	GEANY_FILETYPES_MATLAB,
 	GEANY_FILETYPES_RUBY,
-	GEANY_FILETYPES_SH,
-	GEANY_FILETYPES_TCL,
-
-	/* markup langauges */
-	GEANY_FILETYPES_CSS,
-	GEANY_FILETYPES_DOCBOOK,
-	GEANY_FILETYPES_HTML,
- 	GEANY_FILETYPES_XML,
-
-	/* miscellaneous languages */
-	GEANY_FILETYPES_CONF,
-	GEANY_FILETYPES_DIFF,
-	GEANY_FILETYPES_LATEX,
-	GEANY_FILETYPES_REST,
-	GEANY_FILETYPES_SQL,
+	GEANY_FILETYPES_LUA,
+	GEANY_FILETYPES_FERITE,
+	GEANY_FILETYPES_YAML,
+	GEANY_FILETYPES_C,
+	GEANY_FILETYPES_NSIS,
+	GEANY_FILETYPES_GLSL,
 	GEANY_FILETYPES_PO,
-
-	GEANY_FILETYPES_NONE,	/* must be last filetype */
-	GEANY_MAX_BUILT_IN_FILETYPES	/* Use filetypes_array->len instead */
-} filetype_id;
+	GEANY_FILETYPES_MAKE,
+	GEANY_FILETYPES_TCL,
+	GEANY_FILETYPES_XML,
+	GEANY_FILETYPES_CSS,
+	GEANY_FILETYPES_REST,
+	GEANY_FILETYPES_HASKELL,
+	GEANY_FILETYPES_JAVA,
+	GEANY_FILETYPES_CAML,
+	GEANY_FILETYPES_AS,
+	GEANY_FILETYPES_R,
+	GEANY_FILETYPES_DIFF,
+	GEANY_FILETYPES_HTML,
+	GEANY_FILETYPES_PYTHON,
+	GEANY_FILETYPES_CS,
+	GEANY_FILETYPES_PERL,
+	GEANY_FILETYPES_VALA,
+	GEANY_FILETYPES_PASCAL,
+	GEANY_FILETYPES_LATEX,
+	GEANY_FILETYPES_ASM,
+	GEANY_FILETYPES_CONF,
+	GEANY_FILETYPES_HAXE,
+	GEANY_FILETYPES_CPP,
+	GEANY_FILETYPES_SH,
+	GEANY_FILETYPES_FORTRAN,
+	GEANY_FILETYPES_SQL,
+	GEANY_FILETYPES_F77,
+	GEANY_FILETYPES_DOCBOOK,
+	GEANY_FILETYPES_D,
+	GEANY_FILETYPES_JS,
+	GEANY_FILETYPES_VHDL,
+	GEANY_FILETYPES_ADA,
+	GEANY_FILETYPES_CMAKE,
+	GEANY_FILETYPES_MARKDOWN,
+	/* ^ append items here */
+	GEANY_MAX_BUILT_IN_FILETYPES	/* Don't use this, use filetypes_array->len instead */
+}
+filetype_id;
 
 typedef enum
 {
@@ -90,12 +88,14 @@ typedef enum
 	GEANY_FILETYPE_GROUP_COMPILED,
 	GEANY_FILETYPE_GROUP_SCRIPT,
 	GEANY_FILETYPE_GROUP_MARKUP,
-	GEANY_FILETYPE_GROUP_MISC
+	GEANY_FILETYPE_GROUP_MISC,
+	GEANY_FILETYPE_GROUP_COUNT
 }
 GeanyFiletypeGroupID;
 
 
-/* Safe wrapper to get the id field of a possibly NULL filetype pointer. */
+/* Safe wrapper to get the id field of a possibly NULL filetype pointer.
+ * This shouldn't be necessary since GeanyDocument::file_type is always non-NULL. */
 #define FILETYPE_ID(filetype_ptr) \
 	(((filetype_ptr) != NULL) ? (filetype_ptr)->id : GEANY_FILETYPES_NONE)
 
@@ -123,14 +123,14 @@ struct GeanyFiletype
 	/** Represents the langType of tagmanager (see the table
 	 * in tagmanager/parsers.h), -1 represents all, -2 none. */
 	langType 		  lang;
-	gchar	 		 *name;				/**< Used as name for tagmanager. E.g. "C". */
-	gchar	 		 *title;			/**< Shown in the file open dialog. E.g. "C source file". */
-	gchar	 		 *extension;		/**< Default file extension for new files. */
+	gchar			 *name;				/**< Used as name for tagmanager. E.g. "C". */
+	gchar			 *title;			/**< Shown in the file open dialog. E.g. "C source file". */
+	gchar			 *extension;		/**< Default file extension for new files. */
 	gchar			**pattern;			/**< Array of filename-matching wildcard strings. */
-	gchar	 		 *context_action_cmd;
-	gchar	 		 *comment_open;
-	gchar	 		 *comment_close;
-	gboolean  		  comment_use_indent;
+	gchar			 *context_action_cmd;
+	gchar			 *comment_open;
+	gchar			 *comment_close;
+	gboolean		  comment_use_indent;
 	struct build_programs	*programs;
 	struct build_actions	*actions;	/* TODO: make private */
 	GeanyFiletypeGroupID	group;
@@ -141,18 +141,27 @@ struct GeanyFiletype
 
 extern GPtrArray *filetypes_array;
 
-/* Wrap filetypes_array so it can be used with C array syntax.
- * Example: filetypes[GEANY_FILETYPES_C]->name = ...; */
-#define filetypes	((GeanyFiletype **)filetypes_array->pdata)
+/** Wrap filetypes_array so it can be used with C array syntax.
+ * Example: filetypes[GEANY_FILETYPES_C]->name = ...;
+ * @see filetypes_index(). */
+#define filetypes	((GeanyFiletype **)GEANY(filetypes_array)->pdata)
+
+extern GSList *filetypes_by_title;
 
 
 GeanyFiletype *filetypes_lookup_by_name(const gchar *name);
 
+GeanyFiletype *filetypes_find(GCompareFunc predicate, gpointer user_data);
+
 
 void filetypes_init(void);
 
 void filetypes_init_types(void);
 
+void filetypes_read_extensions(void);
+
+GeanyFiletype *filetypes_index(gint idx);
+
 GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc);
 
 GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename);
