@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.c 3549 2009-02-05 19:04:54Z eht16 $
+ * $Id: geanyobject.c 3976 2009-07-16 15:04:21Z ntrel $
  */
 
 /* A GObject used for connecting and emitting signals when certain events happen,
@@ -27,8 +27,12 @@
  * Mainly used for plugins - see the API docs.
  *
  * Core-only signals:
+ *
  * signal void save_settings(GObject *obj, GKeyFile *keyfile, gpointer user_data);
  * Emitted just before saving main keyfile settings.
+
+ * signal void load_settings(GObject *obj, GKeyFile *keyfile, gpointer user_data);
+ * Emitted just after loading main keyfile settings.
  */
 
 #include "geany.h"
@@ -244,6 +248,15 @@ static void create_signals(GObjectClass *g_object_class)
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER);
+	geany_object_signals[GCB_LOAD_SETTINGS] = g_signal_new (
+		"load-settings",
+		G_OBJECT_CLASS_TYPE (g_object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET (GeanyObjectClass, load_settings),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__POINTER,
+		G_TYPE_NONE, 1,
+		G_TYPE_POINTER);
 }
 
 
@@ -260,12 +273,12 @@ static void geany_object_class_init(GeanyObjectClass *klass)
 
 static void geany_object_init(GeanyObject *self)
 {
-
+	/* nothing to do */
 }
 
 
-GObject* geany_object_new(void)
+GObject *geany_object_new(void)
 {
-	return (GObject*)g_object_new(GEANY_OBJECT_TYPE, NULL);
+	return g_object_new(GEANY_OBJECT_TYPE, NULL);
 }
 
