@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.h 3446 2009-01-04 18:30:42Z eht16 $
+ * $Id: geanyobject.h 3976 2009-07-16 15:04:21Z ntrel $
  */
 
 
@@ -43,6 +43,7 @@ typedef enum
 	GCB_UPDATE_EDITOR_MENU,
 	GCB_EDITOR_NOTIFY,
 	GCB_SAVE_SETTINGS,
+	GCB_LOAD_SETTINGS,
 	GCB_MAX
 }
 GeanyCallbackId;
@@ -84,6 +85,7 @@ struct _GeanyObjectClass
 	void (*update_editor_menu)(const gchar *word, gint click_pos, GeanyDocument *doc);
 	gboolean (*editor_notify)(GeanyEditor *editor, gpointer scnt);
 	void (*save_settings)(GKeyFile *keyfile);
+	void (*load_settings)(GKeyFile *keyfile);
 };
 
 GType		geany_object_get_type	(void);
