@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: keyfile.h 3446 2009-01-04 18:30:42Z eht16 $
+ * $Id: keyfile.h 3707 2009-04-15 22:46:13Z eht16 $
  */
 
 
@@ -47,7 +47,7 @@ void configuration_reload_default_session(void);
 
 void configuration_save_default_session(void);
 
-void configuration_load_session_files(GKeyFile *config);
+void configuration_load_session_files(GKeyFile *config, gboolean read_recent_files);
 
 void configuration_save_session_files(GKeyFile *config);
 
