@@ -2,8 +2,8 @@
  *      navqueue.c - this file is part of Geany, a fast and lightweight IDE
  *
  *      Copyright 2007 Dave Moore <wrex006(at)gmail(dot)com>
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: navqueue.c 2505 2008-04-19 08:14:00Z eht16 $
+ * $Id: navqueue.c 4010 2009-07-21 22:12:13Z eht16 $
  */
 
 /*
@@ -28,24 +28,27 @@
 
 #include "geany.h"
 
-#include "navqueue.h"
 #include "sciwrappers.h"
 #include "document.h"
 #include "utils.h"
 #include "support.h"
+#include "ui_utils.h"
+#include "editor.h"
+#include "navqueue.h"
+#include "toolbar.h"
 
 
 /* for the navigation history queue */
 typedef struct
 {
-	gchar *file;	/* This is the document's filename, in UTF-8 */
+	const gchar *file;	/* This is the document's filename, in UTF-8 */
 	gint pos;
 } filepos;
 
 static GQueue *navigation_queue;
 static guint nav_queue_pos;
 
-static GtkWidget *navigation_buttons[2];
+static GtkAction *navigation_buttons[2];
 
 
 
@@ -54,8 +57,8 @@ void navqueue_init()
 	navigation_queue = g_queue_new();
 	nav_queue_pos = 0;
 
-	navigation_buttons[0] = lookup_widget(app->window, "toolbutton_back");
-	navigation_buttons[1] = lookup_widget(app->window, "toolbutton_forward");
+	navigation_buttons[0] = toolbar_get_action_by_name("NavBack");
+	navigation_buttons[1] = toolbar_get_action_by_name("NavFor");
 }
 
 
@@ -73,23 +76,23 @@ static void adjust_buttons(void)
 {
 	if (g_queue_get_length(navigation_queue) < 2)
 	{
-		gtk_widget_set_sensitive(navigation_buttons[0], FALSE);
-		gtk_widget_set_sensitive(navigation_buttons[1], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[1], FALSE);
 		return;
 	}
 	if (nav_queue_pos == 0)
 	{
-		gtk_widget_set_sensitive(navigation_buttons[0], TRUE);
-		gtk_widget_set_sensitive(navigation_buttons[1], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], TRUE);
+		gtk_action_set_sensitive(navigation_buttons[1], FALSE);
 		return;
 	}
 	/* forward should be sensitive since where not at the start */
-	gtk_widget_set_sensitive(navigation_buttons[1], TRUE);
+	gtk_action_set_sensitive(navigation_buttons[1], TRUE);
 
 	/* back should be sensitive if there's a place to go back to */
 	(nav_queue_pos < g_queue_get_length(navigation_queue) - 1) ?
-		gtk_widget_set_sensitive(navigation_buttons[0], TRUE) :
-			gtk_widget_set_sensitive(navigation_buttons[0], FALSE);
+		gtk_action_set_sensitive(navigation_buttons[0], TRUE) :
+			gtk_action_set_sensitive(navigation_buttons[0], FALSE);
 }
 
 
@@ -106,7 +109,7 @@ queue_pos_matches(guint queue_pos, const gchar *fname, gint pos)
 }
 
 
-static void add_new_position(gchar *utf8_filename, gint pos)
+static void add_new_position(const gchar *utf8_filename, gint pos)
 {
 	filepos *npos;
 	guint i;
@@ -135,37 +138,48 @@ static void add_new_position(gchar *utf8_filename, gint pos)
 /**
  *  Add old file position and new file position to the navqueue, then goes to the new position.
  *
- *  @param old_idx the %document index of the previous position, if set as invalid (-1) then no old
+ *  @param old_doc The document of the previous position, if set as invalid (@c NULL) then no old
  *         position is set
- *  @param new_idx the %document index of the new position, must be valid.
+ *  @param new_doc The document of the new position, must be valid.
  *  @param line the line number of the new position. It is counted with 1 as the first line, not 0.
  *
- *  @return @a TRUE if the cursor has changed the position to @a line or @a FALSE otherwise.
+ *  @return @c TRUE if the cursor has changed the position to @a line or @c FALSE otherwise.
  **/
-gboolean navqueue_goto_line(gint old_idx, gint new_idx, gint line)
+gboolean navqueue_goto_line(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line)
 {
 	gint pos;
 
-	g_return_val_if_fail(DOC_IDX_VALID(new_idx), FALSE);
+	g_return_val_if_fail(new_doc != NULL, FALSE);
 	g_return_val_if_fail(line >= 1, FALSE);
 
-	pos = sci_get_position_from_line(doc_list[new_idx].sci, line - 1);
+	pos = sci_get_position_from_line(new_doc->editor->sci, line - 1);
 
 	/* first add old file position */
-	if (DOC_IDX_VALID(old_idx) && doc_list[old_idx].file_name)
+	if (old_doc != NULL && old_doc->file_name)
 	{
-		gint cur_pos = sci_get_current_position(doc_list[old_idx].sci);
+		gint cur_pos = sci_get_current_position(old_doc->editor->sci);
 
-		add_new_position(doc_list[old_idx].file_name, cur_pos);
+		add_new_position(old_doc->file_name, cur_pos);
 	}
 
 	/* now add new file position */
-	if (doc_list[new_idx].file_name)
+	if (new_doc->file_name)
 	{
-		add_new_position(doc_list[new_idx].file_name, pos);
+		add_new_position(new_doc->file_name, pos);
 	}
 
-	return utils_goto_pos(new_idx, pos);
+	return editor_goto_pos(new_doc->editor, pos, TRUE);
+}
+
+
+static gboolean goto_file_pos(const gchar *file, gint pos)
+{
+	GeanyDocument *doc = document_find_by_filename(file);
+
+	if (doc == NULL)
+		return FALSE;
+
+	return editor_goto_pos(doc->editor, pos, TRUE);
 }
 
 
@@ -180,7 +194,7 @@ void navqueue_go_back()
 
 	/* jump back */
 	fprev = g_queue_peek_nth(navigation_queue, nav_queue_pos + 1);
-	if (utils_goto_file_pos(fprev->file, FALSE, fprev->pos))
+	if (goto_file_pos(fprev->file, fprev->pos))
 	{
 		nav_queue_pos++;
 	}
@@ -203,7 +217,7 @@ void navqueue_go_forward()
 
 	/* jump forward */
 	fnext = g_queue_peek_nth(navigation_queue, nav_queue_pos - 1);
-	if (utils_goto_file_pos(fnext->file, FALSE, fnext->pos))
+	if (goto_file_pos(fnext->file, fnext->pos))
 	{
 		nav_queue_pos--;
 	}
