@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 3504 2009-01-22 20:33:21Z eht16 $
+ * $Id: notebook.c 3808 2009-05-20 15:01:04Z ntrel $
  */
 
 /*
@@ -34,6 +34,7 @@
 #include "treeviews.h"
 #include "support.h"
 #include "callbacks.h"
+#include "utils.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -179,6 +180,77 @@ static gboolean is_position_on_tab_bar(GtkNotebook *notebook, GdkEventButton *ev
 }
 
 
+static void tab_bar_menu_activate_cb(GtkMenuItem *menuitem, gpointer data)
+{
+	GeanyDocument *doc = data;
+
+	if (! DOC_VALID(doc))
+		return;
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
+		document_get_notebook_page(doc));
+}
+
+
+static GtkMenu *get_tab_bar_popup_menu(void)
+{
+	static GtkWidget *menu = NULL;
+	GtkWidget *menu_item, *menu_item_label;
+	const GdkColor *color;
+	GeanyDocument *doc;
+	guint i, len;
+	gchar *base_name;
+	GeanyDocument *current_doc = document_get_current();
+
+	if (G_UNLIKELY(menu == NULL))
+		menu = gtk_menu_new();
+
+	/* clear the old menu items */
+	gtk_container_foreach(GTK_CONTAINER(menu), (GtkCallback) gtk_widget_destroy, NULL);
+
+	len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
+	for (i = 0; i < len; i++)
+	{
+		doc = document_get_from_page(i);
+		if (! DOC_VALID(doc))
+			continue;
+
+		base_name = g_path_get_basename(DOC_FILENAME(doc));
+		menu_item = gtk_menu_item_new_with_label(base_name);
+		gtk_widget_show(menu_item);
+		gtk_container_add(GTK_CONTAINER(menu), menu_item);
+		g_signal_connect(menu_item, "activate", G_CALLBACK(tab_bar_menu_activate_cb), doc);
+
+		color = document_get_status_color(doc);
+		menu_item_label = gtk_bin_get_child(GTK_BIN(menu_item));
+		gtk_widget_modify_fg(menu_item_label, GTK_STATE_NORMAL, color);
+		gtk_widget_modify_fg(menu_item_label, GTK_STATE_ACTIVE, color);
+
+		if (doc == current_doc)
+		{
+			setptr(base_name, g_strconcat("<b>", base_name, "</b>", NULL));
+			gtk_label_set_markup(GTK_LABEL(menu_item_label), base_name);
+		}
+		g_free(base_name);
+	}
+	menu_item = gtk_separator_menu_item_new();
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+
+	menu_item = gtk_menu_item_new_with_mnemonic(_("Close Ot_her Documents"));
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), NULL);
+
+	menu_item = gtk_menu_item_new_with_mnemonic(_("C_lose All"));
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_all1_activate), NULL);
+
+	return GTK_MENU(menu);
+}
+
+
 static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *event,
 										  gpointer user_data)
 {
@@ -195,6 +267,11 @@ static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *eve
 			return TRUE;
 		}
 	}
+	else if (event->button == 3)
+	{
+		gtk_menu_popup(get_tab_bar_popup_menu(), NULL, NULL,
+			NULL, NULL, event->button, event->time);
+	}
 	return FALSE;
 }
 
@@ -274,7 +351,7 @@ notebook_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
 	{
 		gboolean ok = FALSE;
 		/* prevent oscillation between non-homogeneous sized tabs */
-		switch(gtk_notebook_get_tab_pos(notebook))
+		switch (gtk_notebook_get_tab_pos(notebook))
 		{
 			case GTK_POS_LEFT:
 			case GTK_POS_RIGHT:
@@ -452,17 +529,14 @@ gint notebook_new_tab(GeanyDocument *this)
 
 	gtk_widget_show_all(ebox);
 
-	this->priv->tabmenu_label = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(this->priv->tabmenu_label), 0.0, 0);
-
 	document_update_tab_label(this);
 
 	if (file_prefs.tab_order_ltr)
 		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
-			ebox, this->priv->tabmenu_label);
+			ebox, NULL);
 	else
 		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
-			ebox, this->priv->tabmenu_label, 0);
+			ebox, NULL, 0);
 
 	tab_count_changed();
 
