@@ -19,12 +19,10 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 3567 2009-02-08 19:52:21Z eht16 $
+ * $Id: plugins.c 4047 2009-07-30 11:56:25Z ntrel $
  */
 
 /* Code to manage, load and unload plugins. */
-/** @file plugins.c
- * Plugin utility functions. */
 
 #include "geany.h"
 
@@ -60,21 +58,9 @@
 #include "toolbar.h"
 #include "stash.h"
 #include "keyfile.h"
-
-
-#ifdef G_OS_WIN32
-# define PLUGIN_EXT "dll"
-#else
-# define PLUGIN_EXT "so"
-#endif
-
-
-typedef struct GeanyPluginPrivate
-{
-	GeanyAutoSeparator	toolbar_separator;
-	gboolean			resident;
-}
-GeanyPluginPrivate;
+#include "win32.h"
+#include "pluginutils.h"
+#include "pluginprivate.h"
 
 
 typedef struct Plugin
@@ -84,10 +70,8 @@ typedef struct Plugin
 	PluginInfo		info;				/* plugin name, description, etc */
 	PluginFields	fields;
 	GeanyPlugin		public;				/* fields the plugin can read */
-	GeanyPluginPrivate	priv;			/* GeanyPlugin type private data */
+	GeanyPluginPrivate	priv;			/* GeanyPlugin type private data, same as (*public.priv) */
 
-	gulong		*signal_ids;			/* signal IDs to disconnect when unloading */
-	gsize		signal_ids_len;
 	GeanyKeyGroup	*key_group;
 
 	void		(*init) (GeanyData *data);			/* Called when the plugin is enabled */
@@ -111,13 +95,11 @@ static GtkWidget *menu_separator = NULL;
 
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data);
 
-void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item);
-void plugin_module_make_resident(GeanyPlugin *plugin);
-
 
 static PluginFuncs plugin_funcs = {
 	&plugin_add_toolbar_item,
-	&plugin_module_make_resident
+	&plugin_module_make_resident,
+	&plugin_signal_connect
 };
 
 static DocumentFuncs doc_funcs = {
@@ -138,7 +120,8 @@ static DocumentFuncs doc_funcs = {
 	&document_index,
 	&document_save_file_as,
 	&document_rename_file,
-	&document_get_status_color
+	&document_get_status_color,
+	&document_get_basename_for_display
 };
 
 static EditorFuncs editor_funcs = {
@@ -196,7 +179,11 @@ static SciFuncs sci_funcs = {
 	&sci_has_selection,
 	&sci_get_tab_width,
 	&sci_indicator_clear,
-	&sci_indicator_set
+	&sci_indicator_set,
+	&sci_get_contents,
+	&sci_get_contents_range,
+	&sci_get_selection_contents,
+	&sci_set_font
 };
 
 static TemplateFuncs template_funcs = {
@@ -220,7 +207,9 @@ static UtilsFuncs utils_funcs = {
 	&utils_str_casecmp,
 	&utils_get_date_time,
 	&utils_open_browser,
-	&utils_string_replace_first
+	&utils_string_replace_first,
+	&utils_str_middle_truncate,
+	&utils_str_remove_chars
 };
 
 static UIUtilsFuncs uiutils_funcs = {
@@ -347,7 +336,8 @@ geany_data_init(void)
 		&search_prefs,
 		&tool_prefs,
 		&template_prefs,
-		&build_info
+		&build_info,
+		filetypes_by_title
 	};
 	memcpy(&geany_data, &gd, sizeof(GeanyData));
 }
@@ -420,7 +410,7 @@ plugin_check_version(GModule *module)
 
 	g_module_symbol(module, "plugin_version_check", (void *) &version_check);
 
-	if (! version_check)
+	if (G_UNLIKELY(! version_check))
 	{
 		geany_debug("Plugin \"%s\" has no plugin_version_check() function - ignoring plugin!",
 				g_module_name(module));
@@ -432,7 +422,7 @@ plugin_check_version(GModule *module)
 
 		if (result < 0)
 		{
-			ui_set_statusbar(TRUE, _("The plugin \"%s\" is not binary compatible with this "
+			msgwin_status_add(_("The plugin \"%s\" is not binary compatible with this "
 				"release of Geany - please recompile it."), g_module_name(module));
 			geany_debug("Plugin \"%s\" is not binary compatible with this "
 				"release of Geany - recompile it.", g_module_name(module));
@@ -464,16 +454,12 @@ static void add_callbacks(Plugin *plugin, PluginCallback *callbacks)
 	if (len == 0)
 		return;
 
-	plugin->signal_ids_len = len;
-	plugin->signal_ids = g_new(gulong, len);
-
 	for (i = 0; i < len; i++)
 	{
 		cb = &callbacks[i];
 
-		plugin->signal_ids[i] = (cb->after) ?
-			g_signal_connect_after(geany_object, cb->signal_name, cb->callback, cb->user_data) :
-			g_signal_connect(geany_object, cb->signal_name, cb->callback, cb->user_data);
+		plugin_signal_connect(&plugin->public, NULL, cb->signal_name, cb->after,
+			cb->callback, cb->user_data);
 	}
 }
 
@@ -483,7 +469,13 @@ add_kb_group(Plugin *plugin)
 {
 	guint i;
 
-	g_return_if_fail(NZV(plugin->key_group->name));
+	if (!NZV(plugin->key_group->name))
+	{
+		geany_debug("Plugin \"%s\" has not set a name for its keybinding group"
+			" - ignoring all keybindings!",
+			plugin->info.name);
+		return;
+	}
 	g_return_if_fail(! g_str_equal(plugin->key_group->name, keybindings_keyfile_group_name));
 
 	for (i = 0; i < plugin->key_group->count; i++)
@@ -567,8 +559,7 @@ plugin_init(Plugin *plugin)
 	if (callbacks)
 		add_callbacks(plugin, callbacks);
 
-	g_module_symbol(plugin->module, "plugin_key_group",
-		(void *) &plugin->key_group);
+	g_module_symbol(plugin->module, "plugin_key_group", (void *) &plugin->key_group);
 	if (plugin->key_group)
 		add_kb_group(plugin);
 
@@ -671,7 +662,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 		g_free(plugin);
 		return NULL;
 	}
-	geany_debug("Initializing plugin '%s'", plugin->info.name);
+	/*geany_debug("Initializing plugin '%s'", plugin->info.name);*/
 
 	plugin->filename = g_strdup(fname);
 	plugin->module = module;
@@ -690,14 +681,16 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 
 static void remove_callbacks(Plugin *plugin)
 {
-	guint i;
+	GArray *signal_ids = plugin->priv.signal_ids;
+	SignalConnection *sc;
 
-	if (plugin->signal_ids == NULL)
+	if (signal_ids == NULL)
 		return;
 
-	for (i = 0; i < plugin->signal_ids_len; i++)
-		g_signal_handler_disconnect(geany_object, plugin->signal_ids[i]);
-	g_free(plugin->signal_ids);
+	foreach_array(SignalConnection, sc, signal_ids)
+		g_signal_handler_disconnect(sc->object, sc->handler_id);
+
+	g_array_free(signal_ids, TRUE);
 }
 
 
@@ -740,7 +733,7 @@ plugin_free(Plugin *plugin)
 
 	active_plugin_list = g_list_remove(active_plugin_list, plugin);
 
-	if (plugin->module != NULL && ! g_module_close(plugin->module))
+	if (! g_module_close(plugin->module))
 		g_warning("%s: %s", plugin->filename, g_module_error());
 
 	plugin_list = g_list_remove(plugin_list, plugin);
@@ -784,7 +777,7 @@ load_plugins_from_path(const gchar *path)
 	for (item = list; item != NULL; item = g_slist_next(item))
 	{
 		tmp = strrchr(item->data, '.');
-		if (tmp == NULL || utils_str_casecmp(tmp, "." PLUGIN_EXT) != 0)
+		if (tmp == NULL || utils_str_casecmp(tmp, "." G_MODULE_SUFFIX) != 0)
 			continue;
 
 		fname = g_strconcat(path, G_DIR_SEPARATOR_S, item->data, NULL);
@@ -800,10 +793,11 @@ load_plugins_from_path(const gchar *path)
 #ifdef G_OS_WIN32
 static gchar *get_plugin_path()
 {
-	gchar *install_dir = g_win32_get_package_installation_directory(NULL, NULL);
+	gchar *install_dir = win32_get_installation_dir();
 	gchar *path;
 
 	path = g_strconcat(install_dir, "\\lib", NULL);
+	g_free(install_dir);
 
 	return path;
 }
@@ -816,16 +810,21 @@ static void load_all_plugins(void)
 	gchar *path;
 
 	path = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
-	/* first load plugins in ~/.config/geany/plugins/, then in $prefix/lib/geany */
+	/* first load plugins in ~/.config/geany/plugins/ */
 	load_plugins_from_path(path);
 	g_free(path);
+
+	/* load plugins from a custom path */
+	if (NZV(prefs.custom_plugin_path))
+		load_plugins_from_path(prefs.custom_plugin_path);
+
+	/* finally load plugins from $prefix/lib/geany */
 #ifdef G_OS_WIN32
 	path = get_plugin_path();
 #else
 	path = g_strconcat(GEANY_LIBDIR, G_DIR_SEPARATOR_S "geany", NULL);
 #endif
 	load_plugins_from_path(path);
-
 	g_free(path);
 }
 
@@ -945,6 +944,8 @@ void plugins_init(void)
 
 	stash_group_add_toggle_button(group, &prefs.load_plugins,
 		"load_plugins", TRUE, "check_plugins");
+	stash_group_add_entry(group, &prefs.custom_plugin_path,
+		"custom_plugin_path", "", "extra_plugin_path_entry");
 
 	g_signal_connect(geany_object, "save-settings", G_CALLBACK(on_save_settings), NULL);
 	stash_group_add_string_vector(group, &active_plugins_pref, "active_plugins", NULL);
@@ -1041,8 +1042,7 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 
 	gtk_tree_model_get(GTK_TREE_MODEL(pm_widgets.store), &iter,
 		PLUGIN_COLUMN_CHECK, &old_state, PLUGIN_COLUMN_PLUGIN, &p, -1);
-	if (p == NULL)
-		return;
+	g_return_if_fail(p != NULL);
 	state = ! old_state; /* toggle the state */
 
 	/* save the filename of the plugin */
@@ -1119,9 +1119,10 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 	}
 	else
 	{
+		Plugin *p;
 		for (; list != NULL; list = list->next)
 		{
-			Plugin *p = list->data;
+			p = list->data;
 
 			gtk_list_store_append(store, &iter);
 			gtk_list_store_set(store, &iter,
@@ -1233,7 +1234,7 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	gtk_widget_set_name(pm_widgets.dialog, "GeanyDialog");
 	gtk_box_set_spacing(GTK_BOX(vbox), 6);
 
-	gtk_window_set_default_size(GTK_WINDOW(pm_widgets.dialog), 400, 350);
+	gtk_window_set_default_size(GTK_WINDOW(pm_widgets.dialog), 500, 450);
 
 	pm_widgets.tree = gtk_tree_view_new();
 	pm_widgets.store = gtk_list_store_new(
@@ -1291,58 +1292,4 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 }
 
 
-/** Insert a toolbar item before the Quit button, or after the previous plugin toolbar item.
- * A separator is added on the first call to this function, and will be shown when @a item is
- * shown; hidden when @a item is hidden.
- * @note You should still destroy @a item yourself, usually in @ref plugin_cleanup().
- * @param plugin Must be @ref geany_plugin.
- * @param item The item to add. */
-void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item)
-{
-	GtkToolbar *toolbar = GTK_TOOLBAR(main_widgets.toolbar);
-	gint pos;
-	GeanyAutoSeparator *autosep;
-
-	g_return_if_fail(plugin);
-	autosep = &plugin->priv->toolbar_separator;
-
-	if (!autosep->widget)
-	{
-		GtkToolItem *sep;
-
-		pos = toolbar_get_insert_position();
-
-		sep = gtk_separator_tool_item_new();
-		gtk_toolbar_insert(toolbar, sep, pos);
-		autosep->widget = GTK_WIDGET(sep);
-
-		gtk_toolbar_insert(toolbar, item, pos + 1);
-	}
-	else
-	{
-		pos = gtk_toolbar_get_item_index(toolbar, GTK_TOOL_ITEM(autosep->widget));
-		g_return_if_fail(pos >= 0);
-		gtk_toolbar_insert(toolbar, item, pos);
-	}
-	/* hide the separator widget if there are no toolbar items showing for the plugin */
-	ui_auto_separator_add_ref(autosep, GTK_WIDGET(item));
-}
-
-
-/** Ensures that a plugin's module (*.so) will never be unloaded.
- *  This is necessary if you register new GTypes in your plugin, e.g. when using own classes
- *  using the GObject system.
- *
- * @param plugin Must be @ref geany_plugin.
- *
- *  @since 0.16
- */
-void plugin_module_make_resident(GeanyPlugin *plugin)
-{
-	g_return_if_fail(plugin);
-
-	plugin->priv->resident = TRUE;
-}
-
-
 #endif
