@@ -1,8 +1,8 @@
 /*
  *      sciwrappers.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
  *      the Free Software Foundation; either version 2 of the License, or
@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 3000 2008-09-25 18:29:53Z eht16 $
+ * $Id: sciwrappers.c 4042 2009-07-29 17:54:48Z ntrel $
  */
 
 /** @file sciwrappers.h
@@ -46,8 +46,9 @@ void sci_set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_width)
 		gchar tmp_str[15];
 		gint len = SSM(sci, SCI_GETLINECOUNT, 0, 0);
 		gint width;
+
 		g_snprintf(tmp_str, 15, "_%d%d", len, extra_width);
-		width = SSM(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER, (sptr_t) tmp_str);
+		width = sci_text_width(sci, STYLE_LINENUMBER, tmp_str);
 		SSM (sci, SCI_SETMARGINWIDTHN, 0, width);
 		SSM (sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); /* use default behaviour */
 	}
@@ -59,7 +60,8 @@ void sci_set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_width)
 
 void sci_set_mark_long_lines(ScintillaObject * sci, gint type, gint column, const gchar *colour)
 {
-	if (column == 0) type = 2;
+	if (column == 0)
+		type = 2;
 	switch (type)
 	{
 		case 0:
@@ -83,15 +85,6 @@ void sci_set_mark_long_lines(ScintillaObject * sci, gint type, gint column, cons
 }
 
 
-gboolean sci_get_line_numbers(ScintillaObject * sci)
-{
-	gint margin_width;
-
-	margin_width = SSM(sci, SCI_GETMARGINWIDTHN, 0, 0);
-	if( margin_width > 0 ) return TRUE;
-	else return FALSE;
-}
-
 /* symbol margin visibility */
 void sci_set_symbol_margin(ScintillaObject * sci, gboolean set )
 {
@@ -107,12 +100,6 @@ void sci_set_symbol_margin(ScintillaObject * sci, gboolean set )
 	}
 }
 
-gboolean sci_get_symbol_margin(ScintillaObject *sci)
-{
-	if (SSM(sci, SCI_GETMARGINWIDTHN, 1, 0) > 0 ) return TRUE;
-	else return FALSE;
-}
-
 /* folding margin visibility */
 void sci_set_folding_margin_visible(ScintillaObject *sci, gboolean set)
 {
@@ -128,26 +115,12 @@ void sci_set_folding_margin_visible(ScintillaObject *sci, gboolean set)
 	}
 }
 
-gboolean sci_get_folding_margin_visible(ScintillaObject * sci)
-{
-	gint margin_width;
-
-	margin_width = SSM(sci, SCI_GETMARGINWIDTHN, 2, 0);
-	if( margin_width > 0 ) return TRUE;
-	else return FALSE;
-}
-
 /* end of lines */
 void sci_set_visible_eols(ScintillaObject* sci, gboolean set )
 {
 	SSM(sci,SCI_SETVIEWEOL,set,0);
 }
 
-gboolean sci_get_visible_eols(ScintillaObject* sci)
-{
-	return SSM( sci, SCI_GETVIEWEOL,0,0);
-}
-
 void sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
 {
 	if (set)
@@ -156,11 +129,6 @@ void sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
 		SSM(sci,SCI_SETVIEWWS,SCWS_INVISIBLE,0);
 }
 
-gboolean sci_get_visible_white_spaces(ScintillaObject* sci)
-{
-	return SSM( sci, SCI_GETVIEWWS,0,0);
-}
-
 void sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
 {
 	if (set)
@@ -169,11 +137,6 @@ void sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
 }
 
-gboolean sci_get_lines_wrapped(ScintillaObject* sci)
-{
-	return SSM( sci, SCI_GETWRAPMODE,0,0);
-}
-
 gint sci_get_eol_mode( ScintillaObject* sci)
 {
 	return SSM( sci, SCI_GETEOLMODE, 0, 0);
@@ -191,7 +154,8 @@ void sci_convert_eols( ScintillaObject* sci, gint eolmode)
 
 void sci_add_text(ScintillaObject* sci, const gchar* text)
 {
-	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
+	if (text != NULL)
+	{ /* if null text is passed to scintilla will segfault */
 		SSM( sci, SCI_ADDTEXT, strlen(text), (sptr_t) text);
 	}
 }
@@ -207,15 +171,6 @@ void sci_set_text(ScintillaObject* sci, const gchar* text)
 }
 
 
-void sci_add_text_buffer(ScintillaObject* sci, const gchar* text, gint len)
-{
-	if( text != NULL ){ /* if null text is passed to scintilla will segfault */
-		SSM(sci, SCI_CLEARALL, 0, 0);
-		SSM(sci, SCI_ADDTEXT, len, (sptr_t) text);
-	}
-}
-
-
 gboolean sci_can_undo( ScintillaObject* sci )
 {
 	return SSM( sci, SCI_CANUNDO, 0, 0);
@@ -232,10 +187,6 @@ void sci_undo( ScintillaObject* sci )
 {
 	if( sci_can_undo(sci) )
 		SSM( sci, SCI_UNDO, 0, 0);
-	else
-	{ /* change it to a document function */
-
-	}
 }
 
 
@@ -243,10 +194,6 @@ void sci_redo( ScintillaObject* sci )
 {
 	if( sci_can_redo( sci ) )
 		SSM( sci, SCI_REDO,0,0);
-	else
-	{ /* change it to a document function */
-
-	}
 }
 
 
@@ -274,12 +221,6 @@ void sci_set_undo_collection( ScintillaObject* sci, gboolean set )
 }
 
 
-gboolean sci_get_undo_collection( ScintillaObject* sci )
-{
-	return SSM( sci, SCI_GETUNDOCOLLECTION,0,0);
-}
-
-
 void sci_empty_undo_buffer( ScintillaObject* sci )
 {
 	SSM( sci, SCI_EMPTYUNDOBUFFER,0,0);
@@ -460,6 +401,12 @@ void sci_set_selection_end(ScintillaObject* sci, gint position)
 }
 
 
+void sci_set_selection(ScintillaObject *sci, gint anchorPos, gint currentPos)
+{
+	SSM(sci, SCI_SETSEL, anchorPos, currentPos);
+}
+
+
 gint sci_get_line_end_position(ScintillaObject* sci, gint line)
 {
 	return SSM(sci, SCI_GETLINEENDPOSITION, line, 0);
@@ -542,7 +489,6 @@ gint sci_get_line_length(ScintillaObject* sci,gint line)
 }
 
 
-/* TODO: rename/change to use buffer? Otherwise inconsistent with sci_get_text*(). */
 /** Get line contents.
  * @param sci Scintilla widget.
  * @param line_num Line number.
@@ -559,6 +505,9 @@ gchar *sci_get_line(ScintillaObject* sci, gint line_num)
 
 
 /** Get all text.
+ * @deprecated sci_get_text is deprecated and should not be used in newly-written code.
+ * Use sci_get_contents() instead.
+ *
  * @param sci Scintilla widget.
  * @param len Length of @a text buffer, usually sci_get_length() + 1.
  * @param text Text buffer; must be allocated @a len + 1 bytes for null-termination. */
@@ -567,8 +516,25 @@ void sci_get_text(ScintillaObject* sci, gint len, gchar* text)
 	SSM( sci, SCI_GETTEXT, len, (sptr_t) text );
 }
 
+/** Get all text inside a given text length.
+ * @param sci Scintilla widget.
+ * @param len Length of the text to retrieve from the start of the document,
+ *            usually sci_get_length() + 1.
+ * @return A copy of the text. Should be freed when no longer needed.
+ *
+ * @since 0.17
+ */
+gchar *sci_get_contents(ScintillaObject *sci, gint len)
+{
+	gchar *text = g_malloc(len);
+	SSM(sci, SCI_GETTEXT, len, (sptr_t) text);
+	return text;
+}
 
 /** Get selected text.
+ * @deprecated sci_get_selected_text is deprecated and should not be used in newly-written code.
+ * Use sci_get_selection_contents() instead.
+ *
  * @param sci Scintilla widget.
  * @param text Text buffer; must be allocated sci_get_selected_text_length() + 1 bytes
  * for null-termination. */
@@ -577,6 +543,22 @@ void sci_get_selected_text(ScintillaObject* sci, gchar* text)
 	SSM( sci, SCI_GETSELTEXT, 0, (sptr_t) text);
 }
 
+/** Get selected text.
+ * @param sci Scintilla widget.
+ *
+ * @return The selected text. Should be freed when no longer needed.
+ *
+ * @since 0.17
+ */
+gchar *sci_get_selection_contents(ScintillaObject *sci)
+{
+	gint len = sci_get_selected_text_length(sci);
+	gchar *selection = g_malloc(len + 1);
+
+	SSM(sci, SCI_GETSELTEXT, 0, (sptr_t) selection);
+
+	return selection;
+}
 
 /** Get selected text length.
  * @param sci Scintilla widget.
@@ -586,7 +568,6 @@ gint sci_get_selected_text_length(ScintillaObject* sci)
 	return SSM( sci, SCI_GETSELTEXT, 0, 0);
 }
 
-
 gint sci_get_position_from_xy(ScintillaObject* sci, gint x, gint y, gboolean nearby)
 {
 	/* for nearby return -1 if there is no character near to the x,y point. */
@@ -594,13 +575,6 @@ gint sci_get_position_from_xy(ScintillaObject* sci, gint x, gint y, gboolean nea
 }
 
 
-void sci_get_xy_from_position(ScintillaObject* sci,gint pos, gint* x, gint* y)
-{
-	*x = SSM(sci, SCI_POINTXFROMPOSITION,0, (int) pos);
- 	*y = SSM(sci, SCI_POINTYFROMPOSITION,0, (int) pos);
-}
-
-
 /** Check if a line is visible (folding may have hidden it).
  * @param sci Scintilla widget.
  * @param line Line number.
@@ -626,13 +600,6 @@ gint sci_get_fold_level(ScintillaObject* sci, gint line)
 }
 
 
-/* Get the next line after start_line with fold level <= level */
-gint sci_get_last_child(ScintillaObject* sci, gint start_line, gint level)
-{
-	return SSM( sci, SCI_GETLASTCHILD, start_line, level);
-}
-
-
 /* Get the line number of the fold point before start_line, or -1 if there isn't one */
 gint sci_get_fold_parent(ScintillaObject* sci, gint start_line)
 {
@@ -658,12 +625,6 @@ void sci_colourise( ScintillaObject* sci, gint start, gint end)
 }
 
 
-void sci_set_lexer(ScintillaObject * sci, gint lexer)
-{
-	SSM( sci, SCI_SETLEXER, lexer, 0);
-}
-
-
 void sci_clear_all(ScintillaObject * sci)
 {
 	SSM( sci, SCI_CLEARALL, 0, 0);
@@ -684,7 +645,10 @@ void sci_set_tab_width(ScintillaObject * sci, gint width)
 
 /** Get display tab width (this is not indent width, see GeanyIndentPrefs).
  * @param sci Scintilla widget.
- * @return Width. */
+ * @return Width.
+ *
+ * @since 0.15
+ **/
 gint sci_get_tab_width(ScintillaObject * sci)
 {
 	return SSM(sci, SCI_GETTABWIDTH, 0, 0);
@@ -720,7 +684,10 @@ void sci_use_popup(ScintillaObject *sci, gboolean enable)
 
 /** Check if there's a selection.
  * @param sci Scintilla widget.
- * @return Whether a selection is present. */
+ * @return Whether a selection is present.
+ *
+ * @since 0.15
+ **/
 gboolean sci_has_selection(ScintillaObject *sci)
 {
 	if (SSM(sci, SCI_GETSELECTIONEND,0,0) - SSM(sci, SCI_GETSELECTIONSTART,0,0))
@@ -743,8 +710,12 @@ void sci_set_search_anchor(ScintillaObject *sci)
 }
 
 
+/* removes a selection if pos < 0 */
 void sci_set_anchor(ScintillaObject *sci, gint pos)
 {
+	if (pos < 0)
+		pos = sci_get_current_position(sci);
+
 	SSM(sci, SCI_SETANCHOR, pos, 0);
 }
 
@@ -763,6 +734,12 @@ void sci_scroll_lines(ScintillaObject *sci, gint lines)
 }
 
 
+void sci_scroll_columns(ScintillaObject *sci, gint columns)
+{
+	SSM(sci, SCI_LINESCROLL, columns, 0);
+}
+
+
 gint sci_search_next(ScintillaObject *sci, gint flags, const gchar *text)
 {
 	return SSM(sci, SCI_SEARCHNEXT, flags, (sptr_t) text );
@@ -775,7 +752,7 @@ gint sci_search_prev(ScintillaObject *sci, gint flags, const gchar *text)
 }
 
 
-gint sci_find_text(ScintillaObject *sci, gint flags, struct TextToFind *ttf)
+gint sci_find_text(ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf)
 {
 	return SSM(sci, SCI_FINDTEXT, flags, (long) ttf );
 }
@@ -830,19 +807,38 @@ void sci_clear_cmdkey(ScintillaObject *sci, gint key)
 
 
 /** Get text between @a start and @a end.
+ * @deprecated sci_get_text_range is deprecated and should not be used in newly-written code.
+ * Use sci_get_contents_range() instead.
+ *
  * @param sci Scintilla widget.
  * @param start Start.
  * @param end End.
  * @param text Text will be zero terminated and must be allocated (end - start + 1) bytes. */
 void sci_get_text_range(ScintillaObject *sci, gint start, gint end, gchar *text)
 {
-	struct TextRange tr;
+	struct Sci_TextRange tr;
 	tr.chrg.cpMin = start;
 	tr.chrg.cpMax = end;
 	tr.lpstrText = text;
 	SSM(sci, SCI_GETTEXTRANGE, 0, (long) &tr);
 }
 
+/** Get text between @a start and @a end.
+ *
+ * @param sci Scintilla widget.
+ * @param start Start.
+ * @param end End.
+ * @return The text inside the given range. Should be freed when no longer needed.
+ *
+ * @since 0.17
+ */
+gchar *sci_get_contents_range(ScintillaObject *sci, gint start, gint end)
+{
+	gchar *text = g_malloc((end - start) + 1);
+	sci_get_text_range(sci, start, end, text);
+	return text;
+}
+
 
 void sci_line_duplicate(ScintillaObject *sci)
 {
@@ -866,37 +862,25 @@ void sci_insert_text(ScintillaObject *sci, gint pos, const gchar *text)
 }
 
 
-void sci_grab_focus(ScintillaObject *sci)
-{
-	SSM(sci, SCI_GRABFOCUS, 0, 0);
-}
-
-
-void sci_set_cursor(ScintillaObject *sci, gint cursor)
-{
-	SSM(sci, SCI_SETCURSOR, cursor, 0);
-}
-
-
 void sci_target_from_selection(ScintillaObject *sci)
 {
 	SSM(sci, SCI_TARGETFROMSELECTION, 0, 0);
 }
 
 
- void sci_target_start(ScintillaObject *sci, gint start)
+void sci_set_target_start(ScintillaObject *sci, gint start)
 {
 	SSM(sci, SCI_SETTARGETSTART, start, 0);
 }
 
 
-void sci_target_end(ScintillaObject *sci, gint end)
+void sci_set_target_end(ScintillaObject *sci, gint end)
 {
 	SSM(sci, SCI_SETTARGETEND, end, 0);
 }
 
 
-gint sci_target_replace(ScintillaObject *sci, const gchar *text, gboolean regex)
+gint sci_replace_target(ScintillaObject *sci, const gchar *text, gboolean regex)
 {
 	return SSM(sci, (regex) ? SCI_REPLACETARGETRE : SCI_REPLACETARGET, (uptr_t) -1, (sptr_t) text);
 }
@@ -912,16 +896,14 @@ void sci_set_readonly(ScintillaObject *sci, gboolean readonly)
 	SSM(sci, SCI_SETREADONLY, readonly, 0);
 }
 
-gboolean sci_get_readonly(ScintillaObject *sci)
-{
-	return SSM(sci, SCI_GETREADONLY, 0, 0);
-}
-
 /** A simple convenience function for sending Scintilla commands without any parameters.
  * @param sci The Scintilla @c GtkWidget.
  * @param cmd @c SCI_COMMAND.
- * @see http://scintilla.org for the documentation. */
-void sci_cmd(ScintillaObject * sci, gint cmd)
+ * @see http://scintilla.org for the documentation.
+ *
+ *  @since 0.16
+ */
+void sci_send_command(ScintillaObject * sci, gint cmd)
 {
 	SSM(sci, cmd, 0, 0);
 }
@@ -955,24 +937,18 @@ gint sci_get_first_visible_line(ScintillaObject *sci)
 }
 
 
-void sci_set_styling(ScintillaObject *sci, gint len, gint style)
-{
-	if (len < 0 || style < 0) return;
-
-	SSM(sci, SCI_SETSTYLING, len, style);
-}
-
-void sci_start_styling(ScintillaObject *sci, gint pos, gint mask)
-{
-	SSM(sci, SCI_STARTSTYLING, pos, mask);
-}
-
-gint sci_get_indicator(ScintillaObject *sci)
-{
-	return SSM(sci, SCI_GETINDICATORCURRENT, 0, 0);
-}
-
-void sci_set_indicator(ScintillaObject *sci, gint indic)
+/**
+ *  Set the current indicator. This is necessary to define an indicator for a range of text or
+ *  clearing indicators for a range of text.
+ *
+ *  @param sci Scintilla widget.
+ *  @param indic The indicator number to set.
+ *
+ *  @see sci_indicator_clear
+ *
+ *  @since 0.16
+ */
+void sci_indicator_set(ScintillaObject *sci, gint indic)
 {
 	SSM(sci, SCI_SETINDICATORCURRENT, indic, 0);
 }
@@ -982,10 +958,18 @@ void sci_indicator_fill(ScintillaObject *sci, gint pos, gint len)
 	SSM(sci, SCI_INDICATORFILLRANGE, pos, len);
 }
 
-/** Clear a range of text from any set indicators. Starting at @a pos, @a len characters long.
- * @param sci Scintilla widget.
- * @param pos Starting position.
- * @param len Length. */
+/**
+ *  Clear a range of text from the currently set indicator.
+ *  Starting at @a pos, @a len characters long.
+ *  In order to make this function properly, you need to set the current indicator before with
+ *  @ref sci_indicator_set().
+ *
+ *  @param sci Scintilla widget.
+ *  @param pos Starting position.
+ *  @param len Length.
+ *
+ *  @since 0.16
+ */
 void sci_indicator_clear(ScintillaObject *sci, gint pos, gint len)
 {
 	SSM(sci, SCI_INDICATORCLEARRANGE, pos, len);
@@ -1009,7 +993,10 @@ void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
 /** Find a matching brace at @a pos.
  * @param sci Scintilla widget.
  * @param pos Position.
- * @return Matching brace position. */
+ * @return Matching brace position.
+ *
+ * @since 0.15
+ **/
 gint sci_find_matching_brace(ScintillaObject *sci, gint pos)
 {
 	return SSM(sci, SCI_BRACEMATCH, pos, 0);
@@ -1102,4 +1089,17 @@ gint sci_get_position_after(ScintillaObject *sci, gint start)
 	return SSM(sci, SCI_POSITIONAFTER, start, 0);
 }
 
+void sci_lines_split(ScintillaObject *sci, gint pixelWidth)
+{
+	SSM(sci, SCI_LINESSPLIT, pixelWidth, 0);
+}
 
+void sci_lines_join(ScintillaObject *sci)
+{
+	SSM(sci, SCI_LINESJOIN, 0, 0);
+}
+
+gint sci_text_width(ScintillaObject *sci, gint styleNumber, const gchar *text)
+{
+	return SSM(sci, SCI_TEXTWIDTH, styleNumber, (sptr_t) text);
+}
