@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 3629 2009-03-16 13:43:12Z eht16 $
+ * $Id: search.c 3974 2009-07-16 14:37:35Z ntrel $
  */
 
 /*
@@ -83,8 +83,13 @@ static struct
 {
 	gint fif_mode;
 	gchar *fif_extra_options;
+	gboolean fif_case_sensitive;
+	gboolean fif_match_whole_word;
+	gboolean fif_invert_results;
+	gboolean fif_recursive;
+	gboolean fif_use_extra_options;
 }
-settings = {0, NULL};
+settings = {0, NULL, FALSE, FALSE, FALSE, FALSE, FALSE};
 
 static GeanyPrefGroup *fif_prefs = NULL;
 
@@ -171,6 +176,16 @@ static void init_prefs(void)
 		NULL);
 	stash_group_add_entry(group, &settings.fif_extra_options,
 		"fif_extra_options", "", "entry_extra");
+	stash_group_add_toggle_button(group, &settings.fif_case_sensitive,
+		"fif_case_sensitive", TRUE, "check_case");
+	stash_group_add_toggle_button(group, &settings.fif_match_whole_word,
+		"fif_match_whole_word", FALSE, "check_wholeword");
+	stash_group_add_toggle_button(group, &settings.fif_invert_results,
+		"fif_invert_results", FALSE, "check_invert");
+	stash_group_add_toggle_button(group, &settings.fif_recursive,
+		"fif_recursive", FALSE, "check_recursive");
+	stash_group_add_toggle_button(group, &settings.fif_use_extra_options,
+		"fif_use_extra_options", FALSE, "check_extra");
 }
 
 
@@ -944,18 +959,20 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 }
 
 
-static gint search_mark(GeanyDocument *doc, const gchar *search_text, gint flags)
+/* @return Number of matches marked. */
+gint search_mark_all(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gint pos, count = 0;
 	gsize len;
-	struct TextToFind ttf;
+	struct Sci_TextToFind ttf;
 
 	g_return_val_if_fail(doc != NULL, 0);
 
 	/* clear previous search indicators */
 	editor_indicator_clear(doc->editor, GEANY_INDICATOR_SEARCH);
 
-	len = strlen(search_text);
+	if (!NZV(search_text))
+		return 0;
 
 	ttf.chrg.cpMin = 0;
 	ttf.chrg.cpMax = sci_get_length(doc->editor->sci);
@@ -965,7 +982,9 @@ static gint search_mark(GeanyDocument *doc, const gchar *search_text, gint flags
 		pos = sci_find_text(doc->editor->sci, flags, &ttf);
 		if (pos == -1) break;
 
-		editor_indicator_set_on_range(doc->editor, GEANY_INDICATOR_SEARCH, pos, pos + len);
+		len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
+		if (len)
+			editor_indicator_set_on_range(doc->editor, GEANY_INDICATOR_SEARCH, pos, pos + len);
 
 		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
 		count++;
@@ -1059,7 +1078,7 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 
 			case GEANY_RESPONSE_MARK:
 			{
-				gint count = search_mark(doc, search_data.text, search_data.flags);
+				gint count = search_mark_all(doc, search_data.text, search_data.flags);
 
 				if (count == 0)
 					ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.text);
@@ -1590,7 +1609,7 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gchar *buffer, *short_file_name;
-	struct TextToFind ttf;
+	struct Sci_TextToFind ttf;
 	gint count = 0;
 	gint prev_line = -1;
 
