@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.c 3571 2009-02-10 21:11:25Z eht16 $
+ * $Id: templates.c 4088 2009-08-15 17:33:20Z eht16 $
  */
 
 /*
@@ -37,6 +37,8 @@
 #include "document.h"
 #include "filetypes.h"
 #include "ui_utils.h"
+#include "toolbar.h"
+#include "geanymenubuttonaction.h"
 
 
 GeanyTemplatePrefs template_prefs;
@@ -216,14 +218,16 @@ end\n\
 x = StdClass.new\n\
 ";
 
-static const gchar templates_filetype_python[] = "#!/usr/bin/env python\n#\n\
+static const gchar templates_filetype_python[] = "#!/usr/bin/env python\n\
+# -*- coding: utf-8 -*-\n#\n\
 {fileheader}\n\n\
 \n\
 def main():\n\
 	\n\
 	return 0\n\
 \n\
-if __name__ == '__main__': main()\n\
+if __name__ == '__main__':\n\
+	main()\n\
 ";
 
 static const gchar templates_filetype_latex[] = "\
@@ -246,14 +250,29 @@ static gchar *ft_templates[GEANY_MAX_BUILT_IN_FILETYPES] = {NULL};
 	g_strconcat(app->configdir, \
 		G_DIR_SEPARATOR_S GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S, shortname, NULL)
 
-#define TEMPLATES_CREATE_FILE(fname, text)	\
-	if (! g_file_test(fname, G_FILE_TEST_EXISTS)) \
-		utils_write_file(fname, text)
-
 #define TEMPLATES_READ_FILE(fname, contents_ptr) \
 	g_file_get_contents(fname, contents_ptr, NULL, NULL);
 
 
+static void create_template_file_if_necessary(const gchar *filename, const gchar *content)
+{
+	if (! g_file_test(filename, G_FILE_TEST_EXISTS))
+	{
+		if (file_prefs.default_eol_character != SC_EOL_LF)
+		{
+			/* Replace the \n characters in the default template text by the proper
+			 * platform-specific line ending characters. */
+			GString *tmp = g_string_new(content);
+			const gchar *eol_str = (file_prefs.default_eol_character == SC_EOL_CR) ? "\r" : "\r\n";
+
+			utils_string_replace_all(tmp, "\n", eol_str);
+			utils_write_file(filename, tmp->str);
+			g_string_free(tmp, TRUE);
+		}
+		else
+			utils_write_file(filename, content);
+	}
+}
 
 /* FIXME the callers should use GStrings instead of char arrays */
 static gchar *replace_all(gchar *text, const gchar *year, const gchar *date, const gchar *datetime)
@@ -281,11 +300,11 @@ static void init_general_templates(const gchar *year, const gchar *date, const g
 	gchar *template_filename_changelog = TEMPLATES_GET_FILENAME("changelog");
 
 	/* create the template files in the configuration directory, if they don't exist */
-	TEMPLATES_CREATE_FILE(template_filename_fileheader, templates_fileheader);
-	TEMPLATES_CREATE_FILE(template_filename_gpl, templates_gpl_notice);
-	TEMPLATES_CREATE_FILE(template_filename_bsd, templates_bsd_notice);
-	TEMPLATES_CREATE_FILE(template_filename_function, templates_function_description);
-	TEMPLATES_CREATE_FILE(template_filename_changelog, templates_changelog);
+	create_template_file_if_necessary(template_filename_fileheader, templates_fileheader);
+	create_template_file_if_necessary(template_filename_gpl, templates_gpl_notice);
+	create_template_file_if_necessary(template_filename_bsd, templates_bsd_notice);
+	create_template_file_if_necessary(template_filename_function, templates_function_description);
+	create_template_file_if_necessary(template_filename_changelog, templates_changelog);
 
 	/* read the contents */
 	TEMPLATES_READ_FILE(template_filename_fileheader, &templates[GEANY_TEMPLATE_FILEHEADER]);
@@ -324,17 +343,39 @@ static void init_ft_templates(const gchar *year, const gchar *date, const gchar
 
 		switch (ft_id)
 		{
-			case GEANY_FILETYPES_NONE:	TEMPLATES_CREATE_FILE(fname, templates_filetype_none); break;
-			case GEANY_FILETYPES_C:		TEMPLATES_CREATE_FILE(fname, templates_filetype_c); break;
-			case GEANY_FILETYPES_CPP:	TEMPLATES_CREATE_FILE(fname, templates_filetype_cpp); break;
-			case GEANY_FILETYPES_D:		TEMPLATES_CREATE_FILE(fname, templates_filetype_d); break;
-			case GEANY_FILETYPES_JAVA:	TEMPLATES_CREATE_FILE(fname, templates_filetype_java); break;
-			case GEANY_FILETYPES_PASCAL:TEMPLATES_CREATE_FILE(fname, templates_filetype_pascal); break;
-			case GEANY_FILETYPES_PHP:	TEMPLATES_CREATE_FILE(fname, templates_filetype_php); break;
-			case GEANY_FILETYPES_HTML:	TEMPLATES_CREATE_FILE(fname, templates_filetype_html); break;
-			case GEANY_FILETYPES_RUBY:	TEMPLATES_CREATE_FILE(fname, templates_filetype_ruby); break;
-			case GEANY_FILETYPES_PYTHON:    TEMPLATES_CREATE_FILE(fname, templates_filetype_python); break;
-			case GEANY_FILETYPES_LATEX: TEMPLATES_CREATE_FILE(fname, templates_filetype_latex); break;
+			case GEANY_FILETYPES_NONE:
+				create_template_file_if_necessary(fname, templates_filetype_none);
+				break;
+			case GEANY_FILETYPES_C:
+				create_template_file_if_necessary(fname, templates_filetype_c);
+				break;
+			case GEANY_FILETYPES_CPP:
+				create_template_file_if_necessary(fname, templates_filetype_cpp);
+				break;
+			case GEANY_FILETYPES_D:
+				create_template_file_if_necessary(fname, templates_filetype_d);
+				break;
+			case GEANY_FILETYPES_JAVA:
+				create_template_file_if_necessary(fname, templates_filetype_java);
+				break;
+			case GEANY_FILETYPES_PASCAL:
+				create_template_file_if_necessary(fname, templates_filetype_pascal);
+				break;
+			case GEANY_FILETYPES_PHP:
+				create_template_file_if_necessary(fname, templates_filetype_php);
+				break;
+			case GEANY_FILETYPES_HTML:
+				create_template_file_if_necessary(fname, templates_filetype_html);
+				break;
+			case GEANY_FILETYPES_RUBY:
+				create_template_file_if_necessary(fname, templates_filetype_ruby);
+				break;
+			case GEANY_FILETYPES_PYTHON:
+				create_template_file_if_necessary(fname, templates_filetype_python);
+				break;
+			case GEANY_FILETYPES_LATEX:
+				create_template_file_if_necessary(fname, templates_filetype_latex);
+				break;
 			default: break;
 		}
 		TEMPLATES_READ_FILE(fname, &ft_templates[ft_id]);
@@ -360,33 +401,28 @@ on_new_with_template                   (GtkMenuItem     *menuitem,
 
 
 /* template items for the new file menu */
-static void create_new_menu_items(void)
+static void create_new_menu_items(GtkWidget *toolbar_new_file_menu)
 {
-	filetype_id ft_id;
+	GSList *node;
 
-	for (ft_id = 0; ft_id < GEANY_MAX_BUILT_IN_FILETYPES; ft_id++)
+	foreach_slist(node, filetypes_by_title)
 	{
+		GeanyFiletype *ft = node->data;
 		GtkWidget *tmp_menu, *tmp_button;
-		GeanyFiletype *ft = filetypes[ft_id];
 		const gchar *label = ft->title;
 
-		if (ft_templates[ft_id] == NULL)
+		if (ft_templates[ft->id] == NULL)
 			continue;
-		if (ft_id == GEANY_FILETYPES_NONE)
-			label = _("None");
 
 		tmp_menu = gtk_menu_item_new_with_label(label);
 		gtk_widget_show(tmp_menu);
 		gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
 		g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_template), ft);
 
-		if (ui_widgets.new_file_menu != NULL)
-		{
-			tmp_button = gtk_menu_item_new_with_label(label);
-			gtk_widget_show(tmp_button);
-			gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), tmp_button);
-			g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_template), ft);
-		}
+		tmp_button = gtk_menu_item_new_with_label(label);
+		gtk_widget_show(tmp_button);
+		gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), tmp_button);
+		g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_template), ft);
 	}
 }
 
@@ -445,6 +481,7 @@ on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_dat
 static void add_file_item(gpointer data, gpointer user_data)
 {
 	GtkWidget *tmp_menu, *tmp_button;
+	GtkWidget *toolbar_new_file_menu = user_data;
 	gchar *label;
 
 	g_return_if_fail(data);
@@ -456,13 +493,11 @@ static void add_file_item(gpointer data, gpointer user_data)
 	gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
 	g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_file_template), NULL);
 
-	if (ui_widgets.new_file_menu != NULL)
-	{
-		tmp_button = gtk_menu_item_new_with_label(label);
-		gtk_widget_show(tmp_button);
-		gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), tmp_button);
-		g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_file_template), NULL);
-	}
+	tmp_button = gtk_menu_item_new_with_label(label);
+	gtk_widget_show(tmp_button);
+	gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), tmp_button);
+	g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_file_template), NULL);
+
 	g_free(label);
 }
 
@@ -473,12 +508,12 @@ static gint compare_filenames_by_filetype(gconstpointer a, gconstpointer b)
 	GeanyFiletype *ft_b = filetypes_detect_from_extension(b);
 
 	/* sort by filetype name first */
-	if (ft_a != ft_b)
+	if (G_LIKELY(ft_a != ft_b))
 	{
 		/* None filetypes should come first */
-		if (ft_a->id == GEANY_FILETYPES_NONE)
+		if (G_UNLIKELY(ft_a->id == GEANY_FILETYPES_NONE))
 			return -1;
-		if (ft_b->id == GEANY_FILETYPES_NONE)
+		if (G_UNLIKELY(ft_b->id == GEANY_FILETYPES_NONE))
 			return 1;
 
 		return utils_str_casecmp(ft_a->name, ft_b->name);
@@ -487,7 +522,7 @@ static gint compare_filenames_by_filetype(gconstpointer a, gconstpointer b)
 }
 
 
-static gboolean add_custom_template_items(void)
+static gboolean add_custom_template_items(GtkWidget *toolbar_new_file_menu)
 {
 	gchar *path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
 		"files", NULL);
@@ -499,7 +534,7 @@ static gboolean add_custom_template_items(void)
 		return FALSE;
 	}
 	list = g_slist_sort(list, compare_filenames_by_filetype);
-	g_slist_foreach(list, add_file_item, NULL);
+	g_slist_foreach(list, add_file_item, toolbar_new_file_menu);
 	g_slist_foreach(list, (GFunc) g_free, NULL);
 	g_slist_free(list);
 	g_free(path);
@@ -510,23 +545,28 @@ static gboolean add_custom_template_items(void)
 static void create_file_template_menus(void)
 {
 	GtkWidget *sep1, *sep2 = NULL;
+	GtkWidget *toolbar_new_file_menu = NULL;
 
 	new_with_template_menu = ui_lookup_widget(main_widgets.window, "menu_new_with_template1_menu");
-	create_new_menu_items();
+	toolbar_new_file_menu = gtk_menu_new();
+	/* we hold our own ref on the menu in case it is not used in the toolbar */
+	g_object_ref(toolbar_new_file_menu);
+
+	create_new_menu_items(toolbar_new_file_menu);
 
 	sep1 = gtk_separator_menu_item_new();
 	gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep1);
-	if (ui_widgets.new_file_menu != NULL)
-	{
-		sep2 = gtk_separator_menu_item_new();
-		gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), sep2);
-	}
-	if (add_custom_template_items())
+	sep2 = gtk_separator_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), sep2);
+
+	if (add_custom_template_items(toolbar_new_file_menu))
 	{
 		gtk_widget_show(sep1);
-		if (sep2 != NULL)
-			gtk_widget_show(sep2);
+		gtk_widget_show(sep2);
 	}
+
+	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(
+		toolbar_get_action_by_name("New")), toolbar_new_file_menu);
 }
 
 
@@ -553,117 +593,46 @@ void templates_init(void)
  * 6 characters are filled with whitespace when the comment characters include " *" */
 static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, guint indent)
 {
-	gchar *frame_start = "";	/* to add before comment_text */
-	gchar *frame_end = "";		/* to add after comment_text */
-	gchar *line_prefix;			/* to add before every line in comment_text */
+	gchar *frame_start;		/* to add before comment_text */
+	gchar *frame_end;		/* to add after comment_text */
+	gchar *line_prefix;		/* to add before every line in comment_text */
 	gchar *result;
 	gchar *tmp;
 	gchar *prefix;
 	gchar **lines;
 	guint i, len;
+	GeanyFiletype *ft = filetypes_index(filetype_idx);
 
-	/* TODO the following switch could be replaced by some intelligent code which reads
-	 * frame_start, frame_end and line_prefix from the filetype definition files */
-	switch (filetype_idx)
+	g_return_val_if_fail(ft != NULL, NULL);
+
+	if (NZV(ft->comment_open))
 	{
-		case GEANY_FILETYPES_HTML:
-		case GEANY_FILETYPES_XML:
-		case GEANY_FILETYPES_DOCBOOK:
+		if (NZV(ft->comment_close))
 		{
-			frame_start = "<!--\n";
-			frame_end = "-->\n";
+			frame_start = g_strconcat(ft->comment_open, "\n", NULL);
+			frame_end = g_strconcat(ft->comment_close, "\n", NULL);
 			line_prefix = "";
-			break;
 		}
-
-		case GEANY_FILETYPES_PYTHON:
-		case GEANY_FILETYPES_R:
-		case GEANY_FILETYPES_RUBY:
-		case GEANY_FILETYPES_SH:
-		case GEANY_FILETYPES_MAKE:
-		case GEANY_FILETYPES_PERL:
-		case GEANY_FILETYPES_DIFF:
-		case GEANY_FILETYPES_TCL:
-		case GEANY_FILETYPES_CONF:
-		case GEANY_FILETYPES_PO:
-		case GEANY_FILETYPES_YAML:
-		case GEANY_FILETYPES_CMAKE:
+		else
 		{
-			line_prefix = "#";
-			break;
+			frame_start = NULL;
+			frame_end = NULL;
+			line_prefix = ft->comment_open;
 		}
+	}
+	else
+	{	/* use C-like multi-line comments as fallback */
+		frame_start = g_strdup("/*\n");
+		frame_end = g_strdup("*/\n");
+		line_prefix = "";
+	}
 
-		case GEANY_FILETYPES_JS:
-		case GEANY_FILETYPES_HAXE:
-		{
-			line_prefix = "//";
-			break;
-		}
-
-		case GEANY_FILETYPES_LATEX:
-		case GEANY_FILETYPES_MATLAB:
-		{
-			line_prefix = "%";
-			break;
-		}
-
-		case GEANY_FILETYPES_ADA:
-		case GEANY_FILETYPES_HASKELL:
-		case GEANY_FILETYPES_VHDL:
-		case GEANY_FILETYPES_LUA:
-		{
-			line_prefix = "--";
-			break;
-		}
-
-		case GEANY_FILETYPES_F77:
-		case GEANY_FILETYPES_FORTRAN:
-		{
-			line_prefix = "c";
-			break;
-		}
-
-		case GEANY_FILETYPES_ASM:
-		case GEANY_FILETYPES_NSIS:
-		{
-			line_prefix = ";";
-			break;
-		}
-
-		case GEANY_FILETYPES_BASIC:
-		{
-			line_prefix = "'";
-			break;
-		}
-
-		case GEANY_FILETYPES_PASCAL:
-		{
-			frame_start = "{\n";
-			frame_end = "}\n";
-			line_prefix = "";
-			break;
-		}
-
-		case GEANY_FILETYPES_CAML:
-		{
-			frame_start = "(*\n";
-			frame_end = " *)\n";
-			line_prefix = " *";
-			break;
-		}
-
-		case GEANY_FILETYPES_NONE:
-		{
-			line_prefix = "";
-			break;
-		}
-
-		default: /* assume C-like multi-line comment is appropriate */
-		{
-			frame_start = "/*\n";
-			frame_end = " */\n";
-			line_prefix = " *";
-		}
+	/* do some magic to nicely format C-like multi-line comments */
+	if (NZV(frame_start) && frame_start[1] == '*')
+	{
+		/* prefix the string with a space */
+		setptr(frame_end, g_strconcat(" ", frame_end, NULL));
+		line_prefix = " *";
 	}
 
 	/* construct the real prefix with given amount of whitespace */
@@ -672,7 +641,6 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 	prefix = g_strconcat(line_prefix, tmp, NULL);
 	g_free(tmp);
 
-
 	/* add line_prefix to every line of comment_text */
 	lines = g_strsplit(comment_text, "\n", -1);
 	len = g_strv_length(lines) - 1;
@@ -685,10 +653,12 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 	tmp = g_strjoinv("\n", lines);
 
 	/* add frame_start and frame_end */
-	result = g_strconcat(frame_start, tmp, frame_end, NULL);
+	if (frame_start != NULL)
+		result = g_strconcat(frame_start, tmp, frame_end, NULL);
+	else
+		result = g_strconcat(tmp, frame_end, NULL);
 
-	g_free(prefix);
-	g_free(tmp);
+	utils_free_pointers(4, prefix, tmp, frame_start, frame_end, NULL);
 	g_strfreev(lines);
 	return result;
 }
@@ -724,10 +694,10 @@ gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 	else
 		shortname = g_path_get_basename(fname);
 
-	template = utils_str_replace(template, "{filename}", shortname);
-	template = utils_str_replace(template, "{gpl}", templates[GEANY_TEMPLATE_GPL]);
-	template = utils_str_replace(template, "{bsd}", templates[GEANY_TEMPLATE_BSD]);
-	template = utils_str_replace(template, "{datetime}", date);
+	utils_str_replace_all(&template, "{filename}", shortname);
+	utils_str_replace_all(&template, "{gpl}", templates[GEANY_TEMPLATE_GPL]);
+	utils_str_replace_all(&template, "{bsd}", templates[GEANY_TEMPLATE_BSD]);
+	utils_str_replace_all(&template, "{datetime}", date);
 
 	result = make_comment_block(template, ft_id, 8);
 
@@ -756,7 +726,7 @@ gchar *templates_get_template_new_file(GeanyFiletype *ft)
 
 	file_header = templates_get_template_fileheader(ft->id, NULL);	/* file template only used for new files */
 	ft_template = get_file_template(ft);
-	ft_template = utils_str_replace(ft_template, "{fileheader}", file_header);
+	utils_str_replace_all(&ft_template, "{fileheader}", file_header);
 	g_free(file_header);
 	return ft_template;
 }
@@ -775,9 +745,9 @@ gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name
 	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 	gchar *result;
 
-	template = utils_str_replace(template, "{date}", date);
-	template = utils_str_replace(template, "{datetime}", datetime);
-	template = utils_str_replace(template, "{functionname}", (func_name) ? func_name : "");
+	utils_str_replace_all(&template, "{date}", date);
+	utils_str_replace_all(&template, "{datetime}", datetime);
+	utils_str_replace_all(&template, "{functionname}", (func_name) ? func_name : "");
 
 	result = make_comment_block(template, filetype_idx, 3);
 
@@ -792,7 +762,8 @@ gchar *templates_get_template_changelog(void)
 {
 	gchar *date = utils_get_date_time(template_prefs.datetime_format, NULL);
 	gchar *result = g_strdup(templates[GEANY_TEMPLATE_CHANGELOG]);
-	result = utils_str_replace(result, "{date}", date);
+
+	utils_str_replace_all(&result, "{date}", date);
 
 	g_free(date);
 	return result;
@@ -803,6 +774,8 @@ void templates_free_templates(void)
 {
 	gint i;
 	GList *children, *item;
+	GtkWidget *toolbar_new_file_menu = geany_menu_button_action_get_menu(
+					GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")));
 
 	for (i = 0; i < GEANY_MAX_TEMPLATES; i++)
 	{
@@ -813,16 +786,14 @@ void templates_free_templates(void)
 		g_free(ft_templates[i]);
 	}
 	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
-	if (ui_widgets.new_file_menu != NULL)
+	children = gtk_container_get_children(GTK_CONTAINER(toolbar_new_file_menu));
+	foreach_list(item, children)
 	{
-		children = gtk_container_get_children(GTK_CONTAINER(ui_widgets.new_file_menu));
-		for (item = children; item != NULL; item = g_list_next(item))
-		{
-			gtk_widget_destroy(GTK_WIDGET(item->data));
-		}
+		gtk_widget_destroy(GTK_WIDGET(item->data));
 	}
+	g_object_unref(toolbar_new_file_menu);
 	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
-	for (item = children; item != NULL; item = g_list_next(item))
+	foreach_list(item, children)
 	{
 		gtk_widget_destroy(GTK_WIDGET(item->data));
 	}
