@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 3571 2009-02-10 21:11:25Z eht16 $
+ * $Id: utils.c 4014 2009-07-23 16:01:45Z ntrel $
  */
 
 /*
@@ -63,7 +63,7 @@
  *  Tries to open the given URI in a browser.
  *  On Windows, the system's default browser is opened.
  *  On non-Windows systems, the browser command set in the preferences dialog is used. In case
- *  that fails or it is unset, @a xdg-open is used as fallback as well as some other known
+ *  that fails or it is unset, @c xdg-open is used as fallback as well as some other known
  *  browsers.
  *
  *  @param uri The URI to open in the web browser.
@@ -80,7 +80,7 @@ void utils_open_browser(const gchar *uri)
 
 	g_return_if_fail(uri != NULL);
 
-	cmdline = g_strconcat(tool_prefs.browser_cmd, " ", uri, NULL);
+	cmdline = g_strconcat(tool_prefs.browser_cmd, " \"", uri, "\"", NULL);
 	if (! g_spawn_command_line_async(cmdline, NULL))
 	{
 		const gchar *argv[3];
@@ -130,14 +130,14 @@ gint utils_get_line_endings(const gchar* buffer, glong size)
 
 	cr = lf = crlf = 0;
 
-	for ( i = 0; i < size ; i++ )
+	for (i = 0; i < size ; i++)
 	{
-		if ( buffer[i] == 0x0a )
+		if (buffer[i] == 0x0a)
 		{
 			/* LF */
 			lf++;
 		}
-		else if ( buffer[i] == 0x0d )
+		else if (buffer[i] == 0x0d)
 		{
 			if (i >= (size-1))
 			{
@@ -214,12 +214,9 @@ gboolean utils_is_opening_brace(gchar c, gboolean include_angles)
 
 
 /**
- *  Write the given @c text into a file with @c filename.
+ *  Write the given @a text into a file with @a filename.
  *  If the file doesn't exist, it will be created.
- *  If it already exists, it will be overwritten. Internally, g_file_set_contents() is used
- *  to write the file with all its error checking and related limitations like
- *  destroying hard links and possibly losing file permissions. Please read the
- *  API documentation of g_file_set_contents() for details.
+ *  If it already exists, it will be overwritten.
  *
  *  @param filename The filename of the file to write, in locale encoding.
  *  @param text The text to write into the file.
@@ -229,16 +226,48 @@ gboolean utils_is_opening_brace(gchar c, gboolean include_angles)
  **/
 gint utils_write_file(const gchar *filename, const gchar *text)
 {
-	GError *error = NULL;
-
 	g_return_val_if_fail(filename != NULL, ENOENT);
 	g_return_val_if_fail(text != NULL, EINVAL);
 
-	if (! g_file_set_contents(filename, text, -1, &error))
+	if (file_prefs.use_safe_file_saving)
 	{
-		geany_debug("%s: could not write to file %s (%s)", G_STRFUNC, filename, error->message);
-		g_error_free(error);
-		return EIO;
+		GError *error = NULL;
+		if (! g_file_set_contents(filename, text, -1, &error))
+		{
+			geany_debug("%s: could not write to file %s (%s)", G_STRFUNC, filename, error->message);
+			g_error_free(error);
+			return EIO;
+		}
+	}
+	else
+	{
+		FILE *fp;
+		gint bytes_written, len;
+
+		if (filename == NULL)
+			return ENOENT;
+
+		len = strlen(text);
+		fp = g_fopen(filename, "w");
+		if (fp != NULL)
+		{
+			bytes_written = fwrite(text, sizeof (gchar), len, fp);
+			fclose(fp);
+
+			if (len != bytes_written)
+			{
+				geany_debug(
+					"utils_write_file(): written only %d bytes, had to write %d bytes to %s",
+					bytes_written, len, filename);
+				return EIO;
+			}
+		}
+		else
+		{
+			geany_debug("utils_write_file(): could not write to file %s (%s)",
+				filename, g_strerror(errno));
+			return errno;
+		}
 	}
 	return 0;
 }
@@ -253,7 +282,7 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 {
 	const gchar *begin, *cur;
 
-	if (size < 3)
+	if (G_UNLIKELY(size < 3))
 	{	/* Smallest tag is "<p>" which is 3 characters */
 		return NULL;
 	}
@@ -301,7 +330,7 @@ const gchar *utils_get_eol_name(gint eol_mode)
 
 gboolean utils_atob(const gchar *str)
 {
-	if (str == NULL)
+	if (G_UNLIKELY(str == NULL))
 		return FALSE;
 	else if (strcmp(str, "TRUE") == 0 || strcmp(str, "true") == 0)
 		return TRUE;
@@ -312,13 +341,26 @@ gboolean utils_atob(const gchar *str)
 /* NULL-safe version of g_path_is_absolute(). */
 gboolean utils_is_absolute_path(const gchar *path)
 {
-	if (! path || *path == '\0')
+	if (! NZV(path))
 		return FALSE;
 
 	return g_path_is_absolute(path);
 }
 
 
+/* Skips root if path is absolute, do nothing otherwise.
+ * This is a relative-safe version of g_path_skip_root().
+ */
+const gchar *utils_path_skip_root(const gchar *path)
+{
+	const gchar *path_relative;
+
+	path_relative = g_path_skip_root(path);
+
+	return (path_relative != NULL) ? path_relative : path;
+}
+
+
 gdouble utils_scale_round(gdouble val, gdouble factor)
 {
 	/*val = floor(val * factor + 0.5);*/
@@ -339,8 +381,8 @@ gdouble utils_scale_round(gdouble val, gdouble factor)
  *
  *  The input strings should be in UTF-8 or locale encoding.
  *
- *  @param s1 Pointer to first string or @a NULL.
- *  @param s2 Pointer to second string or @a NULL.
+ *  @param s1 Pointer to first string or @c NULL.
+ *  @param s2 Pointer to second string or @c NULL.
  *
  *  @return an integer less than, equal to, or greater than zero if @a s1 is found, respectively,
  *          to be less than, to match, or to be greater than @a s2.
@@ -389,13 +431,67 @@ gint utils_str_casecmp(const gchar *s1, const gchar *s2)
 
 
 /**
- *  @a NULL-safe string comparison. Returns @a TRUE if both @c a and @c b are @a NULL
- *  or if @c a and @c b refer to valid strings which are equal.
+ *  Truncates the input string to a given length.
+ *  Characters are removed from the middle of the string, so the start and the end of string
+ *  won't change.
  *
- *  @param a Pointer to first string or @a NULL.
- *  @param b Pointer to second string or @a NULL.
+ *  @param string Input string.
+ *  @param truncate_length The length in characters of the resulting string.
  *
- *  @return @a TRUE if @c a equals @c b, else @a FALSE.
+ *  @return A copy of @a string which is truncated to @a truncate_length characters,
+ *          should be freed when no longer needed.
+ *
+ *  @since 0.17
+ */
+/* This following function is taken from Gedit. */
+gchar *utils_str_middle_truncate(const gchar *string, guint truncate_length)
+{
+	GString *truncated;
+	guint length;
+	guint n_chars;
+	guint num_left_chars;
+	guint right_offset;
+	guint delimiter_length;
+	const gchar *delimiter = "\342\200\246";
+
+	g_return_val_if_fail(string != NULL, NULL);
+
+	length = strlen(string);
+
+	g_return_val_if_fail(g_utf8_validate(string, length, NULL), NULL);
+
+	/* It doesnt make sense to truncate strings to less than the size of the delimiter plus 2
+	 * characters (one on each side) */
+	delimiter_length = g_utf8_strlen(delimiter, -1);
+	if (truncate_length < (delimiter_length + 2))
+		return g_strdup(string);
+
+	n_chars = g_utf8_strlen(string, length);
+
+	/* Make sure the string is not already small enough. */
+	if (n_chars <= truncate_length)
+		return g_strdup (string);
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimiter_length) / 2;
+	right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
+
+	truncated = g_string_new_len(string, g_utf8_offset_to_pointer(string, num_left_chars) - string);
+	g_string_append(truncated, delimiter);
+	g_string_append(truncated, g_utf8_offset_to_pointer(string, right_offset));
+
+	return g_string_free(truncated, FALSE);
+}
+
+
+/**
+ *  @c NULL-safe string comparison. Returns @c TRUE if both @a a and @a b are @c NULL
+ *  or if @a a and @a b refer to valid strings which are equal.
+ *
+ *  @param a Pointer to first string or @c NULL.
+ *  @param b Pointer to second string or @c NULL.
+ *
+ *  @return @c TRUE if @a a equals @a b, else @c FALSE.
  **/
 gboolean utils_str_equal(const gchar *a, const gchar *b)
 {
@@ -412,7 +508,7 @@ gboolean utils_str_equal(const gchar *a, const gchar *b)
 
 
 /**
- *  Remove the extension from @c filename and return the result in a newly allocated string.
+ *  Remove the extension from @a filename and return the result in a newly allocated string.
  *
  *  @param filename The filename to operate on.
  *
@@ -420,13 +516,15 @@ gboolean utils_str_equal(const gchar *a, const gchar *b)
  **/
 gchar *utils_remove_ext_from_filename(const gchar *filename)
 {
-	gchar *last_dot = strrchr(filename, '.');
+	gchar *last_dot;
 	gchar *result;
 	gint i;
 
-	if (filename == NULL) return NULL;
+	g_return_val_if_fail(filename != NULL, NULL);
 
-	if (! last_dot) return g_strdup(filename);
+	last_dot = strrchr(filename, '.');
+	if (! last_dot)
+		return g_strdup(filename);
 
 	/* assumes extension is small, so extra bytes don't matter */
 	result = g_malloc(strlen(filename));
@@ -503,24 +601,23 @@ gint utils_is_file_writeable(const gchar *locale_filename)
 
 
 /* Replaces all occurrences of needle in haystack with replacement.
- * Warning: haystack will be freed.
- * New code should use utils_string_replace_all() instead (freeing arguments
- * is unusual behaviour).
+ * Warning: *haystack must be a heap address; it may be freed and reassigned.
+ * Note: utils_string_replace_all() will always be faster when @a replacement is longer
+ * than @a needle.
  * All strings have to be NULL-terminated.
  * See utils_string_replace_all() for details. */
-gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *replacement)
+void utils_str_replace_all(gchar **haystack, const gchar *needle, const gchar *replacement)
 {
 	GString *str;
 
-	if (haystack == NULL)
-		return NULL;
+	g_return_if_fail(*haystack != NULL);
 
-	str = g_string_new(haystack);
+	str = g_string_new(*haystack);
 
-	g_free(haystack);
+	g_free(*haystack);
 	utils_string_replace_all(str, needle, replacement);
 
-	return g_string_free(str, FALSE);
+	*haystack = g_string_free(str, FALSE);
 }
 
 
@@ -538,14 +635,16 @@ gint utils_strpos(const gchar *haystack, const gchar *needle)
 	{
 		for (i = 0; (i < haystack_length) && pos == -1; i++)
 		{
-			if (haystack[i] == needle[0] && needle_length == 1)	return i;
+			if (haystack[i] == needle[0] && needle_length == 1)
+				return i;
 			else if (haystack[i] == needle[0])
 			{
 				for (j = 1; (j < needle_length); j++)
 				{
 					if (haystack[i+j] == needle[j])
 					{
-						if (pos == -1) pos = i;
+						if (pos == -1)
+							pos = i;
 					}
 					else
 					{
@@ -635,10 +734,10 @@ gchar *utils_get_initials(const gchar *name)
  *  @param config A GKeyFile object.
  *  @param section The group name to look in for the key.
  *  @param key The key to find.
- *  @param default_value The default value which will be returned when @c section or @c key
+ *  @param default_value The default value which will be returned when @a section or @a key
  *         don't exist.
  *
- *  @return The value associated with c key as an integer, or the given default value if the value
+ *  @return The value associated with @a key as an integer, or the given default value if the value
  *          could not be retrieved.
  **/
 gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gchar *key,
@@ -647,10 +746,11 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
 	gint tmp;
 	GError *error = NULL;
 
-	if (config == NULL) return default_value;
+	if (G_UNLIKELY(config == NULL))
+		return default_value;
 
 	tmp = g_key_file_get_integer(config, section, key, &error);
-	if (error)
+	if (G_UNLIKELY(error))
 	{
 		g_error_free(error);
 		return default_value;
@@ -668,7 +768,7 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
  *  @param default_value The default value which will be returned when @c section or @c key
  *         don't exist.
  *
- *  @return The value associated with c key as a boolean, or the given default value if the value
+ *  @return The value associated with @a key as a boolean, or the given default value if the value
  *          could not be retrieved.
  **/
 gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const gchar *key,
@@ -677,10 +777,11 @@ gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const
 	gboolean tmp;
 	GError *error = NULL;
 
-	if (config == NULL) return default_value;
+	if (G_UNLIKELY(config == NULL))
+		return default_value;
 
 	tmp = g_key_file_get_boolean(config, section, key, &error);
-	if (error)
+	if (G_UNLIKELY(error))
 	{
 		g_error_free(error);
 		return default_value;
@@ -695,7 +796,7 @@ gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const
  *  @param config A GKeyFile object.
  *  @param section The group name to look in for the key.
  *  @param key The key to find.
- *  @param default_value The default value which will be returned when @c section or @c key
+ *  @param default_value The default value which will be returned when @a section or @a key
  *         don't exist.
  *
  *  @return A newly allocated string, either the value for @a key or a copy of the given
@@ -707,10 +808,11 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 	gchar *tmp;
 	GError *error = NULL;
 
-	if (config == NULL) return g_strdup(default_value);
+	if (G_UNLIKELY(config == NULL))
+		return g_strdup(default_value);
 
 	tmp = g_key_file_get_string(config, section, key, &error);
-	if (error)
+	if (G_UNLIKELY(error))
 	{
 		g_error_free(error);
 		return g_strdup(default_value);
@@ -723,7 +825,7 @@ gchar *utils_get_hex_from_color(GdkColor *color)
 {
 	gchar *buffer = g_malloc0(9);
 
-	if (color == NULL) return NULL;
+	g_return_val_if_fail(color != NULL, NULL);
 
 	g_snprintf(buffer, 8, "#%02X%02X%02X",
 	      (guint) (utils_scale_round(color->red / 256, 255)),
@@ -734,6 +836,18 @@ gchar *utils_get_hex_from_color(GdkColor *color)
 }
 
 
+guint utils_invert_color(guint color)
+{
+	guint r, g, b;
+
+	r = 0xffffff - color;
+	g = 0xffffff - (color >> 8);
+	b = 0xffffff - (color >> 16);
+
+	return (r | (g << 8) | (b << 16));
+}
+
+
 /* Get directory from current file in the notebook.
  * Returns dir string that should be freed or NULL, depending on whether current file is valid.
  * Returned string is in UTF-8 encoding */
@@ -760,7 +874,8 @@ gchar *utils_get_current_file_dir_utf8(void)
 /* very simple convenience function */
 void utils_beep(void)
 {
-	if (prefs.beep_on_errors) gdk_beep();
+	if (prefs.beep_on_errors)
+		gdk_beep();
 }
 
 
@@ -783,7 +898,8 @@ gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 	frac = 0;
 
 	val = size * block_size;
-	if (val == 0) return g_strdup(u);
+	if (val == 0)
+		return g_strdup(u);
 
 	if (display_unit)
 	{
@@ -793,12 +909,12 @@ gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 	else
 	{
 		++u;
-		while ((val >= KILOBYTE) && (u < zero_and_units + sizeof(zero_and_units) - 1))
+		while ((val >= 1024) && (u < zero_and_units + sizeof(zero_and_units) - 1))
 		{
 			f = fmt_tenths;
 			++u;
-			frac = ((((gint)(val % KILOBYTE)) * 10) + (KILOBYTE/2)) / KILOBYTE;
-			val /= KILOBYTE;
+			frac = ((((gint)(val % 1024)) * 10) + (1024/2)) / 1024;
+			val /= 1024;
 		}
 		if (frac >= 10)
 		{		/* We need to round up here. */
@@ -812,7 +928,7 @@ gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 }
 
 
- guint utils_get_value_of_hex(const gchar ch)
+ static guint utils_get_value_of_hex(const gchar ch)
 {
 	if (ch >= '0' && ch <= '9')
 		return ch - '0';
@@ -827,14 +943,15 @@ gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 
 /* utils_strtod() converts a string containing a hex colour ("0x00ff00") into an integer.
  * Basically, it is the same as strtod() would do, but it does not understand hex colour values,
- * before ANSI-C99. With with_route set, it takes strings of the format "#00ff00". */
+ * before ANSI-C99. With with_route set, it takes strings of the format "#00ff00".
+ * Returns -1 on failure. */
 gint utils_strtod(const gchar *source, gchar **end, gboolean with_route)
 {
 	guint red, green, blue, offset = 0;
 
-	if (source == NULL)
-		return -1;
-	else if (with_route && (strlen(source) != 7 || source[0] != '#'))
+	g_return_val_if_fail(source != NULL, -1);
+
+	if (with_route && (strlen(source) != 7 || source[0] != '#'))
 		return -1;
 	else if (! with_route && (strlen(source) != 8 || source[0] != '0' ||
 		(source[1] != 'x' && source[1] != 'X')))
@@ -915,7 +1032,7 @@ gchar **utils_read_file_in_array(const gchar *filename)
 	gchar **result = NULL;
 	gchar *data;
 
-	if (filename == NULL) return NULL;
+	g_return_val_if_fail(filename != NULL, NULL);
 
 	g_file_get_contents(filename, &data, NULL, NULL);
 
@@ -936,6 +1053,8 @@ gboolean utils_str_replace_escape(gchar *string)
 	gsize i, j, len;
 	guint unicodechar;
 
+	g_return_val_if_fail(string != NULL, FALSE);
+
 	j = 0;
 	len = strlen(string);
 	for (i = 0; i < len; i++)
@@ -1068,7 +1187,8 @@ gboolean utils_wrap_string(gchar *string, gint wrapstart)
 	gchar *pos, *linestart;
 	gboolean ret = FALSE;
 
-	if (wrapstart < 0) wrapstart = 80;
+	if (wrapstart < 0)
+		wrapstart = 80;
 
 	for (pos = linestart = string; *pos != '\0'; pos++)
 	{
@@ -1171,8 +1291,7 @@ gchar **utils_strv_new(const gchar *first, ...)
 	gchar *str;
 	gchar **strv;
 
-	if (first == NULL)
-		return NULL;
+	g_return_val_if_fail(first != NULL, NULL);
 
 	strvlen = 1;	/* for first argument */
 
@@ -1228,11 +1347,11 @@ gint utils_mkdir(const gchar *path, gboolean create_parent_dirs)
  *  in the list should be freed after use.
  *
  *  @param path The path of the directory to scan, in locale encoding.
- *  @param length The location to store the number of non-@a NULL data items in the list,
- *                unless @a NULL.
- *  @param error The is the location for storing a possible error, or @a NULL.
+ *  @param length The location to store the number of non-@c NULL data items in the list,
+ *                unless @c NULL.
+ *  @param error The is the location for storing a possible error, or @c NULL.
  *
- *  @return A newly allocated list or @a NULL if no files found. The list and its data should be
+ *  @return A newly allocated list or @c NULL if no files found. The list and its data should be
  *          freed when no longer needed.
  **/
 GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
@@ -1254,7 +1373,8 @@ GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
 	while (1)
 	{
 		const gchar *filename = g_dir_read_name(dir);
-		if (filename == NULL) break;
+		if (filename == NULL)
+			break;
 
 		list = g_slist_insert_sorted(list, g_strdup(filename), (GCompareFunc) utils_str_casecmp);
 		len++;
@@ -1272,7 +1392,7 @@ gboolean utils_str_has_upper(const gchar *str)
 {
 	gunichar c;
 
-	if (str == NULL || *str == '\0' || ! g_utf8_validate(str, -1, NULL))
+	if (! NZV(str) || ! g_utf8_validate(str, -1, NULL))
 		return FALSE;
 
 	while (*str != '\0')
@@ -1295,6 +1415,7 @@ static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
 	guint ret = 0;
 	gssize pos;
 
+	g_return_val_if_fail(haystack != NULL, 0);
 	if (haystack->len == 0)
 		return FALSE;
 	g_return_val_if_fail(NZV(needle), 0);
@@ -1311,7 +1432,7 @@ static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
 		 * (we have to be careful to only use haystack->str as its address may change) */
 		stack = haystack->str + pos;
 
-		if (replace)
+		if (G_LIKELY(replace))
 		{
 			g_string_insert(haystack, pos, replace);
 			stack = haystack->str + pos + strlen(replace);	/* skip past replacement */
@@ -1324,15 +1445,15 @@ static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
 
 
 /**
- * Replaces all occurrences of @c needle in @c haystack with @c replace.
+ * Replaces all occurrences of @a needle in @a haystack with @a replace.
  * As of Geany 0.16, @a replace can match @a needle, so the following will work:
  * @code utils_string_replace_all(text, "\n", "\r\n"); @endcode
  *
  * @param haystack The input string to operate on. This string is modified in place.
  * @param needle The string which should be replaced.
- * @param replace The replacement for @c needle.
+ * @param replace The replacement for @a needle.
  *
- * @return @a amount of replacements done
+ * @return amount of replacements done
  **/
 guint utils_string_replace_all(GString *haystack, const gchar *needle, const gchar *replace)
 {
@@ -1341,15 +1462,15 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
 
 
 /**
- * Convenience function to replace only the first occurrence of @c needle in @c haystack
- * with @ replace.
+ * Convenience function to replace only the first occurrence of @a needle in @a haystack
+ * with @a replace.
  * For details, see utils_string_replace_all().
  *
  * @param haystack The input string to operate on. This string is modified in place.
  * @param needle The string which should be replaced.
- * @param replace The replacement for @c needle.
+ * @param replace The replacement for @a needle.
  *
- * @return @a amount of replacements done
+ * @return amount of replacements done
  *
  *  @since 0.16
  */
@@ -1407,7 +1528,7 @@ static gboolean check_error(GError **error)
  *  @param exit_status The child exit status, as returned by waitpid().
  *  @param error The return location for error or @a NULL.
  *
- *  @return @a TRUE on success, @a FALSE if an error was set.
+ *  @return @c TRUE on success, @c FALSE if an error was set.
  **/
 gboolean utils_spawn_sync(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
 						  GSpawnChildSetupFunc child_setup, gpointer user_data, gchar **std_out,
@@ -1453,7 +1574,7 @@ gboolean utils_spawn_sync(const gchar *dir, gchar **argv, gchar **env, GSpawnFla
  *  @param child_pid The return location for child process ID, or NULL.
  *  @param error The return location for error or @a NULL.
  *
- *  @return @a TRUE on success, @a FALSE if an error was set.
+ *  @return @c TRUE on success, @c FALSE if an error was set.
  **/
 gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
 						   GSpawnChildSetupFunc child_setup, gpointer user_data, GPid *child_pid,
@@ -1481,14 +1602,16 @@ gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFl
 
 /* Similar to g_build_path() but (re)using a fixed buffer, so never free it.
  * This assumes a small enough resulting string length to be kept without freeing,
- * but this should be the case for filenames. */
+ * but this should be the case for filenames.
+ * @warning As the buffer is reused, you can't call this recursively, e.g. for a
+ * function argument and within the function called. */
 const gchar *utils_build_path(const gchar *first, ...)
 {
 	static GString *buffer = NULL;
 	const gchar *str;
 	va_list args;
 
-	if (!buffer)
+	if (! buffer)
 		buffer = g_string_new(first);
 	else
 		g_string_assign(buffer, first);
@@ -1571,7 +1694,7 @@ gboolean utils_is_remote_path(const gchar *path)
 		static gchar *fuse_path = NULL;
 		static gsize len = 0;
 
-		if (fuse_path == NULL)
+		if (G_UNLIKELY(fuse_path == NULL))
 		{
 			fuse_path = g_build_filename(g_get_home_dir(), ".gvfs", NULL);
 			len = strlen(fuse_path);
@@ -1587,3 +1710,83 @@ gboolean utils_is_remote_path(const gchar *path)
 }
 
 
+/* Remove all relative and untidy elements from the path of @a filename.
+ * @param filename must be a valid absolute path.
+ * @see tm_get_real_path() - also resolves links. */
+void utils_tidy_path(gchar *filename)
+{
+	GString *str = g_string_new(filename);
+	const gchar *c, *needle;
+	gchar *tmp;
+	gssize pos;
+
+	g_return_if_fail(g_path_is_absolute(filename));
+
+	/* replace "/./" and "//" */
+	utils_string_replace_all(str, G_DIR_SEPARATOR_S "." G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
+	utils_string_replace_all(str, G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
+
+	/* replace "/.." */
+	needle = G_DIR_SEPARATOR_S "..";
+	while (1)
+	{
+		c = strstr(str->str, needle);
+		if (c == NULL)
+			break;
+		else
+		{
+			pos = c - str->str;
+			if (pos <= 3)
+				break;	/* bad path */
+
+			g_string_erase(str, pos, strlen(needle));	/* erase "/.." */
+
+			tmp = g_strndup(str->str, pos);	/* path up to "/.." */
+			c = g_strrstr(tmp, G_DIR_SEPARATOR_S);
+			g_return_if_fail(c);
+
+			pos = c - tmp;	/* position of previous "/" */
+			g_string_erase(str, pos, strlen(c));
+			g_free(tmp);
+		}
+	}
+	g_return_if_fail(strlen(str->str) <= strlen(filename));
+	strcpy(filename, str->str);
+	g_string_free(str, TRUE);
+}
+
+
+/* @warning Doesn't include null terminating character. */
+#define foreach_str(char_ptr, string) \
+	for (char_ptr = string; *char_ptr; char_ptr++)
+
+/**
+ *  Replace or remove characters from a string in place.
+ *
+ *  @param string String to search.
+ *  @param chars Characters to remove.
+ *
+ *  @return @a string - return value is only useful when nesting function calls, e.g.:
+ *  @code str = utils_str_remove_chars(g_strdup("f_o_o"), "_"); @endcode
+ *
+ *  @see @c g_strdelimit.
+ **/
+gchar *utils_str_remove_chars(gchar *string, const gchar *chars)
+{
+	const gchar *r;
+	gchar *w = string;
+
+	g_return_val_if_fail(string, NULL);
+	if (!NZV(chars))
+		return string;
+
+	foreach_str(r, string)
+	{
+		if (!strchr(chars, *r))
+			*w++ = *r;
+	}
+	*w = 0x0;
+	return string;
+}
+
+
