@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 3446 2009-01-04 18:30:42Z eht16 $
+ *  $Id: win32.c 4098 2009-08-16 17:27:01Z eht16 $
  */
 
 /*
@@ -80,11 +80,11 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **error);
 
 
-static gchar *get_file_filters()
+static wchar_t *get_file_filters(void)
 {
 	gchar *string;
-	gint i, j, len;
-
+	guint i, j, len;
+	static wchar_t title[1024];
 	GString *str = g_string_sized_new(100);
 	GString *all_patterns = g_string_sized_new(100);
 	gchar *tmp;
@@ -117,14 +117,18 @@ static gchar *get_file_filters()
 	{
 		if (string[i] == '\t') string[i] = '\0';
 	}
-	return string;
+	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
+	g_free(string);
+
+	return title;
 }
 
 
-static gchar *get_filters(gboolean project_files)
+static wchar_t *get_filters(gboolean project_files)
 {
 	gchar *string;
 	gint i, len;
+	static wchar_t title[1024];
 
 	if (project_files)
 	{
@@ -145,29 +149,31 @@ static gchar *get_filters(gboolean project_files)
 	{
 		if (string[i] == '\t') string[i] = '\0';
 	}
-	return string;
+	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
+	g_free(string);
+
+	return title;
 }
 
 
-/* Converts the given UTF-8 filename into something usable for Windows and
+/* Converts the given UTF-8 filename or directory name into something usable for Windows and
  * returns the directory part of the given filename. */
-static gchar *get_dir(const gchar *utf8_filename)
+static wchar_t *get_dir_for_path(const gchar *utf8_filename)
 {
+	static wchar_t w_dir[MAX_PATH];
 	gchar *result;
-	/* don't use utils_get_locale_from_utf8() here because it does only g_strdup() on Windows */
-	gchar *locale_filename = g_locale_from_utf8(utf8_filename, -1, NULL, NULL, NULL);
 
-	/* g_file_test() needs the UTF-8 name, the resulted string is used with the Windows API
-	 * where we need the locale encoding */
-	if (! g_file_test(utf8_filename, G_FILE_TEST_IS_DIR))
-	{
-		result = g_path_get_dirname(locale_filename);
-		g_free(locale_filename);
-	}
+	if (g_file_test(utf8_filename, G_FILE_TEST_IS_DIR))
+		result = (gchar*) utf8_filename;
 	else
-		result = locale_filename;
+		result = g_path_get_dirname(utf8_filename);
 
-	return result;
+	MultiByteToWideChar(CP_UTF8, 0, result, -1, w_dir, sizeof(w_dir));
+
+	if (result != utf8_filename)
+		g_free(result);
+
+	return w_dir;
 }
 
 
@@ -180,16 +186,17 @@ INT CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
 	switch(uMsg)
 	{
 		case BFFM_INITIALIZED:
-			SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM) pData);
+		{
+			SendMessageW(hwnd, BFFM_SETSELECTIONW, TRUE, pData);
 			break;
-
+		}
 		case BFFM_SELCHANGED:
 		{
 			/* set the status window to the currently selected path. */
-			static TCHAR szDir[MAX_PATH];
-			if (SHGetPathFromIDList((LPITEMIDLIST) lp, szDir))
+			static wchar_t szDir[MAX_PATH];
+			if (SHGetPathFromIDListW((LPITEMIDLIST) lp, szDir))
 			{
-				SendMessage(hwnd, BFFM_SETSTATUSTEXT, 0, (LPARAM) szDir);
+				SendMessageW(hwnd, BFFM_SETSTATUSTEXTW, 0, (LPARAM) szDir);
 			}
 			break;
 		}
@@ -204,11 +211,13 @@ INT CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
 gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
 										const gchar *initial_dir)
 {
-	BROWSEINFO bi;
+	BROWSEINFOW bi;
 	LPCITEMIDLIST pidl;
-	gchar *fname = g_malloc(MAX_PATH);
-	gchar *dir = get_dir(initial_dir);
 	gchar *result = NULL;
+	wchar_t fname[MAX_PATH];
+	wchar_t w_title[512];
+
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
 
 	if (parent == NULL)
 		parent = main_widgets.window;
@@ -216,31 +225,23 @@ gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
 	memset(&bi, 0, sizeof bi);
 	bi.hwndOwner = GDK_WINDOW_HWND(parent->window);
 	bi.pidlRoot = NULL;
-	bi.lpszTitle = title;
+	bi.lpszTitle = w_title;
 	bi.lpfn = BrowseCallbackProc;
-	bi.lParam = (LPARAM) dir;
+	bi.lParam = (LPARAM) get_dir_for_path(initial_dir);
 	bi.ulFlags = BIF_DONTGOBELOWDOMAIN | BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
 
-	pidl = SHBrowseForFolder(&bi);
-	g_free(dir);
+	pidl = SHBrowseForFolderW(&bi);
 
 	/* convert the strange Windows folder list item something into an usual path string ;-) */
 	if (pidl != 0)
 	{
-		if (SHGetPathFromIDList(pidl, fname))
+		if (SHGetPathFromIDListW(pidl, fname))
 		{
-			/* Convert the resulting filename into UTF-8 (from whatever encoding it has at
-			 * this moment). Don't use utils_get_utf8_from_locale() here because it does only
-			 * g_strdup() on Windows. */
-			setptr(fname, g_locale_to_utf8(fname, -1, NULL, NULL, NULL));
-			result = fname;
+			result = g_malloc0(MAX_PATH * 2);
+			WideCharToMultiByte(CP_UTF8, 0, fname, -1, result, MAX_PATH * 2, NULL, NULL);
 		}
 		/* SHBrowseForFolder() probably leaks memory here, but how to free the allocated memory? */
 	}
-	else
-	{
-		g_free(fname);
-	}
 	return result;
 }
 
@@ -255,14 +256,16 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 								      const gchar *initial_dir, gboolean allow_new_file,
 								      gboolean project_file_filter)
 {
-	OPENFILENAME of;
+	OPENFILENAMEW of;
 	gint retval;
-	gchar *fname = g_malloc(2048);
-	gchar *filters = get_filters(project_file_filter);
-	gchar *dir = get_dir(initial_dir);
+	wchar_t fname[MAX_PATH];
+	wchar_t w_title[512];
+	gchar *filename;
 
 	fname[0] = '\0';
 
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+
 	if (parent == NULL)
 		parent = main_widgets.window;
 
@@ -274,24 +277,21 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 	of.lStructSize = sizeof of;
 #endif
 	of.hwndOwner = GDK_WINDOW_HWND(parent->window);
-	of.lpstrFilter = filters;
+	of.lpstrFilter = get_filters(project_file_filter);
 
 	of.lpstrCustomFilter = NULL;
 	of.nFilterIndex = 0;
 	of.lpstrFile = fname;
-	of.lpstrInitialDir = dir;
+	of.lpstrInitialDir = get_dir_for_path(initial_dir);
 	of.nMaxFile = 2048;
 	of.lpstrFileTitle = NULL;
-	of.lpstrTitle = title;
-	of.lpstrDefExt = "";
+	of.lpstrTitle = w_title;
+	of.lpstrDefExt = L"";
 	of.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_HIDEREADONLY;
 	if (! allow_new_file)
 		of.Flags |= OFN_FILEMUSTEXIST;
 
-	retval = GetOpenFileName(&of);
-
-	g_free(dir);
-	g_free(filters);
+	retval = GetOpenFileNameW(&of);
 
 	if (! retval)
 	{
@@ -302,12 +302,13 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 			g_free(error);
 		}
-		g_free(fname);
 		return NULL;
 	}
 	/* convert the resulting filename into UTF-8 (from whatever encoding it has at this moment) */
-	setptr(fname, g_locale_to_utf8(fname, -1, NULL, NULL, NULL));
-	return fname;
+	filename = g_malloc0(MAX_PATH * 2);
+	WideCharToMultiByte(CP_UTF8, 0, fname, -1, filename, MAX_PATH * 2, NULL, NULL);
+
+	return filename;
 }
 
 
@@ -315,14 +316,18 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
  * Returns: TRUE if the dialog was not cancelled. */
 gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 {
-	OPENFILENAME of;
+	OPENFILENAMEW of;
 	gint retval;
-	gchar *fname = g_malloc(2048);
-	gchar *filters = get_file_filters();
+	gchar tmp[MAX_PATH];
+	wchar_t fname[MAX_PATH];
+	wchar_t w_dir[MAX_PATH];
 
 	fname[0] = '\0';
 
-	/* initialize file dialog info struct */
+	if (initial_dir != NULL)
+		MultiByteToWideChar(CP_UTF8, 0, initial_dir, -1, w_dir, sizeof(w_dir));
+
+	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
 #ifdef OPENFILENAME_SIZE_VERSION_400
 	of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
@@ -330,29 +335,27 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 	of.lStructSize = sizeof of;
 #endif
 	of.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
-	of.lpstrFilter = filters;
+	of.lpstrFilter = get_file_filters();
 
 	of.lpstrCustomFilter = NULL;
 	of.nFilterIndex = GEANY_FILETYPES_NONE + 1;
 	of.lpstrFile = fname;
-	of.lpstrInitialDir = initial_dir;
+	of.lpstrInitialDir = (initial_dir != NULL) ? w_dir : NULL;
 	of.nMaxFile = 2048;
 	of.lpstrFileTitle = NULL;
 	of.lpstrTitle = NULL;
-	of.lpstrDefExt = "";
+	of.lpstrDefExt = L"";
 	if (file_open)
 	{
 		of.Flags = OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST | OFN_EXPLORER;
-		retval = GetOpenFileName(&of);
+		retval = GetOpenFileNameW(&of);
 	}
 	else
 	{
 		of.Flags = OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
-		retval = GetSaveFileName(&of);
+		retval = GetSaveFileNameW(&of);
 	}
 
-	g_free(filters);
-
 	if (!retval)
 	{
 		if (CommDlgExtendedError())
@@ -361,39 +364,39 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 			g_snprintf(error, sizeof error, "File dialog box error (%x)", (int)CommDlgExtendedError());
 			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 		}
-		g_free(fname);
 		return FALSE;
 	}
 
 	if (file_open)
 	{
-		gchar file_name[255];
-		gint x;
+		guint x;
 
 		x = of.nFileOffset - 1;
-		if (x != strlen(fname))
+		if (x != wcslen(fname))
 		{	/* open a single file */
-
-			/* convert the resulting filename into UTF-8 */
-			setptr(fname, g_locale_to_utf8(fname, -1, NULL, NULL, NULL));
-
-			document_open_file(fname, of.Flags & OFN_READONLY, NULL, NULL);
+			WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
+			document_open_file(tmp, of.Flags & OFN_READONLY, NULL, NULL);
 		}
 		else
 		{	/* open multiple files */
+			gchar file_name[MAX_PATH];
+			gchar dir_name[MAX_PATH];
+
+			WideCharToMultiByte(CP_UTF8, 0, fname, of.nFileOffset,
+				dir_name, sizeof(dir_name), NULL, NULL);
 			for (; ;)
 			{
 				if (! fname[x])
 				{
-					gchar *utf8_filename;
-					if (! fname[x+1]) break;
+					if (! fname[x+1])
+						break;
 
-					g_snprintf(file_name, 254, "%s\\%s", fname, fname + x + 1);
+					WideCharToMultiByte(CP_UTF8, 0, fname + x + 1, -1,
+						tmp, sizeof(tmp), NULL, NULL);
+					g_snprintf(file_name, 511, "%s\\%s", dir_name, tmp);
 
 					/* convert the resulting filename into UTF-8 */
-					utf8_filename = g_locale_to_utf8(file_name, -1, NULL, NULL, NULL);
-					document_open_file(utf8_filename, of.Flags & OFN_READONLY, NULL, NULL);
-					g_free(utf8_filename);
+					document_open_file(file_name, of.Flags & OFN_READONLY, NULL, NULL);
 				}
 				x++;
 			}
@@ -402,22 +405,19 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 	else
 	{
 		GeanyDocument *doc = document_get_current();
-		/* convert the resulting filename into UTF-8 */
-		gchar *utf8 = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
 
-		document_save_file_as(doc, utf8);
-		g_free(utf8);
+		WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
+		document_save_file_as(doc, tmp);
 	}
-	g_free(fname);
 	return (retval != 0);
 }
 
 
 void win32_show_font_dialog(void)
 {
-	CHOOSEFONT cf;
 	gint retval;
-	static LOGFONT lf;        /* logical font structure */
+	CHOOSEFONT cf;
+	LOGFONT lf;        /* logical font structure */
 
 	memset(&cf, 0, sizeof cf);
 	cf.lStructSize = sizeof cf;
@@ -427,17 +427,7 @@ void win32_show_font_dialog(void)
 
 	retval = ChooseFont(&cf);
 
-	if (!retval)
-	{
-		if (CommDlgExtendedError())
-		{
-			/*gchar error[100];*/
-			/*snprintf(error, 255, _("Font dialog box error (%x)"), (int)CommDlgExtendedError());*/
-			/*MessageBox(NULL, "Font not availab", _("Error"), MB_OK | MB_ICONERROR);*/
-		}
-		return;
-	}
-	else
+	if (retval)
 	{
 		gchar *editorfont = g_strdup_printf("%s %d", lf.lfFaceName, (cf.iPointSize / 10));
 		ui_set_editor_font(editorfont);
@@ -478,21 +468,24 @@ void win32_show_color_dialog(const gchar *colour)
 
 void win32_show_pref_file_dialog(GtkEntry *item)
 {
-	OPENFILENAME of;
-	gint retval;
-	gchar *fname = g_malloc(512);
-	gchar **field, *filename, *tmp;
-	gchar *filters = get_filters(FALSE);
+	OPENFILENAMEW of;
+	gint retval, len;
+	wchar_t fname[MAX_PATH];
+	gchar tmp[MAX_PATH];
+	gchar **field, *filename;
 
 	fname[0] = '\0';
 
 	/* cut the options from the command line */
 	field = g_strsplit(gtk_entry_get_text(GTK_ENTRY(item)), " ", 2);
-	if (field[0] && g_file_test(field[0], G_FILE_TEST_EXISTS))
+	if (field[0])
 	{
 		filename = g_find_program_in_path(field[0]);
-		strcpy(fname, filename);
-		g_free(filename);
+		if (filename != NULL && g_file_test(filename, G_FILE_TEST_EXISTS))
+		{
+			MultiByteToWideChar(CP_UTF8, 0, filename, -1, fname, sizeof(fname));
+			g_free(filename);
+		}
 	}
 
 	/* initialize file dialog info struct */
@@ -504,21 +497,19 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 #endif
 	of.hwndOwner = GDK_WINDOW_HWND(ui_widgets.prefs_dialog->window);
 
-	of.lpstrFilter = filters;
+	of.lpstrFilter = get_filters(FALSE);
 	of.lpstrCustomFilter = NULL;
 	of.nFilterIndex = 1;
 
 	of.lpstrFile = fname;
-	of.nMaxFile = 512;
+	of.nMaxFile = 2048;
 	of.lpstrFileTitle = NULL;
 	/*of.lpstrInitialDir = g_get_home_dir();*/
 	of.lpstrInitialDir = NULL;
 	of.lpstrTitle = NULL;
-	of.lpstrDefExt = "exe";
+	of.lpstrDefExt = L"exe";
 	of.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_EXPLORER;
-	retval = GetOpenFileName(&of);
-
-	g_free(filters);
+	retval = GetOpenFileNameW(&of);
 
 	if (!retval)
 	{
@@ -529,28 +520,24 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 		}
 		g_strfreev(field);
-		g_free(fname);
 		return;
 	}
 
-	if ((of.nFileOffset - 1) != strlen(fname))
+	len = WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
+	if ((of.nFileOffset - 1) != len)
 	{
-		tmp = g_strdup(fname);
 		if (g_strv_length(field) > 1)
 			/* add the command line args of the old command */
 			/** TODO this fails badly when the old command contained spaces, we need quoting here */
 			filename = g_strconcat(tmp, " ", field[1], NULL);
 		else
 		{
-			filename = tmp;
-			tmp = NULL;
+			filename = g_strdup(tmp);
 		}
 		gtk_entry_set_text(GTK_ENTRY(item), filename);
 		g_free(filename);
-		g_free(tmp);
 	}
 	g_strfreev(field);
-	g_free(fname);
 }
 
 
@@ -618,7 +605,7 @@ gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gcha
 /* Little wrapper for _waccess(), returns errno or 0 if there was no error */
 gint win32_check_write_permission(const gchar *dir)
 {
-	static wchar_t w_dir[512];
+	static wchar_t w_dir[MAX_PATH];
 	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, sizeof w_dir);
 	if (_waccess(w_dir, R_OK | W_OK) != 0)
 		return errno;
@@ -656,6 +643,12 @@ gint win32_message_dialog_unsaved(const gchar *msg)
 /* Just a simple wrapper function to open a browser window */
 void win32_open_browser(const gchar *uri)
 {
+	if (strncmp(uri, "file://", 7) == 0)
+	{
+		uri += 7;
+		while (*uri == '/')
+			uri++;
+	}
 	ShellExecute(NULL, "open", uri, NULL, NULL, SW_SHOWNORMAL);
 }
 
@@ -703,7 +696,7 @@ static void debug_setup_console()
 }
 
 
-void win32_init_debug_code()
+void win32_init_debug_code(void)
 {
 	if (app->debug_mode)
 	{
@@ -716,7 +709,7 @@ void win32_init_debug_code()
 }
 
 
-gchar *win32_get_hostname()
+gchar *win32_get_hostname(void)
 {
 	gchar hostname[100];
 	DWORD size = sizeof(hostname);
@@ -762,9 +755,11 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 
 	if (flags & G_SPAWN_SEARCH_PATH)
 	{
-		retval = SearchPath(NULL,
-					argv[0], ".exe", MAX_PATH, buffer, lpPart);
-		g_snprintf(cmdline, MAX_PATH, "\"%s\"", buffer);
+		retval = SearchPath(NULL, argv[0], ".exe", MAX_PATH, buffer, lpPart);
+		if (retval > 0)
+			g_snprintf(cmdline, MAX_PATH, "\"%s\"", buffer);
+		else
+			g_strlcpy(cmdline, argv[0], sizeof(cmdline));
 		cmdpos = 1;
 	}
 
@@ -782,7 +777,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
 			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, msg);
+				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -796,7 +791,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
 			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, msg);
+				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -818,7 +813,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdout pipe creation failed (%d)", (gint) GetLastError());
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -827,12 +822,12 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	SetHandleInformation(gw_spawn.hChildStdoutRd, HANDLE_FLAG_INHERIT, 0);
 
 	/* Create a pipe for the child process's STDERR. */
-	if (!CreatePipe(&(gw_spawn.hChildStderrRd), &(gw_spawn.hChildStderrWr), &saAttr, 0))
+	if (! CreatePipe(&(gw_spawn.hChildStderrRd), &(gw_spawn.hChildStderrWr), &saAttr, 0))
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stderr pipe creation failed");
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -841,12 +836,12 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	SetHandleInformation(gw_spawn.hChildStderrRd, HANDLE_FLAG_INHERIT, 0);
 
 	/* Create a pipe for the child process's STDIN.  */
-	if (!CreatePipe(&(gw_spawn.hChildStdinRd), &(gw_spawn.hChildStdinWr), &saAttr, 0))
+	if (! CreatePipe(&(gw_spawn.hChildStdinRd), &(gw_spawn.hChildStdinWr), &saAttr, 0))
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdin pipe creation failed");
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -861,7 +856,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		*exit_status = gw_spawn.dwExitCode;
 	}
 
-	if (!fSuccess)
+	if (! fSuccess)
 	{
 		geany_debug("win32_spawn: Create process failed");
 		if (error != NULL)
@@ -873,7 +868,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	if (std_out != NULL)
 	{
 		ReadFromPipe(gw_spawn.hChildStdoutRd, gw_spawn.hChildStdoutWr, hStdoutTempFile, error);
-		if (!GetContentFromHandle(hStdoutTempFile, &stdout_content, error))
+		if (! GetContentFromHandle(hStdoutTempFile, &stdout_content, error))
 		{
 			return FALSE;
 		}
@@ -883,7 +878,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	if (std_err != NULL)
 	{
 		ReadFromPipe(gw_spawn.hChildStderrRd, gw_spawn.hChildStderrWr, hStderrTempFile, error);
-		if (!GetContentFromHandle(hStderrTempFile, &stderr_content, error))
+		if (! GetContentFromHandle(hStderrTempFile, &stderr_content, error))
 		{
 			return FALSE;
 		}
@@ -899,57 +894,69 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	gchar * buffer;
 	DWORD dwRead;
 
-	filesize = GetFileSize(hFile,  NULL);
+	filesize = GetFileSize(hFile, NULL);
 	if (filesize < 1)
 	{
 		*content = NULL;
 		return TRUE;
 	}
 
-	buffer = g_malloc(sizeof(gchar*) * (filesize+1));
-	if (!buffer)
+	buffer = g_malloc(filesize + 1);
+	if (! buffer)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Alloc failed");
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
 
-	SetFilePointer(hFile,0, NULL, FILE_BEGIN);
-	if (!ReadFile(hFile, buffer, filesize, &dwRead,
-				NULL) || dwRead == 0)
+	SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
+	if (! ReadFile(hFile, buffer, filesize, &dwRead, NULL) || dwRead == 0)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Cannot read tempfile");
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
 
-	if (!CloseHandle (hFile))
+	if (! CloseHandle(hFile))
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: CloseHandle failed (%d)", (gint) GetLastError());
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		g_free(buffer);
 		*content = NULL;
 		return FALSE;
 	}
+	buffer[filesize] = '\0';
 	*content = buffer;
 	return TRUE;
 }
 
 
+gchar *win32_expand_environment_variables(const gchar *str)
+{
+	gchar expCmdline[32768]; /* 32768 is the limit for ExpandEnvironmentStrings() */
+
+	if (ExpandEnvironmentStrings((LPCTSTR) str, (LPTSTR) expCmdline, sizeof(expCmdline)) != 0)
+		return g_strdup(expCmdline);
+	else
+		return g_strdup(str);
+}
+
+
 static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error)
 {
 	PROCESS_INFORMATION piProcInfo;
 	STARTUPINFO siStartInfo;
 	BOOL bFuncRetn = FALSE;
+	gchar *expandedCmdline;
 
 	/* Set up members of the PROCESS_INFORMATION structure. */
 	ZeroMemory(&piProcInfo, sizeof(PROCESS_INFORMATION) );
@@ -963,9 +970,12 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 	siStartInfo.hStdInput  = gw_spawn->hChildStdinRd;
 	siStartInfo.dwFlags   |= STARTF_USESTDHANDLES;
 
+	/* Expand environment variables like %blah%. */
+	expandedCmdline = win32_expand_environment_variables(szCmdline);
+
 	/* Create the child process. */
 	bFuncRetn = CreateProcess(NULL,
-		szCmdline,     /* command line */
+		expandedCmdline,             /* command line */
 		NULL,          /* process security attributes */
 		NULL,          /* primary thread security attributes */
 		TRUE,          /* handles are inherited */
@@ -975,19 +985,21 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 		&siStartInfo,  /* STARTUPINFO pointer */
 		&piProcInfo);  /* receives PROCESS_INFORMATION */
 
+	g_free(expandedCmdline);
+
 	if (bFuncRetn == 0)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("CreateChildProcess: CreateProcess failed (%s)", msg);
 		if (*error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
 	else
 	{
-		int i;
-		DWORD               dwStatus;
+		gint i;
+		DWORD dwStatus;
 
 		for (i = 0; i < 2 && (dwStatus = WaitForSingleObject(piProcInfo.hProcess, 30*1000)) == WAIT_TIMEOUT; i++)
 		{
@@ -1000,7 +1012,7 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("GetExitCodeProcess failed: %s", msg);
 			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, msg);
+				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 			g_free(msg);
 		}
 		CloseHandle(piProcInfo.hProcess);
@@ -1018,12 +1030,12 @@ static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **err
 
 	/* Close the write end of the pipe before reading from the
 	   read end of the pipe. */
-	if (!CloseHandle(hWrite))
+	if (! CloseHandle(hWrite))
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("ReadFromPipe: Closing handle failed");
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return;
 	}
@@ -1031,10 +1043,10 @@ static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **err
 	/* Read output from the child process, and write to parent's STDOUT. */
 	for (;;)
 	{
-		if( !ReadFile(hRead, chBuf, BUFSIZE, &dwRead,
-				NULL) || dwRead == 0) break;
+		if (! ReadFile(hRead, chBuf, BUFSIZE, &dwRead, NULL) || dwRead == 0)
+			break;
 
-		if (!WriteFile(hFile, chBuf, dwRead, &dwWritten, NULL))
+		if (! WriteFile(hFile, chBuf, dwRead, &dwWritten, NULL))
 			break;
 	}
 }
@@ -1059,7 +1071,7 @@ static HANDLE GetTempFileHandle(GError **error)
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: GetTempPath failed (%d)", (gint) GetLastError());
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1074,7 +1086,7 @@ static HANDLE GetTempFileHandle(GError **error)
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileName failed (%d)", (gint) GetLastError());
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1092,7 +1104,7 @@ static HANDLE GetTempFileHandle(GError **error)
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: Second CreateFile failed (%d)", (gint) GetLastError());
 		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, msg);
+			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1183,5 +1195,18 @@ gchar *win32_get_shortcut_target(const gchar *file_name)
 		return path;
 }
 
+void win32_set_working_directory(const gchar *dir)
+{
+	SetCurrentDirectory(dir);
+}
+
+gchar *win32_get_installation_dir(void)
+{
+#if GLIB_CHECK_VERSION(2, 16, 0)
+	return g_win32_get_package_installation_directory_of_module(NULL);
+#else
+	return g_win32_get_package_installation_directory(NULL, NULL);
+#endif
+}
 
 #endif
