@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.h 3446 2009-01-04 18:30:42Z eht16 $
+ *  $Id: win32.h 4050 2009-08-02 13:33:39Z eht16 $
  */
 
 
@@ -48,9 +48,11 @@ gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
 
 gint win32_check_write_permission(const gchar *dir);
 
-void win32_init_debug_code();
+void win32_init_debug_code(void);
 
-gchar *win32_get_hostname();
+void win32_set_working_directory(const gchar *dir);
+
+gchar *win32_get_hostname(void);
 
 gboolean win32_get_exit_status(GPid child_pid);
 
@@ -59,4 +61,8 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 
 gchar *win32_get_shortcut_target(const gchar *file_name);
 
+gchar *win32_get_installation_dir(void);
+
+gchar *win32_expand_environment_variables(const gchar *str);
+
 #endif
