@@ -1,5 +1,5 @@
 /*
-*   $Id: get.h 1653 2007-06-29 15:42:37Z eht16 $
+*   $Id: get.h 3352 2008-12-09 19:00:45Z eht16 $
 *
 *   Copyright (c) 1998-2002, Darren Hiebert
 *
@@ -29,8 +29,9 @@
 /*  Is the character valid as the first character of a C identifier?
  *  C++ allows '~' in destructors.
  *  VMS allows '$' in identifiers.
+ *  Vala allows '@' in identifiers.
  */
-#define isident1(c)  (isalpha(c) || (c) == '_' || (c) == '~' || (c) == '$')
+#define isident1(c)  (isalpha(c) || (c) == '_' || (c) == '~' || (c) == '$' || (c) == '@')
 
 /*
 *   FUNCTION PROTOTYPES
@@ -44,7 +45,8 @@ extern void cppEndStatement (void);
 extern void cppUngetc (const int c);
 extern int cppGetc (void);
 extern int skipOverCComment (void);
-extern char *getArglistFromPos(fpos_t startPosition, const char *tokenName);
+extern char *getArglistFromFilePos(fpos_t startPosition, const char *tokenName);
+extern char *getArglistFromBufferPos(int startPosition, const char *tokenName);
 extern char *getArglistFromStr(char *buf, const char *name);
 
 #endif  /* _GET_H */
