@@ -48,8 +48,10 @@
     F77Parser, \
     GLSLParser, \
     MatlabParser, \
-    ValaParser
-
+    ValaParser, \
+    ActionScriptParser, \
+    NsisParser, \
+    MarkdownParser
 /*
 langType of each parser
  0	CParser
@@ -86,6 +88,9 @@ langType of each parser
 31  GLSLParser
 32  MatlabParser
 33  ValaParser
+34  ActionScriptParser
+35  NsisParser
+36  MarkdownParser
 */
 #endif	/* _PARSERS_H */
 
