@@ -7,6 +7,18 @@
 *
 */
 
+/*!
+ * @file tm_workspace.h
+ The TMWorkspace structure is meant to be used as a singleton to store application
+ wide tag information.
+
+ The workspace is intended to contain a list of global tags
+ and a set of work objects (projects or individual files). You need not use the
+ workspace, though, to use tag manager, unless you need things like global tags
+ and a place to store all current open projects and individual files. TMWorkspace
+ is derived from TMWorkObject.
+*/
+
 #include "general.h"
 
 #include <stdio.h>
@@ -130,8 +142,10 @@ static TMTagAttrType global_tags_sort_attrs[] =
 
 gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 {
+	guchar buf[BUFSIZ];
 	FILE *fp;
 	TMTag *tag;
+	gboolean format_pipe = FALSE;
 
 	if (NULL == (fp = g_fopen(tags_file, "r")))
 		return FALSE;
@@ -139,12 +153,34 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 		return FALSE;
 	if (NULL == theWorkspace->global_tags)
 		theWorkspace->global_tags = g_ptr_array_new();
-	while (NULL != (tag = tm_tag_new_from_file(NULL, fp, mode)))
+	if ((NULL == fgets((gchar*) buf, BUFSIZ, fp)) || ('\0' == *buf))
+		return FALSE; /* early out on error */
+	else
+	{	/* We read the first line for the format specification. */
+		if (buf[0] == '#' && strstr((gchar*) buf, "format=pipe") != NULL)
+			format_pipe = TRUE;
+		else if (buf[0] == '#' && strstr((gchar*) buf, "format=tagmanager") != NULL)
+			format_pipe = FALSE;
+		else
+		{	/* We didn't find a valid format specification, so we try to auto-detect the format
+			 * by counting the pipe characters on the first line and asumme pipe format when
+			 * we find more than one pipe on the line. */
+			guint i, pipe_cnt = 0;
+			for (i = 0; i < BUFSIZ && buf[i] != '\0' && pipe_cnt < 2; i++)
+			{
+				if (buf[i] == '|')
+					pipe_cnt++;
+			}
+			format_pipe = (pipe_cnt > 1);
+		}
+		rewind(fp); /* reset the file pointer, to start reading again from the beginning */
+	}
+	while (NULL != (tag = tm_tag_new_from_file(NULL, fp, mode, format_pipe)))
 		g_ptr_array_add(theWorkspace->global_tags, tag);
 	fclose(fp);
 
 	/* resort the whole array, because tm_tags_find expects a sorted array and it is not sorted
-	 * when global.tags, php.tags and latex.tags are loaded at the same time */
+	 * when c99.tags, php.tags and latex.tags are loaded at the same time */
 	tm_tags_sort(theWorkspace->global_tags, global_tags_sort_attrs, TRUE);
 
 	return TRUE;
@@ -184,8 +220,9 @@ static void write_includes_file(FILE *fp, GList *includes_files)
 	{
 		char *str = g_strdup_printf("#include \"%s\"\n", (char*)node->data);
 		int str_len = strlen(str);
+		size_t size;
 
-		fwrite(str, str_len, 1, fp);
+		size = fwrite(str, str_len, 1, fp);
 		free(str);
 		node = g_list_next (node);
 	}
@@ -211,30 +248,15 @@ static void append_to_temp_file(FILE *fp, GList *file_list)
 		}
 		else
 		{
-			fwrite(contents, length, 1, fp);
-			fwrite("\n", 1, 1, fp);	/* in case file doesn't end in newline (e.g. windows). */
+			size_t size;
+			size = fwrite(contents, length, 1, fp);
+			size = fwrite("\n", 1, 1, fp);	/* in case file doesn't end in newline (e.g. windows). */
 			g_free(contents);
 		}
 		node = g_list_next (node);
 	}
 }
 
-static gint get_global_tag_type_mask(gint lang)
-{
-	switch (lang)
-	{
-		case 0:
-		case 1:
-			/* C/C++ */
-			return tm_tag_class_t | tm_tag_typedef_t | tm_tag_enum_t | tm_tag_enumerator_t |
-				tm_tag_prototype_t |
-				tm_tag_function_t | tm_tag_method_t |	/* for inline functions */
-				tm_tag_macro_t | tm_tag_macro_with_arg_t;
-		default:
-			return tm_tag_max_t;
-	}
-}
-
 gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre_process,
 	const char **includes, int includes_count, const char *tags_file, int lang)
 {
@@ -346,12 +368,13 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 	 */
 	if (pre_process != NULL)
 	{
+		int ret;
 		command = g_strdup_printf("%s %s | grep -v -E '^\\s*(G_BEGIN_DECLS|G_END_DECLS)\\s*$' > %s",
 							  pre_process, temp_file, temp_file2);
 #ifdef TM_DEBUG
 		g_message("Executing: %s", command);
 #endif
-		system(command);
+		ret = system(command);
 		g_free(command);
 		g_unlink(temp_file);
 		g_free(temp_file);
@@ -376,7 +399,7 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
-	tags_array = tm_tags_extract(source_file->tags_array, get_global_tag_type_mask(lang));
+	tags_array = tm_tags_extract(source_file->tags_array, tm_tag_max_t);
 	if ((NULL == tags_array) || (0 == tags_array->len))
 	{
 		if (tags_array)
@@ -394,6 +417,7 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
+	fprintf(fp, "# format=tagmanager\n");
 	for (i = 0; i < tags_array->len; ++i)
 	{
 		tm_tag_write(TM_TAG(tags_array->pdata[i]), fp, tm_tag_attr_type_t
@@ -493,7 +517,7 @@ gboolean tm_workspace_update(TMWorkObject *workspace, gboolean force
 	}
 	if (update_tags)
 		tm_workspace_recreate_tags_array();
-	workspace->analyze_time = time(NULL);
+	/* workspace->analyze_time = time(NULL); */
 	return update_tags;
 }
 
@@ -712,6 +736,461 @@ tm_get_current_function (GPtrArray * file_tags, const gulong line)
 }
 
 
+static int
+find_scope_members_tags (const GPtrArray * all, GPtrArray * tags,
+						 const langType langJava, const char *name,
+						 const char *filename, gboolean no_definitions)
+{
+	GPtrArray *local = g_ptr_array_new ();
+	unsigned int i;
+	TMTag *tag;
+	size_t len = strlen (name);
+	for (i = 0; (i < all->len); ++i)
+	{
+		tag = TM_TAG (all->pdata[i]);
+		if (no_definitions && filename && tag->atts.entry.file &&
+			0 != strcmp (filename,
+						 tag->atts.entry.file->work_object.short_name))
+		{
+			continue;
+		}
+		if (tag && tag->atts.entry.scope && tag->atts.entry.scope[0] != '\0')
+		{
+			if (0 == strncmp (name, tag->atts.entry.scope, len))
+			{
+				g_ptr_array_add (local, tag);
+			}
+		}
+	}
+	if (local->len > 0)
+	{
+		unsigned int j;
+		TMTag *tag2;
+		char backup = 0;
+		char *s_backup = NULL;
+		char *var_type = NULL;
+		char *scope;
+		for (i = 0; (i < local->len); ++i)
+		{
+			tag = TM_TAG (local->pdata[i]);
+			scope = tag->atts.entry.scope;
+			if (scope && 0 == strcmp (name, scope))
+			{
+				g_ptr_array_add (tags, tag);
+				continue;
+			}
+			s_backup = NULL;
+			j = 0;				/* someone could write better code :P */
+			while (scope)
+			{
+				if (s_backup)
+				{
+					backup = s_backup[0];
+					s_backup[0] = '\0';
+					if (0 == strcmp (name, tag->atts.entry.scope))
+					{
+						j = local->len;
+						s_backup[0] = backup;
+						break;
+					}
+				}
+				if (tag->atts.entry.file
+					&& tag->atts.entry.file->lang == langJava)
+				{
+					scope = strrchr (tag->atts.entry.scope, '.');
+					if (scope)
+						var_type = scope + 1;
+				}
+				else
+				{
+					scope = strrchr (tag->atts.entry.scope, ':');
+					if (scope)
+					{
+						var_type = scope + 1;
+						scope--;
+					}
+				}
+				if (s_backup)
+				{
+					s_backup[0] = backup;
+				}
+				if (scope)
+				{
+					if (s_backup)
+					{
+						backup = s_backup[0];
+						s_backup[0] = '\0';
+					}
+					for (j = 0; (j < local->len); ++j)
+					{
+						if (i == j)
+							continue;
+						tag2 = TM_TAG (local->pdata[j]);
+						if (tag2->atts.entry.var_type &&
+							0 == strcmp (var_type, tag2->atts.entry.var_type))
+						{
+							break;
+						}
+					}
+					if (s_backup)
+						s_backup[0] = backup;
+				}
+				if (j < local->len)
+				{
+					break;
+				}
+				s_backup = scope;
+			}
+			if (j == local->len)
+			{
+				g_ptr_array_add (tags, tag);
+			}
+		}
+	}
+	g_ptr_array_free (local, TRUE);
+	return (int) tags->len;
+}
+
+
+#if 0
+static int
+find_namespace_members_tags (const GPtrArray * all, GPtrArray * tags,
+						 	const langType langJava, const char *name,
+						 	const char *filename)
+{
+	GPtrArray *local = g_ptr_array_new ();
+	unsigned int i;
+	TMTag *tag;
+	size_t len = strlen (name);
+
+	g_return_val_if_fail (all != NULL, 0);
+
+	for (i = 0; (i < all->len); ++i)
+	{
+		tag = TM_TAG (all->pdata[i]);
+		if (filename && tag->atts.entry.file &&
+			0 != strcmp (filename,
+						 tag->atts.entry.file->work_object.short_name))
+		{
+			continue;
+		}
+
+		if (tag && tag->atts.entry.scope && tag->atts.entry.scope[0] != '\0')
+		{
+			if (0 == strncmp (name, tag->atts.entry.scope, len))
+			{
+				g_ptr_array_add (local, tag);
+			}
+		}
+	}
+
+	if (local->len > 0)
+	{
+		char *scope;
+		for (i = 0; (i < local->len); ++i)
+		{
+			tag = TM_TAG (local->pdata[i]);
+			scope = tag->atts.entry.scope;
+
+			/* if we wanna complete something like
+			 * namespace1::
+			 * we'll just return the tags that have "namespace1"
+			 * as their scope. So we won't return classes/members/namespaces
+			 * under, for example, namespace2, where namespace1::namespace2
+			 */
+			if (scope && 0 == strcmp (name, scope))
+			{
+				g_ptr_array_add (tags, tag);
+			}
+		}
+	}
+
+	g_ptr_array_free (local, TRUE);
+	return (int) tags->len;
+}
+
+const GPtrArray *
+tm_workspace_find_namespace_members (const GPtrArray * file_tags, const char *name,
+								 gboolean search_global)
+{
+	static GPtrArray *tags = NULL;
+	GPtrArray *local = NULL;
+	char *new_name = (char *) name;
+	char *filename = NULL;
+	int found = 0, del = 0;
+	static langType langJava = -1;
+	TMTag *tag = NULL;
+
+	g_return_val_if_fail ((theWorkspace && name && name[0] != '\0'), NULL);
+
+	if (!tags)
+		tags = g_ptr_array_new ();
+
+	while (1)
+	{
+		const GPtrArray *tags2;
+		int got = 0, types = (tm_tag_class_t | tm_tag_namespace_t |
+								tm_tag_struct_t | tm_tag_typedef_t |
+								tm_tag_union_t | tm_tag_enum_t);
+
+		if (file_tags)
+		{
+			g_ptr_array_set_size (tags, 0);
+			got = fill_find_tags_array (tags, file_tags,
+										  new_name, NULL, types, FALSE, -1, FALSE);
+		}
+
+
+		if (got)
+		{
+			tags2 = tags;
+		}
+		else
+		{
+			TMTagAttrType attrs[] = {
+				tm_tag_attr_name_t, tm_tag_attr_type_t,
+				tm_tag_attr_none_t
+			};
+			tags2 = tm_workspace_find (new_name, types, attrs, FALSE, -1);
+		}
+
+		if ((tags2) && (tags2->len == 1) && (tag = TM_TAG (tags2->pdata[0])))
+		{
+			if (tag->type == tm_tag_typedef_t && tag->atts.entry.var_type
+				&& tag->atts.entry.var_type[0] != '\0')
+			{
+				new_name = tag->atts.entry.var_type;
+				continue;
+			}
+			filename = (tag->atts.entry.file ?
+						tag->atts.entry.file->work_object.short_name : NULL);
+			if (tag->atts.entry.scope && tag->atts.entry.scope[0] != '\0')
+			{
+				del = 1;
+				if (tag->atts.entry.file &&
+					tag->atts.entry.file->lang == langJava)
+				{
+					new_name = g_strdup_printf ("%s.%s",
+												tag->atts.entry.scope,
+												new_name);
+				}
+				else
+				{
+					new_name = g_strdup_printf ("%s::%s",
+												tag->atts.entry.scope,
+												new_name);
+				}
+			}
+			break;
+		}
+		else
+		{
+			return NULL;
+		}
+	}
+
+	g_ptr_array_set_size (tags, 0);
+
+	if (tag && tag->atts.entry.file)
+	{
+		local = tm_tags_extract (tag->atts.entry.file->work_object.tags_array,
+								 (tm_tag_function_t |
+								  tm_tag_field_t | tm_tag_enumerator_t |
+								  tm_tag_namespace_t | tm_tag_class_t ));
+	}
+	else
+	{
+		local = tm_tags_extract (theWorkspace->work_object.tags_array,
+								 (tm_tag_function_t | tm_tag_prototype_t |
+								  tm_tag_member_t |
+								  tm_tag_field_t | tm_tag_enumerator_t |
+								  tm_tag_namespace_t | tm_tag_class_t ));
+	}
+
+	if (local)
+	{
+		found = find_namespace_members_tags (local, tags,
+										 langJava, new_name, filename);
+		g_ptr_array_free (local, TRUE);
+	}
+
+
+	if (!found && search_global)
+	{
+		GPtrArray *global = tm_tags_extract (theWorkspace->global_tags,
+											 (tm_tag_member_t |
+											  tm_tag_prototype_t |
+											  tm_tag_field_t |
+											  tm_tag_method_t |
+											  tm_tag_function_t |
+											  tm_tag_enumerator_t |
+											  tm_tag_namespace_t |
+											  tm_tag_class_t ));
+
+		if (global)
+		{
+			find_namespace_members_tags (global, tags, langJava,
+									 new_name, filename);
+/*/
+			DEBUG_PRINT ("returning these");
+  		    gint i;
+			for (i=0; i < tags->len; i++) {
+				TMTag *cur_tag;
+
+				cur_tag = (TMTag*)g_ptr_array_index (tags, i);
+				tm_tag_print (cur_tag, stdout );
+			}
+/*/
+			g_ptr_array_free (global, TRUE);
+		}
+	}
+
+
+	if (del)
+	{
+		g_free (new_name);
+	}
+
+	return tags;
+}
+#endif
+
+const GPtrArray *
+tm_workspace_find_scope_members (const GPtrArray * file_tags, const char *name,
+								 gboolean search_global, gboolean no_definitions)
+{
+	static GPtrArray *tags = NULL;
+	GPtrArray *local = NULL;
+	char *new_name = (char *) name;
+	char *filename = NULL;
+	int found = 0, del = 0;
+	static langType langJava = -1;
+	TMTag *tag = NULL;
+
+	/* FIXME */
+	/* langJava = getNamedLanguage ("Java"); */
+
+	g_return_val_if_fail ((theWorkspace && name && name[0] != '\0'), NULL);
+
+	if (!tags)
+		tags = g_ptr_array_new ();
+
+	while (1)
+	{
+		const GPtrArray *tags2;
+		int got = 0, types = (tm_tag_class_t | tm_tag_namespace_t |
+								tm_tag_struct_t | tm_tag_typedef_t |
+								tm_tag_union_t | tm_tag_enum_t);
+
+		if (file_tags)
+		{
+			g_ptr_array_set_size (tags, 0);
+			got = fill_find_tags_array (tags, file_tags,
+										  new_name, NULL, types, FALSE, -1, FALSE);
+		}
+		if (got)
+		{
+			tags2 = tags;
+		}
+		else
+		{
+			TMTagAttrType attrs[] = {
+				tm_tag_attr_name_t, tm_tag_attr_type_t,
+				tm_tag_attr_none_t
+			};
+			tags2 = tm_workspace_find (new_name, types, attrs, FALSE, -1);
+		}
+
+		if ((tags2) && (tags2->len == 1) && (tag = TM_TAG (tags2->pdata[0])))
+		{
+			if (tag->type == tm_tag_typedef_t && tag->atts.entry.var_type
+				&& tag->atts.entry.var_type[0] != '\0')
+			{
+				char *tmp_name;
+				tmp_name = tag->atts.entry.var_type;
+				if (strcmp(tmp_name, new_name) == 0) {
+					new_name = NULL;
+				}
+				else {
+					new_name = tmp_name;
+				}
+				continue;
+			}
+			filename = (tag->atts.entry.file ?
+						tag->atts.entry.file->work_object.short_name : NULL);
+			if (tag->atts.entry.scope && tag->atts.entry.scope[0] != '\0')
+			{
+				del = 1;
+				if (tag->atts.entry.file &&
+					tag->atts.entry.file->lang == langJava)
+				{
+					new_name = g_strdup_printf ("%s.%s",
+												tag->atts.entry.scope,
+												new_name);
+				}
+				else
+				{
+					new_name = g_strdup_printf ("%s::%s",
+												tag->atts.entry.scope,
+												new_name);
+				}
+			}
+			break;
+		}
+		else
+		{
+			return NULL;
+		}
+	}
+
+	g_ptr_array_set_size (tags, 0);
+
+	if (no_definitions && tag && tag->atts.entry.file)
+	{
+		local = tm_tags_extract (tag->atts.entry.file->work_object.tags_array,
+								 (tm_tag_function_t | tm_tag_prototype_t |
+								  tm_tag_member_t | tm_tag_field_t |
+								  tm_tag_method_t | tm_tag_enumerator_t));
+	}
+	else
+	{
+		local = tm_tags_extract (theWorkspace->work_object.tags_array,
+								 (tm_tag_function_t | tm_tag_prototype_t |
+								  tm_tag_member_t | tm_tag_field_t |
+								  tm_tag_method_t | tm_tag_enumerator_t));
+	}
+	if (local)
+	{
+		found = find_scope_members_tags (local, tags, langJava, new_name,
+										 filename, no_definitions);
+		g_ptr_array_free (local, TRUE);
+	}
+	if (!found && search_global)
+	{
+		GPtrArray *global = tm_tags_extract (theWorkspace->global_tags,
+											 (tm_tag_member_t |
+											  tm_tag_prototype_t |
+											  tm_tag_field_t |
+											  tm_tag_method_t |
+											  tm_tag_function_t |
+											  tm_tag_enumerator_t
+											  |tm_tag_struct_t | tm_tag_typedef_t |
+											  tm_tag_union_t | tm_tag_enum_t));
+		if (global)
+		{
+			find_scope_members_tags (global, tags, langJava, new_name,
+									 filename, no_definitions);
+			g_ptr_array_free (global, TRUE);
+		}
+	}
+	if (del)
+	{
+		g_free (new_name);
+	}
+
+	return tags;
+}
+
 const GPtrArray *tm_workspace_get_parents(const gchar *name)
 {
 	static TMTagAttrType type[] = { tm_tag_attr_name_t, tm_tag_attr_none_t };
